(* ========================================================================== *)
(*               Connecting HOL Light and Mathematica                         *)
(*                                                                            *)
(*            (c) Copyright, Ons Seddiki  2013-14                             *)
(*                           Hardware Verification Group,                     *)
(*                           Concordia University                             *)
(*                                                                            *)
(*                Contact:   <o_sed@encs.concordia.ca>                        *)
(*                                                                            *)
(* This file is the principal file. It loads all the modules:ParserSplitter  *)
(*  mathematicaphrasebook and parserandcollector.                             *)                            
(* =========================================================================  *)




#use "parserandsplitter.ml";;
#use "mathematicaphrasebook.ml";;
#use "parserandcollector.ml";;




 let file1 = "tag.txt"
let file2 = "variable.txt"
let file3 = "interval.txt"

let mathematica =  new_specification ["Mathematica"]  (MESON[] `?x. x = T`)


let ecrire_fichier r file= 
let canal_sortie = open_out file in
  output_string canal_sortie r;
close_out canal_sortie

let rec splitchar s c =
    if String.contains s c then
        (String.sub s 0 ((String.index s c))) :: (splitchar (String.sub s ((String.index s c)+ 1) ((String.length s) - (String.index s c) - 1)) c)
    else [s] 

let transformer_res s = 
 let s1= String.sub s 0 6 in 
if (s1 ="vector") then 
let str = String.sub s 7 ((String.length s) - 8) in 
let l = splitchar str ';' in 
String.concat "\\/" l 
else 
s

let call_solve str = 
  let var =   read_line() in
     let r=ecrire_fichier var file2  in 
        let ino= eval str in
         let a= call_mathematicaphrasebook () in let res= transformer_res (call_collector()) in
mk_thm ([`Mathematica`],parse_term ( str ^" ==> "^ res))

let contains s1 s2 =
    let re = Str.regexp_string s2
    in
        try ignore (Str.search_forward re s1 0); true
              with Not_found -> false

let display_eigen_pair s = 
let indexbo= String.index s '(' in 
let indexbc = String.index s ')' in 
let subs = String.sub s indexbo (indexbc - indexbo + 1) in
let ino1= eval ("eigenval ("^ subs ^")")  in
let a1= call_mathematicaphrasebook () in 
let res1 =call_collector()in
let ino2= eval ("eigenvect ("^ subs ^")")  in
let a2= call_mathematicaphrasebook () in 
let res2 =call_collector1() in 
let str = String.sub res1 7 ((String.length  res1) - 8) in 
let l = splitchar str ';' in 
let strr = String.sub res2 7 ((String.length  res2) - 8) in
let indbc =  (String.index strr ']') + 1 in 
let eigenvect1 = String.sub strr 1 ((String.length  strr) - indbc + 1) in
print_string eigenvect1; print_newline();
print_string  (List.hd l); print_newline();

print_string (List.nth l 1); print_newline();


let indpv =  (String.index_from strr indbc ';') + 1 in 
let eigenvect2 = String.sub strr (indpv + 1)  ((String.length  strr) - indpv - 2 ) in
 "eigen_pair " ^ subs^ " " ^ "(" ^ eigenvect1 ^ ", " ^ (List.hd l)^ ")"^ " /\\ " ^"eigen_pair " ^ subs^ " " ^ "("^ eigenvect2 ^ "," ^ (List.nth l 1)^ ")" 


let display_eigen_pair_axiom s = 
let fres = display_eigen_pair s in 
mk_thm ([`Mathematica`],parse_term(fres ))


let display_eigen_pair_subgoal s = 
let fres = display_eigen_pair s in 
 e(ASSERT_TAC (parse_term(fres )))


let contains s1 s2 =
    let re = Str.regexp_string s2
    in
        try ignore (Str.search_forward re s1 0); true
              with Not_found -> false


let call_mathematica str tag = 
(*let str = string_of_term term in*) 
let r=ecrire_fichier (tag^ "\n") file1 in  (match tag with 

|"Simplify" -> let ino= eval str in
		let a= call_mathematicaphrasebook () in let res =call_collector() in
                if  (String.contains str '>') or (String.contains str '<')  then 
					(match res with 
					|"False" -> mk_thm ([`Mathematica`],parse_term("~ ("^str^ ")")) 
					|"True" -> mk_thm ([`Mathematica`],parse_term(str)) 
					|_ ->mk_thm ([`Mathematica`],parse_term(str ^" ==> "^ res)))
		else 
					mk_thm ([`Mathematica`],parse_term(str ^" = "^ res))

|"FullSimplify" -> if (contains str "eigen_pair" ) then 
let s= String.sub str 0 10 in 
display_eigen_pair_axiom str
else 

let ino= eval str in
		let a= call_mathematicaphrasebook () in 
                if  (String.contains str '>') or (String.contains str '<')  then 
				
                let res=call_collector() in	(match res with 
					|"False" -> mk_thm ([`Mathematica`],parse_term("~ ("^str^ ")")) 
					|"True" ->mk_thm ([`Mathematica`],parse_term(str)) 
					|_ ->mk_thm ([`Mathematica`],parse_term(str ^" ==> "^ res)))
		else if ( contains str "and") then 
               let res=call_collector1() in
                      let str = Str.global_replace (Str.regexp "and")  "/\\ " str in (match res with 
					|"False" -> mk_thm ([`Mathematica`],parse_term("~ ("^str^ ")")) 
					|"True" -> mk_thm ([`Mathematica`],parse_term(str)) 			
                           		|_ ->mk_thm ([`Mathematica`],parse_term(str ^" ==> "^ res)))
                 
                else
                       let res=call_collector() in
					mk_thm ([`Mathematica`],parse_term(str ^" = "^ res))

|"Factor" -> let ino= eval str in
		let a= call_mathematicaphrasebook () in let res =call_collector() in
                mk_thm ([`Mathematica`],parse_term ( str ^" = "^ res))
|"Solve" -> begin   print_endline "Enter the variable to find :" ;  call_solve str ; end

|"FindRoot" -> begin   print_endline "Enter the variable to find :" ;
		let var =   read_line() in
 		let r=ecrire_fichier var file2  in 
               print_endline "Enter the starting point :" ;
		let interval =   read_line() in
		let v=ecrire_fichier interval file3  in 
		let ino= eval str in
		let a= call_mathematicaphrasebook () in let res = transformer_res (call_collector()) in 
 		mk_thm ([`Mathematica`],parse_term ( str ^" ==> "^ res))
		 end
| _ -> failwith "Input error")



let call_mathematica_goal str tag  = 
(*if subgoal = "subgoal" then*)
(*let str = string_of_term term in*) 
let r=ecrire_fichier (tag^ "\n") file1 in  (match tag with 

|"Simplify" -> let ino= eval str in
		let a= call_mathematicaphrasebook () in let res =call_collector() in
                if  (String.contains str '>') or (String.contains str '<')  then 
					(match res with 
					|"False" ->  e(ASSERT_TAC (parse_term("~ ("^str^ ")")))
					|"True" ->  e(ASSERT_TAC (parse_term(str)))
					|_ -> e(ASSERT_TAC (parse_term(str ^" ==> "^ res))))
		else 
					 e(ASSERT_TAC (parse_term(str ^" = "^ res)))

|"FullSimplify" ->  if (String.contains str 'e') & (String.contains str 'i') & (String.contains str 'g') & (String.contains str 'n') &  (String.contains str 'p') & (String.contains str 'a') & (String.contains str 'r') then 
let s= String.sub str 0 10 in 
display_eigen_pair_subgoal str
else 

let ino= eval str in
		let a= call_mathematicaphrasebook () in 
                if  (String.contains str '>') or (String.contains str '<')  then 
				
                let res=call_collector() in	(match res with 
					|"False" ->  e(ASSERT_TAC (parse_term("~ ("^str^ ")")))
					|"True" ->  e(ASSERT_TAC (parse_term(str)))
					|_ ->e(ASSERT_TAC (parse_term(str ^" ==> "^ res))))
		else if ( contains str "and") then 
               let res=call_collector1() in
                      let str = Str.global_replace (Str.regexp "and")  "/\\ " str in (match res with 
					|"False" ->  e(ASSERT_TAC (parse_term("~ ("^str^ ")")))
					|"True" ->  e(ASSERT_TAC (parse_term(str)))
					|_ ->e(ASSERT_TAC (parse_term(str ^" ==> "^ res))))
                 
                else
                       let res=call_collector() in
					 e(ASSERT_TAC (parse_term(str ^" = "^ res)))


| _ -> failwith "Input error")

let prove_arith expr = 
call_mathematica_goal   expr "FullSimplify" ;
e (REAL_ARITH_TAC)




