(*===========================================================================*)
(*                                                                           *)
(*            (c) Copyright, Ons Seddiki  2013-14                            *)
(*                           Hardware Verification Group,                    *)
(*                           Concordia University                            *)
(*                                                                           *)
(*                Contact:   <o_sed@encs.concordia.ca>                       *)
(*                                                                           *)
(* This file translates HOL Light statement to the OpenMath object and stores*)
(*  in an OpenMath XML encoding.                                                          *)
(* ========================================================================= *)

#load "str.cma";;

    (* types *)
 let file = "OpenMathinput.xml"

type lexeme =
  | Nil
  | Numf of float
  | Num of int
  | Str of string
  | Symb of string
  | List of lexeme list

type expr =
  | Nbf of float
  | Nb of int
  | Stri of string
  | Complex of expr * char* expr
  | Complex1 of expr
  | Matrix2 of expr * expr * expr *  expr
  | Matrix3 of expr * expr * expr *  expr * expr * expr *  expr * expr * expr
  | Matrix4 of expr * expr * expr *  expr * expr * expr * expr *  expr * expr * expr * expr *  expr * expr * expr * expr *  expr
  | Vector3 of expr *  expr *  expr
  | Vector2 of expr *  expr
  | Vector4 of expr *  expr *  expr *  expr
  | Eigenvect  of expr
  | Eigenval  of expr
  | Gt of expr * expr
  | Egt of expr * expr
  | Ls of expr * expr
  | Els of expr * expr
  | Eq of expr * expr
  | Neq of expr * expr
  | Add of expr * expr
  | Sub of expr * expr
  | Mul of expr * expr
  | Div of expr * expr
  | Pow of expr * expr
  | Integrate of expr * expr
  | Diff of expr * expr
  | Partialdiff of expr * expr
  | Negs of string
  | Negn of int
  | Negf of float
  | Negl of expr
  | Exp of expr
  | Sin of expr
  | Arcsin of expr
  | Cos of expr
  | Arccos of expr
  | Tan of expr
  | Arctan of expr
  | Abs of expr
  | Sqrt of expr
  | Ln of expr
  | Arg of expr
  | Conj of expr
  | Im of expr
  | Re of expr
  | Det of expr
  | Cross of expr * expr
  | And of expr * expr
  | Complexmat of expr
  | Vector1 of expr
  | Cx  of expr
  | Cexp  of expr
  | Inverse of expr
  | Sum of expr * expr* expr* expr
  | Integrate_finite of expr * expr
  | Intervale of expr * expr
  | Lambda of expr * expr
    (*    Lexical analyzer: it takes as input a string, it gives a list of lexical items (tokens) called lexlist which does not contain Nil or List. The list of tokens is given in the opposite direction compared to the input string  *)

let ASSERT_TAC tm = SUBGOAL_THEN ` tm` ASSUME_TAC

let valeur c = float_of_int( (int_of_char c) - 48 )

let string_of_char = String.make 1

let lex str =
 let rec lexi str i lexlist =
    if i >= String.length str then lexlist
    else let c = str.[i] in match c with
      |'(' |')'|'='|'^'| '[' |']'| '%'|'\\'  -> lexi str (i+1) ((Symb (string_of_char c))::lexlist)
      |'+' -> if (String.length str) > (i + 1) && str.[i+1] = '+'  then lexi str (i+2) ((Symb "++")::lexlist) else if (String.length str) > (i + 1) && str.[i+1] <> '+'  then lexi str (i+2) ((Symb "+")::lexlist) else  failwith "Input Invalid in Operation '+' "
      |'-' -> if (String.length str) > (i + 1) && str.[i+1] = '+'  then lexi str (i+2) ((Symb "--")::lexlist) else if (String.length str) > (i + 1) && str.[i+1] <> '+'  then lexi str (i+2) ((Symb "-")::lexlist) else  failwith "Input Invalid in Operation '-' "
      |'*' -> if (String.length str) > (i + 1) && str.[i+1] = '+'  then lexi str (i+2) ((Symb "**")::lexlist) else if (String.length str) > (i + 1) && str.[i+1] <> '+'  then lexi str (i+2) ((Symb "*")::lexlist) else  failwith "Input Invalid in Operation '*' "
      |'/' -> if (String.length str) > (i + 1) && str.[i+1] = '+'  then lexi str (i+2) ((Symb "//")::lexlist) else if (String.length str) > (i + 1) && str.[i+1] <> '+'  then lexi str (i+2) ((Symb "/")::lexlist) else  failwith "Input Invalid in Operation '*' "
      |'<' -> if (String.length str) > (i + 1) && str.[i+1] = '='  then lexi str (i+2) ((Symb "<=")::lexlist) else if (String.length str) > (i + 1) && str.[i+1] <> '='  then lexi str (i+2) ((Symb "<")::lexlist) else  failwith "Input Invalid in Operation '<' "
      |'>' -> if (String.length str) > (i + 1) && str.[i+1] = '='  then lexi str  (i+2) ((Symb ">=")::lexlist) else if (String.length str) > (i + 1) && str.[i+1] <> '='  then lexi str (i+2) ((Symb ">")::lexlist) else  failwith "Input Invalid in Operation '>' "
      |'!' -> if (String.length str) > (i + 1) && str.[i+1] = '='  then lexi str (i+2) ((Symb "!=")::lexlist)  else  failwith "Input Invalid in Operation '!' "

      |('a'..'z') | ('A' .. 'Z')-> let rec var str i lst is_string  =
                       if i = String.length str then (lst,i)
                       else begin
                         let c = str.[i] in match c with
                           |('0'..'9') ->   if is_string then (var str (i+1) ((string_of_char c)::lst) true)  else  failwith "Input Invalid: Check the variables "
                           |('a'..'z') | ('A' .. 'Z')|'_' ->   if is_string then (var str (i+1) ((string_of_char c)::lst) true)  else  failwith "Input Invalid: Check the variables "
                           |_ -> (lst,i)
                       end
                       in let (lst,j) = (var str i [] true) in let strr = (String.concat "" (List.rev lst)) in (match strr with
                                         | "rpow" ->  lexi str j ((Symb strr)::lexlist)

                                         | "pow" ->  lexi str j ((Symb strr)::lexlist)
                                         | "sin" -> lexi str j ((Symb (String.concat "" (List.rev lst)))::lexlist)
                                         | "csin" -> lexi str j ((Symb (String.concat "" (List.rev lst)))::lexlist)
                                         | "sqrt" -> lexi str j ((Symb (String.concat "" (List.rev lst)))::lexlist)
                                         | "cos" -> lexi str j ((Symb (String.concat "" (List.rev lst)))::lexlist)
                                         | "ccos" -> lexi str j ((Symb (String.concat "" (List.rev lst)))::lexlist)
                                         | "conj" -> lexi str j ((Symb (String.concat "" (List.rev lst)))::lexlist)
                                         | "complex" -> lexi str j ((Symb (String.concat "" (List.rev lst)))::lexlist)
                                         | "tan" -> lexi str j ((Symb (String.concat "" (List.rev lst)))::lexlist)
                                         | "abs" -> lexi str j ((Symb (String.concat "" (List.rev lst)))::lexlist)
                                         | "arg" -> lexi str j ((Symb (String.concat "" (List.rev lst)))::lexlist)
                                         | "asn" -> lexi str j ((Symb (String.concat "" (List.rev lst)))::lexlist)
                                         | "arccos" -> lexi str j ((Symb (String.concat "" (List.rev lst)))::lexlist)
                                         | "arctan" -> lexi str j ((Symb (String.concat "" (List.rev lst)))::lexlist)
                                         | "exp" -> lexi str j ((Symb (String.concat "" (List.rev lst)))::lexlist)
                                         | "eigenval" -> lexi str j ((Symb (String.concat "" (List.rev lst)))::lexlist)
                                         | "eigenvect" -> lexi str j ((Symb (String.concat "" (List.rev lst)))::lexlist)
                                         | "mat2x2" -> lexi str j ((Symb (String.concat "" (List.rev lst)))::lexlist)
                                         | "mat3x3" -> lexi str j ((Symb (String.concat "" (List.rev lst)))::lexlist)
                                         | "mat4x4" -> lexi str j ((Symb (String.concat "" (List.rev lst)))::lexlist)
                                         | "vector" -> lexi str j ((Symb (String.concat "" (List.rev lst)))::lexlist)
                                         | "ln" -> lexi str j ((Symb (String.concat "" (List.rev lst)))::lexlist)
                                         | "im" -> lexi str j ((Symb (String.concat "" (List.rev lst)))::lexlist)
                                         | "int" -> lexi str j ((Symb (String.concat "" (List.rev lst)))::lexlist)
                                         | "re" -> lexi str j ((Symb (String.concat "" (List.rev lst)))::lexlist)
                                         | "diff" -> lexi str j ((Symb (String.concat "" (List.rev lst)))::lexlist)
                                         | "det" -> lexi str j ((Symb (String.concat "" (List.rev lst)))::lexlist)
                                         | "partialdiff" -> lexi str j ((Symb (String.concat "" (List.rev lst)))::lexlist)
                                         | "ccross" -> lexi str j ((Symb (String.concat "" (List.rev lst)))::lexlist)
                                         |"Cx" ->   lexi str j ((Symb (String.concat "" (List.rev lst)))::lexlist)
                                         | "cexp" -> lexi str j ((Symb (String.concat "" (List.rev lst)))::lexlist)
                                        | "and" -> lexi str j ((Symb (String.concat "" (List.rev lst)))::lexlist)
                                        | "inverse" -> lexi str j ((Symb (String.concat "" (List.rev lst)))::lexlist)
                                        | "sigma" -> lexi str j ((Symb (String.concat "" (List.rev lst)))::lexlist)
                                         | "real_integral" -> lexi str j ((Symb (String.concat "" (List.rev lst)))::lexlist)
                                         | "real_interval" -> lexi str j ((Symb (String.concat "" (List.rev lst)))::lexlist)
                                         |_ ->   lexi str j ((Str strr)::lexlist))
      |('0'..'9') -> let rec number str i nb is_flott eps = (*  number returns the number in the string str starting at character i, and the index of the character following the numberf*)
                       if i = String.length str then (nb,i,is_flott)
                       else begin
                         let c = str.[i] in match c with
                           |('0'..'9') ->  if is_flott then number str (i+1) (nb +. (valeur c)*.eps) true (eps /. 10.)
                                          else number str (i+1) (10. *. nb +. (valeur c)) false eps
                           |'.' -> if is_flott then failwith "Input Invalid: Check the float numbers or the reals " else number str (i+1) nb true 0.1
                           |_ -> (nb,i,is_flott)
                       end
                       in let (nb,j,is_flott) = (number str i 0. false 57.) in if is_flott then lexi str j ((Numf (nb))::lexlist) else  lexi str j ((Num (int_of_float (nb)))::lexlist)

      |' ' |'\n' | '\r' | '\t'-> lexi str (i+1) lexlist
      |',' | ';'| '.' -> lexi str (i+1) lexlist

      |_ -> failwith "File parserandsplitter: Input Invalid: Operation not existing"
  in lexi str 0 []


    (* splitting the lexeme list according parentheses: it replaces any pair of brackets by lexemes list. It takse as input a list of currently Lexeme which does not contain Nil or List, and returns a Lexeme list which does not contain (symb '(') or (Symb') ') even in the existing sub-lists *)


(* Auxiliary function: takes as input a list of Lexeme and Lexeme, adds the lexeme to the end of the list and the deeper, ie if the end of the list is a list, it is added at the end thereof and so on, but considering that lists ending by Nil are full*)

let rec add lexlist lex = match lexlist with
  | [] -> [lex]
  | Nil::_ -> failwith "File parserandsplitter: Invalid Input : Bracket missing"
  | (Num _)::_ | (Symb _)::_ | (Numf _)::_ |(Str _)::_  -> lex::lexlist
  | (List [])::t -> (List [lex])::t
  | (List (Nil::l))::t -> lex::(List (Nil::l))::t
  | (List l)::t -> (List (add l lex))::t


let split liste =
  let rec splitter fait reste = match reste with
    | [] -> fait
    | h::t when (h = Symb ")") -> splitter (add fait (List [])) t
    | h::t when ( h = Symb "]") -> splitter (add fait (List [])) t
    | h::t when (h = Symb "(") -> splitter (add fait Nil) t
    | h::t when (h = Symb "[") -> splitter (add fait Nil) t
    | h::t ->                     splitter (add fait h) t;
  in (splitter [] liste)

let rec purge liste = match liste with (* It removes all Nil list of lexical items (including those in sub-lists) *)
  | [] -> []
  | h::t -> match h with
              |Nil -> purge t
              |Num _ | Symb _ |Numf _ |Str _  -> h::(purge t)
              |List l -> (List (purge l))::(purge t)





let rec unaries lexlist = match lexlist with
  | [] -> []
  | Nil::t -> t
  | [Num _] -> lexlist
  | [Numf _] -> lexlist
  | [Str _] -> lexlist
  |[List l] ->[List (unaries l)]
  | [Symb _] -> failwith "File parserandsplitter: Invalid Input : Operation misused"
  | (List l)::b::t -> (List (unaries l))::b::(unaries t)
  | (Num n)::b::t -> (Num n)::b::(unaries t)
  | (Numf n)::b::t -> (Numf n)::b::(unaries t)
  | (Str n)::b::t -> (Str n)::b::(unaries t)
  | (Symb c)::(Num a)::(Num n)::t -> (Symb c):: (Num a):: (Num n)::(unaries t)
  | (Symb c)::(Numf a)::(Numf n)::t -> (Symb c):: (Numf a):: (Numf n)::(unaries t)
  | (Symb c)::(Str a)::(Str n)::t -> (Symb c):: (Str a):: (Str n)::(unaries t)
  | (Symb c)::h::t -> let rec unary_solve lexlist = match lexlist with
                        | [Symb _]  -> failwith "File parserandsplitter: Invalid Input : Operation misused"
                        | (Symb c)::(Num n)::t -> ([Symb c; Num n], t)
                        | (Symb c)::(Numf n)::t -> ([Symb c; Numf n], t)
                        | (Symb c)::(Str n)::t -> ([Symb c; Str n], t)
                        | (Symb c)::(List l)::t -> ([Symb c; List (unaries l)], t)
                        | (Symb c)::t -> let r = unary_solve t in ([Symb c; List (fst r)], (snd r))
                      in let r = unary_solve lexlist in match (snd r) with
                                                          | [] ->[List (fst r)]
                                                          | Nil::_ -> failwith "File parserandsplitter: Invalid Input"

                                                          | h::t -> (List (fst r))::h::(unaries t)





let prior c = if (c = "+") or (c = "-") or (c = "--")or (c = "++") then 1   else 2
let rec priority  = function
  |[] | [_] -> 3
 (* |[a;b] -> failwith "Syntax error16"*)
  |t :: (Symb v):: q -> if ((priority q) = 3) then (prior v) else  (max (prior v) (priority q))
  |_ -> failwith "File parserandsplitter: Invalid priority between operations"

let rec splitlex l lex = match l with
x :: after -> if x = lex then [[];after] else  [List.flatten [x:: List.hd (splitlex after lex)]; List.flatten (List.tl (splitlex after lex)) ]
|[] -> failwith "File parserandsplitter: Invalid splitlex liste"

let contains s1 s2 =
    let re = Str.regexp_string s2
    in
        try ignore (Str.search_forward re s1 0); true
              with Not_found -> false


let rec syntax liste = match liste with
  | [] |[Symb _] -> failwith "File parserandsplitter: Invalid Input"
  | [Num n] -> Nb n
  | [Numf f] -> Nbf f
  | [Str s] -> Stri s
  | [List l] -> syntax l
  | [Symb "-";  Num n] -> Negn n
  | [Symb "-";  Str n] -> Negs n
  | [Symb "-";  Numf n] -> Negf n
  | [Symb "-";   n] -> Negl (syntax [n])
  | [Symb "+"; n] -> syntax [n]
  | [Symb "cos"; n] -> Cos (syntax [n])
  | [Symb "sin"; n] -> Sin (syntax [n])
  | [Symb "ccos"; n] -> Cos (syntax [n])
  | [Symb "csin"; n] -> Sin (syntax [n])
  | [Symb "abs"; n] -> Abs (syntax [n])
  | [Symb "tan"; n] -> Tan (syntax [n])
  | [Symb "sqrt"; n] -> Sqrt (syntax [n])
  | [Symb "asn"; n] -> Arcsin (syntax [n])
  | [Symb "arccos"; n] -> Arccos (syntax [n])
  | [Symb "arctan"; n] -> Arctan (syntax [n])
  | [Symb "exp"; n] -> Exp (syntax [n])
  | [Symb "ln"; n] -> Ln (syntax [n])
  | [Symb "arg"; n] -> Arg (syntax [n])
  | [Symb "conj"; n] -> Conj (syntax [n])
  | [Symb "im"; n] -> Im (syntax [n])
  | [Symb "re"; n] -> Re (syntax [n])
  | [Symb "det"; n] -> Det (syntax [n])
  | [Symb "complex"; List [a;b]] -> Complex (syntax [a],',',syntax [b])

  | [Symb "Cx"; n] -> Cx (syntax [n])
  | [Symb "cexp"; n] -> Cexp (syntax [n])
  | [Symb "eigenvect"; n] -> Eigenvect (syntax [n])
  | [Symb "eigenval"; n] -> Eigenval (syntax [n])
  | [Symb "inverse"; n] -> Inverse (syntax [n])
  | [Symb "int"; List [a;b]] -> Integrate (syntax [a],syntax [b])

  |  [List  [Symb "real_integral"; List a ]; b]-> Integrate_finite(syntax  a ,syntax [b])

  |[Symb "real_interval"; List [a; b]]-> Intervale(syntax  [a] ,syntax [b])
  | [Symb "real_interval"; List [a;b]] -> Intervale(syntax [a],syntax [b])
  | [Symb "int"; List [a;b;c]] -> Integrate (syntax (a :: [b]),syntax [c])
  | [Symb "diff"; List [a;b]] -> Diff (syntax [a],syntax [b])
  | [Symb "diff"; List [a;b;c]] -> Diff (syntax (a :: [b]),syntax [c])
  | [Symb "partialdiff"; List [a;b]] -> Partialdiff (syntax [a],syntax [b])
  | [Symb "mat2x2"; List [a;b;c;d]] -> Matrix2 (syntax [a],syntax [b],syntax [c],syntax [d])
  | [Symb "mat2x2"; List [List a; List b; List c; List d]] -> Matrix2 (syntax a,syntax b,syntax c,syntax d)
  | [Symb "mat2x2";a;b;c;d] -> Matrix2 (syntax [a],syntax [b],syntax [c],syntax [d])

| [Symb "mat3x3"; List [a;b;c;d;e;f;g;h;i]] -> Matrix3 (syntax [a],syntax [b],syntax [c],syntax [d],syntax [e],syntax [f],syntax [g],syntax [h],syntax [i])

  | [Symb "mat3x3";a;b;c;d;e;f;g;h;i] -> Matrix3 (syntax [a],syntax [b],syntax [c],syntax [d],syntax [e],syntax [f],syntax [g],syntax [h],syntax [i])

| [Symb "mat4x4"; List [a;b;c;d;e;f;g;h;i;j;k;l;m;n;q;p]] -> Matrix4 (syntax [a],syntax [b],syntax [c],syntax [d],syntax [e],syntax [f],syntax [g],syntax [h],syntax [i],syntax [j],syntax [k],syntax [l],syntax [m],syntax [n],syntax [q],syntax [p])

  | [Symb "mat4x4";a;b;c;d;e;f;g;h;i;j;k;l;m;n;q;p] -> Matrix4 (syntax [a],syntax [b],syntax [c],syntax [d],syntax [e],syntax [f],syntax [g],syntax [h],syntax [i],syntax [j],syntax [k],syntax [l],syntax [m],syntax [n],syntax [q],syntax [p])

  | [Symb "vector"; List [a;b;c]] -> Vector3 (syntax [a],syntax [b],syntax [c])
  | [Symb "vector"; List [a;b]] -> Vector2 (syntax [a],syntax [b])
  | [Symb "vector"; List [a;b;c;d]] -> Vector4 (syntax [a],syntax [b],syntax [c],syntax [d])

  | [List[Symb "vector"; List a];  b; c ] -> Vector3 (syntax a,syntax [b],syntax [c])
  | [List[Symb "vector"; List a];  b ] -> Vector2 (syntax a,syntax [b])
  | [List[Symb "vector"; List a];  b; c ;d] -> Vector4 (syntax a,syntax [b],syntax [c],syntax [d])

   (*| [Symb "vector"; List [ List a;List b;List c]] -> Vector3 (syntax a,syntax b,syntax c)
   | [Symb "vector"; List [ List a;List b]] -> Vector2(syntax a,syntax b)
   | [Symb "vector"; List [ List a;List b;List c;List d]] -> Vector4 (syntax a,syntax b,syntax c,syntax d)*)
  | [Symb "sigma";List [ List a; b;c;d]] -> Sum (syntax a,syntax [b],syntax [c],syntax [d])
   (*| [Symb "int"; List [ List a; b;c;d]] -> Integrate_finite  (syntax a,syntax [b],syntax [c],syntax [d])*)
  
| [List [Symb "\\"; a]; b] -> Lambda (syntax [a],syntax [b])
|[Symb "\\"; a; b]-> Lambda (syntax [a],syntax [b])

| before::(Symb c)::after -> if ((prior c) < priority after)  then (match c with
                                | "+" -> Add ((syntax [before]),(syntax after))
                                | "-" -> Sub ((syntax [before]),(syntax after))
                                | "*" -> Mul ((syntax [before]),(syntax after))
                                | "ccross" -> Cross ((syntax [before]),(syntax after))
                                | "%" -> Mul ((syntax [before]),(syntax after))
                                | "/" -> Div ((syntax [before]),(syntax after))
                                | "++" -> Add ((syntax [before]),(syntax after))
                                | "--" -> Sub ((syntax [before]),(syntax after))
                                | "**" -> Mul ((syntax [before]),(syntax after))
                                | "//" -> Div ((syntax [before]),(syntax after))
                                | "pow" -> Pow ((syntax [before]),(syntax after))
                                | "rpow" -> Pow ((syntax [before]),(syntax after))
                                | _ -> failwith "This expression does not exist")
                              else syntax ((List [before; Symb c; List.hd after])::(List.tl after));

  | _ -> failwith "File parserandsplitter: The input expression misused: missing brackets"



let rec atom liste = match liste with
| [List l] -> atom l
| _->  if (List.mem (Symb "=") liste) then   (match "=" with
                                |   "=" -> Eq ((syntax (List.hd (splitlex liste (Symb "=")))),(syntax(List.hd(List.tl(splitlex liste (Symb "="))))))
                                | _ -> failwith "File parserandsplitter: The Invalid Input: \"=\" Operator")
       else if (List.mem (Symb "!=") liste) then   (match "!=" with
                                |  "!=" -> Neq ((syntax (List.hd (splitlex liste (Symb "!=")))),(syntax(List.hd(List.tl(splitlex liste (Symb "!="))))))
                                | _ -> failwith "File parserandsplitter: The Invalid Input: \"!=\" Operator")
      else if (List.mem (Symb "<=") liste) then   (match "<=" with
                                | "<=" -> Els ((syntax (List.hd (splitlex liste (Symb "<=")))),(syntax(List.hd(List.tl(splitlex liste (Symb "<="))))))
                                | _ -> failwith "File parserandsplitter: The Invalid Input: \"<=\" Operator")
     else if (List.mem (Symb "<") liste) then   (match "<" with
                                |  "<" -> Ls ((syntax (List.hd (splitlex liste (Symb "<")))),(syntax(List.hd(List.tl(splitlex liste (Symb "<"))))))
                                | _ -> failwith "File parserandsplitter: The Invalid Input: \"<\" Operator")
    else if (List.mem (Symb ">=") liste) then   (match ">=" with
                                |  ">=" -> Egt ((syntax (List.hd (splitlex liste (Symb ">=")))),(syntax(List.hd(List.tl(splitlex liste (Symb ">="))))))
                                | _ -> failwith "File parserandsplitter: The Invalid Input: \">=\" Operator")
  else if (List.mem (Symb ">") liste) then   (match ">" with
                                |  ">" -> Gt ((syntax (List.hd (splitlex liste (Symb ">")))),(syntax(List.hd(List.tl(splitlex liste (Symb ">"))))))
                                | _ -> failwith "File parserandsplitter: The Invalid Input: \">\" Operator")
 else if (List.mem (Symb "and") liste) then   (match "and" with
                                |  "and" -> And ((atom (List.hd (splitlex liste (Symb "and")))),(atom(List.hd(List.tl(splitlex liste (Symb "and"))))))
                                | _ -> failwith "File parserandsplitter: The Invalid Input: \"and\" Operator")

else failwith "File parserandsplitter: The input expression misused: missing brackets"


    (*    Printer: there takes as input a expression of operations and it gives his description OpenMath Object *)


let rec calc expr = match expr with
   | Nb n -> "<OMI>" ^ string_of_int n ^ "</OMI>" ^ "\n"
  | Nbf f -> "<OMF dec = \"" ^  string_of_float  f ^ "\"/>" ^ "\n"
  | Stri s ->  if  s="pi" then   "<OMS name=\"pi\" cd=\"nums1\"/>" ^ "\n" else  if  s="ii" then   "<OMS name=\"I\" cd=\"nums1\"/>" ^ "\n"  else  if  s="e" then   "<OMS name=\"e\" cd=\"nums1\"/>" ^ "\n"   else "<OMV name= \"" ^ s ^ "\"/>" ^ "\n"
  | Add(a,b) -> "<OMA>"^ "\n"^ "<OMS name=\"plus\" cd=\"arith1\"/>" ^ "\n" ^ calc a ^ calc b  ^"</OMA>"^ "\n"
  | Sub(a,b) -> "<OMA>"^ "\n"^ "<OMS name=\"minus\" cd=\"arith1\"/>" ^ "\n" ^ calc a ^ calc b  ^"</OMA>"^ "\n"
  | Mul(a,b) -> "<OMA>"^ "\n"^ "<OMS name=\"times\" cd=\"arith1\"/>" ^ "\n" ^ calc a ^ calc b  ^"</OMA>"^ "\n"
  | Div(a,b) -> (*let cb = calc b in (if cb = 0. then (failwith "Division par zéro !") else*) "<OMA>"^ "\n"^ "<OMS name=\"divide\" cd=\"arith1\"/>" ^ "\n" ^ calc a ^ calc b  ^"</OMA>"^ "\n"
  | Gt(a,b) -> "<OMA>"^ "\n"^ "<OMS name=\"gt\" cd=\"relation1\"/>" ^ "\n" ^ calc a ^ calc b  ^"</OMA>"^ "\n"
  | Egt(a,b) -> "<OMA>"^ "\n"^ "<OMS name=\"geq\" cd=\"relation1\"/>" ^ "\n" ^ calc a ^ calc b  ^"</OMA>"^ "\n"
  | Ls(a,b) -> "<OMA>"^ "\n"^ "<OMS name=\"lt\" cd=\"relation1\"/>" ^ "\n" ^ calc a ^ calc b  ^"</OMA>"^ "\n"
  | Els(a,b) -> "<OMA>"^ "\n"^ "<OMS name=\"leq\" cd=\"relation1\"/>" ^ "\n" ^ calc a ^ calc b  ^"</OMA>"^ "\n"
  | Eq(a,b) -> "<OMA>"^ "\n"^ "<OMS name=\"eq\" cd=\"relation1\"/>" ^ "\n" ^ calc a ^ calc b  ^"</OMA>"^ "\n"
  | Neq(a,b) -> "<OMA>"^ "\n"^ "<OMS name=\"neq\" cd=\"relation1\"/>" ^ "\n" ^ calc a ^ calc b  ^"</OMA>"^ "\n"
  | Negn a ->  "<OMI>" ^ "-" ^ string_of_int a ^ "</OMI>" ^ "\n"
  | Negs a -> if  a="ii" then "<OMA>"^ "\n"^  "<OMS name=\"complex_cartesian\" cd=\"complex1\" />" ^ "\n" ^ "<OMI> 0 </OMI>" ^ "\n" ^ "<OMI>-1</OMI>" ^ "\n"^"</OMA>"^ "\n"else "<OMV name= \"" ^ "-" ^ a ^ "\"/>" ^ "\n"
  | Negf a -> "<OMV dec= \"" ^ "-" ^ string_of_float a ^ "\"/>" ^ "\n"
  | Negl a -> "<OMA>"^ "\n"^ "<OMS name=\"unary_minus\" cd=\"arith1\"/>" ^ "\n" ^ calc a  ^"</OMA>"^ "\n"
  | Sin a -> "<OMA>"^ "\n"^"<OMS name=\"sin\" cd=\"transc1\"/>" ^ "\n" ^ calc a  ^"</OMA>"^ "\n"
  | Arcsin a -> "<OMA>"^ "\n"^"<OMS name=\"arcsin\" cd=\"transc1\"/>" ^ "\n" ^ calc a ^"</OMA>"^ "\n"
  | Cos a ->"<OMA>"^ "\n"^"<OMS name=\"cos\" cd=\"transc1\"/>" ^ "\n" ^ calc a ^"</OMA>"^ "\n"
  | Arccos a ->"<OMA>"^ "\n"^"<OMS name=\"arccos\" cd=\"transc1\"/>" ^ "\n" ^ calc a ^"</OMA>"^ "\n"
  | Tan a ->"<OMA>"^ "\n"^"<OMS name=\"tan\" cd=\"transc1\"/>" ^ "\n" ^ calc a ^"</OMA>"^ "\n"
  | Arctan a ->"<OMA>"^ "\n"^"<OMS name=\"arctan\" cd=\"transc1\"/>" ^ "\n" ^ calc a ^"</OMA>"^ "\n"
  | Pow (a,b) ->"<OMA>"^ "\n"^"<OMS name=\"power\" cd=\"arith1\"/>" ^ "\n" ^ calc a^ calc b ^"</OMA>"^ "\n"
  | Abs a ->"<OMA>"^ "\n"^"<OMS name=\"abs\" cd=\"arith1\"/>" ^ "\n" ^ calc a ^"</OMA>"^ "\n"
  | Sqrt a ->"<OMA>"^ "\n"^"<OMS name=\"root\" cd=\"arith1\"/>" ^ "\n" ^calc a^"<OMI>" ^ string_of_int 2 ^ "</OMI>" ^ "\n" ^"</OMA>"^ "\n"
  | Exp a ->"<OMA>"^ "\n"^"<OMS name=\"exp\" cd=\"transc1\"/>" ^ "\n" ^ calc a ^"</OMA>"^ "\n"
  | Ln a ->"<OMA>"^ "\n"^"<OMS name=\"ln\" cd=\"transc1\"/>" ^ "\n" ^ calc a ^"</OMA>"^ "\n"
  | Complex(a,',',b) ->"<OMA>"^ "\n"^"<OMS name=\"complex_cartesian\" cd=\"complex1\"/>" ^ "\n" ^ calc a ^ calc b ^"</OMA>"^ "\n"
  | Complex1(a) ->"<OMA>"^ "\n"^"<OMS name=\"complex_cartesian\" cd=\"complex1\"/>" ^ "\n" ^ calc a   ^"</OMA>"^ "\n"
  | Eigenval(a) ->"<OMA>"^ "\n"^"<OMS name=\"eigenvalues\" cd=\"linalgeig1\"/>" ^ "\n"^ calc a ^"</OMA>"^ "\n"

  | Eigenvect(a) ->"<OMA>"^ "\n"^"<OMS name=\"eigenvector\" cd=\"linalg4\"/>" ^ "\n"^ calc a ^"</OMA>"^ "\n"
  | Matrix2(a,b,c,d) ->"<OMA>"^ "\n"^"<OMS name=\"matrix\" cd=\"linalg2\"/>" ^ "\n"^  "<OMA>"^ "\n"^"<OMS name=\"matrixrow\" cd=\"linalg2\"/>" ^ "\n"^ calc a ^ calc b ^"</OMA>"^ "\n"^"<OMA>"^ "\n"^"<OMS name=\"matrixrow\" cd=\"linalg2\"/>" ^ "\n"^ calc c ^ calc d ^"</OMA>"^ "\n" ^"</OMA>"^ "\n"

  | Matrix3(a,b,c,d,e,f,g,h,i) ->"<OMA>"^ "\n"^"<OMS name=\"matrix\" cd=\"linalg2\"/>" ^ "\n"^  "<OMA>"^ "\n"^"<OMS name=\"matrixrow\" cd=\"linalg2\"/>" ^ "\n"^ calc a ^ calc b ^ calc c ^"</OMA>"^ "\n"^"<OMA>"^ "\n"^"<OMS name=\"matrixrow\" cd=\"linalg2\"/>" ^ "\n"^ calc d ^ calc e ^ calc f ^"</OMA>"^ "\n" ^  "<OMA>"^ "\n"^"<OMS name=\"matrixrow\" cd=\"linalg2\"/>" ^ "\n"^ calc g ^ calc h^ calc i  ^"</OMA>"^ "\n"^"</OMA>"^ "\n"

  | Matrix4(a,b,c,d,e,f,g,h,i,j,k,l,m,n,q,p) ->"<OMA>"^ "\n"^"<OMS name=\"matrix\" cd=\"linalg2\"/>" ^ "\n"^  "<OMA>"^ "\n"^"<OMS name=\"matrixrow\" cd=\"linalg2\"/>" ^ "\n"^ calc a ^ calc b ^ calc c ^ calc d ^"</OMA>"^ "\n"^"<OMA>"^ "\n"^"<OMS name=\"matrixrow\" cd=\"linalg2\"/>" ^ "\n"^ calc e ^ calc f ^ calc g  ^ calc h ^"</OMA>"^ "\n" ^  "<OMA>"^ "\n"^"<OMS name=\"matrixrow\" cd=\"linalg2\"/>" ^ "\n"^ calc i ^ calc j ^ calc k ^ calc l  ^"</OMA>"^ "\n"^ "<OMA>"^ "\n"^"<OMS name=\"matrixrow\" cd=\"linalg2\"/>" ^ "\n"^ calc m ^ calc n ^ calc q ^ calc p  ^"</OMA>"^ "\n"^"</OMA>"^ "\n"


  | Vector3 (a,b,c) ->"<OMA>"^ "\n"^"<OMS name=\"matrixrow\" cd=\"linalg2\"/>" ^ "\n"^ calc a ^ calc b^ calc c ^"</OMA>"^ "\n"

   | Vector2 (a,b) ->"<OMA>"^ "\n"^"<OMS name=\"matrixrow\" cd=\"linalg2\"/>" ^ "\n"^ calc a ^ calc b ^"</OMA>"^ "\n"

 | Vector4 (a,b,c,d) ->"<OMA>"^ "\n"^"<OMS name=\"matrixrow\" cd=\"linalg2\"/>" ^ "\n"^ calc a ^ calc b^ calc c^ calc d ^"</OMA>"^ "\n"

| Vector1 (a) ->"<OMA>"^ "\n"^"<OMS name=\"matrixrow\" cd=\"linalg2\"/>" ^ "\n"^ calc a ^"</OMA>"^ "\n"
  | Inverse(a) ->"<OMA>"^ "\n"^"<OMS name=\"inverse\" cd=\"linalg1\"/>" ^ "\n"^ calc a ^"</OMA>"^ "\n"
  | Arg a -> "<OMA>"^ "\n"^"<OMS name=\"argument\" cd=\"complex1\"/>" ^ "\n" ^ calc a  ^"</OMA>"^ "\n"
  | Conj a -> "<OMA>"^ "\n"^"<OMS name=\"conjugate\" cd=\"complex1\"/>" ^ "\n" ^ calc a  ^"</OMA>"^ "\n"
  | Re a -> "<OMA>"^ "\n"^"<OMS name=\"real\" cd=\"complex1\"/>" ^ "\n" ^ calc a  ^"</OMA>"^ "\n"
  | Im a -> "<OMA>"^ "\n"^"<OMS name=\"imaginary\" cd=\"complex1\"/>" ^ "\n" ^ calc a  ^"</OMA>"^ "\n"
  | Det a -> "<OMA>"^ "\n"^"<OMS name=\"determinant\" cd=\"linalg1\"/>" ^ "\n" ^ calc a  ^"</OMA>"^ "\n"
  | Integrate(a,b) -> "<OMA>"^ "\n"^"<OMS name=\"int\" cd=\"calculus1\"/>" ^ "\n" ^ "<OMBIND>"^  "\n"^"<OMS name=\"lambda\" cd=\"fns1\"/>" ^ "\n" ^"<OMBVAR>" ^ "\n"^calc b^ " </OMBVAR>" ^ "\n"^calc a^"</OMBIND>" ^ "\n" ^"</OMA>"^ "\n"
  | Diff(a,b) -> "<OMA>"^ "\n"^"<OMS name=\"diff\" cd=\"calculus1\"/>" ^ "\n" ^ "<OMBIND>"^  "\n"^"<OMS name=\"lambda\" cd=\"fns1\"/>" ^ "\n" ^"<OMBVAR>" ^ "\n"^calc b^ " </OMBVAR>" ^ "\n"^calc a^"</OMBIND>" ^ "\n" ^"</OMA>"^ "\n"
 | Partialdiff(a,b) -> "<OMA>"^ "\n"^"<OMS name=\"D\" cd=\"calculus1\"/>" ^ "\n" ^ "<OMBIND>"^  "\n"^"<OMS name=\"lambda\" cd=\"fns1\"/>" ^ "\n" ^"<OMBVAR>" ^ "\n"^calc b^ " </OMBVAR>" ^ "\n"^calc a^"</OMBIND>" ^ "\n" ^"</OMA>"^ "\n"
  | Complexmat a -> calc a
  | Cross (a,b) ->"<OMA>"^ "\n"^"<OMS name=\"vectorproduct\" cd=\"linalg1\"/>" ^ "\n" ^ calc a^ calc b ^"</OMA>"^ "\n"
  | And (a,b) ->"<OMA>"^ "\n"^"<OMS name=\"and\" cd=\"logic1\"/>" ^ "\n" ^ calc a^ calc b ^"</OMA>"^ "\n"
| Cx a -> calc a
  | Cexp a -> "<OMA>"^ "\n"^"<OMS name=\"power\" cd=\"arith1\"/>" ^ "\n" ^"<OMS name=\"e\" cd=\"nums1\"/>" ^ "\n" ^ calc a ^"</OMA>"^ "\n"
 | Sum(a,b,c,d) -> "<OMA>"^ "\n"^"<OMS name=\"sum\" cd=\"arith1\"/>" ^ "\n" ^  "<OMA>"^ "\n"^"<OMS name=\"integer_interval\" cd=\"interval1\"/>"^ "\n"^  calc c^ calc d ^"</OMA>"^ "\n"^ "<OMBIND>"^  "\n"^"<OMS name=\"lambda\" cd=\"fns1\"/>" ^ "\n" ^"<OMBVAR>" ^ "\n"^calc b^ "</OMBVAR>" ^ "\n"^calc a^"</OMBIND>" ^ "\n" ^"</OMA>"^ "\n"
  |  Integrate_finite (a,c) -> "<OMA>"^ "\n"^"<OMS name=\"defint\" cd=\"calculus1\"/>" ^ "\n"^ calc a^calc c ^"</OMA>"^ "\n"
  |  Intervale (c,d) ->   "<OMA>"^ "\n"^"<OMS name=\"interval\" cd=\"interval1\"/>"^ "\n"^  calc c^ calc d ^"</OMA>"^ "\n"
  |Lambda (a,b) -> "<OMBIND>"^  "\n"^"<OMS name=\"lambda\" cd=\"fns1\"/>" ^ "\n" ^"<OMBVAR>" ^ "\n"^calc a^ "</OMBVAR>" ^ "\n"^calc b^"</OMBIND>" ^ "\n"
  | _ -> failwith "File parserandsplitter: The OpenMath description is unvalide or does not exist"




let ecrire_fichier r=
let canal_sortie = open_out file in
  output_string canal_sortie r;
close_out canal_sortie

let  transformer_terme str =
let len = String.length str in
let res =  String.create len in
for i = 0 to pred len do
if (str.[i] = '&' ) or (str.[i] = '#' )then res.[i] <- ' '
else
   res.[i] <- str.[i];
done;
(res)

let evall str =
unaries (purge (split (lex (transformer_terme str))))

let eval str =
if (String.contains str '=') or (String.contains str '<') or (String.contains str '>') or (String.contains str '!') or (contains str "and") then
begin
let  res = calc (atom (unaries (purge (split (lex (transformer_terme str)))))) in
  let r= "<OMOBJ>"^ "\n" ^ res ^ "</OMOBJ>" ^ "\n" in
ecrire_fichier r
 end
else
let  res = calc (syntax (unaries (purge (split (lex (transformer_terme str)))))) in
  let r= "<OMOBJ>"^ "\n" ^ res ^ "</OMOBJ>" ^ "\n" in
ecrire_fichier r

