/*
 * $Id: MathematicaSocketHandler.java,v 1.4 2002/10/31 09:54:01 mriem Exp $
 *
 * Copyright (c) 1999-2002, RIACA, Technische Universiteit Eindhoven (TU/e).
 * All Rights Reserved.
 *
 * The contents of this file are subject to the RIACA Public License
 * Version 1.0 (the "License"); you may not use this file except in
 * compliance with the License. A copy of the License is available at
 * http://www.riaca.win.tue.nl/
 */

package mathematica.service;

import java.io.BufferedReader;
import java.io.InputStreamReader;
import java.io.PrintWriter;

import nl.tue.win.riaca.mathematica.link.MathematicaLink;
import nl.tue.win.riaca.service.socket.Handler;
import nl.tue.win.riaca.service.socket.Host;

/**
 * A class that handles the actual connection to/from a Mathematica engine. <p>
 * 
 * @author Manfred Riem
 * @version 1.0
 */

public class MathematicaSocketHandler
    extends Handler
{
    /**
     * Default constructor. <p>
     *
     * @since 1.0
     */

    public MathematicaSocketHandler()
    {
	super();
	
	// System.out.println( "MathematicaSocketHandler.<init>" );
    }

    
    /**
     * Default constructor. <p>
     * 
     * @since 1.0
     */
   
    public MathematicaSocketHandler( Host fHost )
    {
	super( fHost );
	
	// System.out.println( "MathematicaSocketHandler.<init>" );
    }
    
    
    /**
     * Handles what needs to be done. <p>
     * 
     * @since 1.0
     */
    
    public void handle()
    {
	// System.out.println( "MathematicaSocketHandler.handle" );
	
	MathematicaSocketService tService  = 
	    (MathematicaSocketService) mHost.getService();
	
	MathematicaLink tLink = tService.getLink();
	
	if ( tLink != null )
	{
	    try
	    {
		/*
		 * We need the socket input-stream to read the actual request,
		 * and the socket output-stream to write the actual response.
		 */
		
		BufferedReader tSocketInput = 
		    new BufferedReader(
		        new InputStreamReader( mSocket.getInputStream() ) );
		
		PrintWriter tSocketOutput = 
		    new PrintWriter( mSocket.getOutputStream() );
		
		String tInput  = "";
		String tOutput = "";
		
		while( !tSocketInput.ready() )
		{
		    try
		    {
			Thread.sleep( 500 );
		    }
		    catch( InterruptedException tException ) {}
		}
        
		tInput = tSocketInput.readLine();
		
		if ( !tInput.startsWith( "Quit" ) )
		{
		    tOutput = tLink.execute( tInput );
		    
		    tSocketOutput.println( tOutput );
		    tSocketOutput.flush();
		}
	    }
	    catch( Exception tException ) 
	    {
		tException.printStackTrace();
	    }
	}
	
	try
	{
	    mSocket.close();
	}
	catch( Exception tException ) {}
    }
}

