/*
 * $Id: MathematicaSocketService.java,v 1.2 2002/08/15 10:26:39 mriem Exp $
 *
 * Copyright (c) 1999-2002, RIACA, Technische Universiteit Eindhoven (TU/e).
 * All Rights Reserved.
 *
 * The contents of this file are subject to the RIACA Public License
 * Version 1.0 (the "License"); you may not use this file except in
 * compliance with the License. A copy of the License is available at
 * http://www.riaca.win.tue.nl/
 */

package mathematica.service;

import mathematicaGUI.MathematicaShell;
import nl.tue.win.riaca.mathematica.link.MathematicaLink;
import nl.tue.win.riaca.service.socket.SocketService;

/**
 * A Mathematica service to run the Mathematica package across the web. <p>
 * 
 * @author Manfred Riem (mriem@win.tue.nl)
 * @version 1.0
 */

public class MathematicaSocketService
    extends SocketService
{
    /**
     * Stores the link. <p>
     *
     * @since 1.0
     */
    
    protected MathematicaLink mLink;
    
    
    /**
     * Starts the service. <p>
     * 
     * @since 1.0
     */
    
    public boolean start( )
    {
	if ( super.start() == false ) return false;

	// System.out.println( "MathematicaSocketService.start" );
	
	//C:\Program Files\Wolfram Research\Mathematica\8.0
	
	
	try
	{
		String tCommand = mProperties.getProperty( "command" );
		tCommand = tCommand.concat("-linkmode launch -linkname 'math -mathlink'");
		mLink = new MathematicaLink();
		mLink.start( tCommand );


		MathematicaShell shell = new MathematicaShell();
		String res= "";
		tCommand = shell.test1();
		System.out.println ("Mathematica Input = " + tCommand);
		res=mLink.execute(tCommand);
		System.out.println ("Mathematica Output = " + res);

		System.out.println ("Result = \n " + shell.test(res));
		mLink.stop();
		System.exit(0);

	}
	catch( Exception tException )
	{
	    tException.printStackTrace();
	    
	    return false;
	}
	
	return true;
    }
    
    
    /**
     * Stops the service. <p>
     * 
     * @since 1.0
     */
    
    public boolean stop()
    {
	// System.out.println( "MathematicaSocketService.stop" );
	
	if ( super.stop() == false ) return false;

	mLink.stop();
	mLink = null;
	
	return true;
    }
    
    
    /**
     * Get the link. <p>
     *
     * @since 1.0
     */

    public MathematicaLink getLink()
    {
	// System.out.println( "MathematicaSocketService.getLink" );

	return mLink;
    }


    /**
     * Set the link. <p>
     *
     * @since 1.0
     */

    public void setLink( MathematicaLink fLink )
    {
	// System.out.println( "MathematicaSocketService.setLink" );

	mLink = fLink;
    }
}
