/*
 * $Id: MathematicaSocketServiceAppl.java,v 1.1 2002/10/31 09:54:01 mriem Exp $
 *
 * Copyright (c) 2002, RIACA, Technische Universiteit Eindhoven (TU/e).
 * All Rights Reserved.
 */

package mathematica.service;

import java.awt.BorderLayout;
import java.awt.FlowLayout;
import java.awt.GridLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;
import java.util.Properties;

import javax.swing.JButton;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JTextField;


/**
 * Simple service application starter.
 *
 * @author Manfred Riem (mriem@win.tue.nl).
 * @version 1.0
 */

public class MathematicaSocketServiceAppl 
    extends JFrame
    {
        /**
	 * 
	 */
	private static final long serialVersionUID = 1L;
		/**
         * Constructor.
         *
         * @since 1.0
         */
        
        public MathematicaSocketServiceAppl()
        {
            //System.out.println( "MathematicaSocketServiceAppl.<init>" );
    	
            mHost    = new JTextField( "localhost" );
            mPort    = new JTextField( "1235" );
            mBacklog = new JTextField( "5" );
    	mStart   = new JButton   ( "Start" );
            mStop    = new JButton   ( "Stop" );
            mCommand = new JTextField(
    	  "/tools/Mathematica/Executable/math " );
    	
            init();
        }
        

        /**
         * Initializes the components.
         *
         * @since 1.0
         */
        
        public void init()
        {
            //System.out.println( "MathematicaSocketServiceAppl.init" );
    	
            JPanel jpanel = new JPanel();
            jpanel.setLayout(new GridLayout(4, 2));
            jpanel.add(new JLabel("hostname"));
            jpanel.add(mHost);
            jpanel.add(new JLabel("portnumber"));
            jpanel.add(mPort);
            jpanel.add(new JLabel("backlog"));
            jpanel.add(mBacklog);
            jpanel.add(new JLabel("command"));
            jpanel.add(mCommand);
            JPanel jpanel1 = new JPanel();
            jpanel1.setLayout(new FlowLayout());
            jpanel1.add(mStart);
            jpanel1.add(mStop);
            getContentPane().setLayout(new BorderLayout());
            getContentPane().add(jpanel, "Center");
            getContentPane().add(jpanel1, "East");

            mStart.addActionListener(new ActionListener() 
    	{
    	    public void actionPerformed(ActionEvent actionevent)
                {
                    buttonStart();
                }
            });
    	
            mStop.addActionListener(new ActionListener() 
    	{
                public void actionPerformed(ActionEvent actionevent)
                {
                    buttonStop();
                }
            });
    	
            addWindowListener( new WindowAdapter() 
    	{
    	    public void windowClosing(WindowEvent windowevent)
                {
                    menuFileExit();
                }
            });
        }
        

        /**
         * Start button.
         *
         * @since 1.0
         */
        
        public void buttonStart()
        {
           // System.out.println("MathematicaSocketServiceAppl.buttonStart");
        
    	if(mService == null)
            {
                mService = new MathematicaSocketService();
                Properties properties = new Properties();
                properties.setProperty("hostname", mHost.getText());
                properties.setProperty("port", mPort.getText());
                properties.setProperty("backlog", mBacklog.getText());
                properties.setProperty("command", mCommand.getText());
    	    properties.setProperty("handler", "nl.tue.win.riaca.mathematica.service.MathematicaSocketHandler" );
                mService.setProperties(properties);
                mService.start();
            }
        }


        /**
         * Stop button.
         *
         * @since 1.0
         */
        
        public void buttonStop()
        {
           // System.out.println("MathematicaSocketServiceAppl.buttonStop");
            if(mService != null)
                mService.stop();
            mService = null;
        }
        

        /**
         * Main entry point.
         *
         * @since 1.0
         */

        public static void main(String args[])
        {
           // System.out.println("MathematicaSocketServiceAppl.main");
            MathematicaSocketServiceAppl mathematicasocketserviceframe = new MathematicaSocketServiceAppl();
            mathematicasocketserviceframe.setSize(700, 400);
            mathematicasocketserviceframe.setVisible(true);
        }

        
        /**
         * File | Exit menu
         *
         * @since 1.0
         */
        
        public void menuFileExit()
        {
           // System.out.println("MathematicaSocketServiceAppl.menuFileExit");
            if(mService != null)
                buttonStop();
            System.exit(0);
        }
        
        protected JTextField mHost;
        protected JTextField mPort;
        protected JTextField mBacklog;
        protected JTextField mCommand;
        protected JButton mStart;
        protected JButton mStop;
        protected MathematicaSocketService mService;
    }
