/*
 * $Id: MathematicaCDCodec.java,v 1.4 2002/12/06 12:45:34 mriem Exp $
 *
 * Copyright (c) 1999-2002, RIACA, Technische Universiteit Eindhoven (TU/e).
 * All Rights Reserved.
 *
 * The contents of this file are subject to the RIACA Public License
 * Version 1.0 (the "License"); you may not use this file except in
 * compliance with the License. A copy of the License is available at
 * http://www.riaca.win.tue.nl/
 */

package mathematicaCodec;

import nl.tue.win.riaca.openmath.codec.*;
import nl.tue.win.riaca.openmath.lang.*;
import nl.tue.win.riaca.openmath.io.*;
import org.xml.sax.*;
import java.util.*;
import java.io.*;

/**
 * A codec that translates (XML-encoded) OpenMath objects as defined in the 
 * Content Dictionary it implements to their backengine equivalent and 
 * vice-versa. <p>
 *
 * Note that this kind of Codec always needs a master codec as parent to 
 * terminate properly. <p>
 *
 * @author Manfred Riem (mriem@win.tue.nl)
 * @version 1.3
 */

public abstract class MathematicaCDCodec
    extends CDCodec
{
    /**
     * Constructor. <p>
     *
     * @since 1.0
     */

    public MathematicaCDCodec() 
    {
        super();
        
        // System.out.println( "MathematicaCDCodec.<init>" );
    }


    /**
     * Constructor (parameterized). <p>
     *
     * @since 1.0
     */

    public MathematicaCDCodec( Codec fParent )
    {
        super( fParent );
        
	// System.out.println( "MathematicaCDCodec.<init>" );
    }
    
    
    /**
     * Register the decoders.
     *
     * @since 1.0
     */
    
    public abstract void registerDecoders();
    
    
    /**
     * Deregister the decoder.
     *
     * @since 1.0
     */

    public abstract void deregisterDecoders();
    

    
    /* -------------------------------------------------------------------
   
         Encode utility methods
    
       ------------------------------------------------------------------- */

    
    /**
     * Encodes an binary application. <p>
     *
     * @since 1.0
     */
    
    public String encodeBinary( OMObject fOMObject, 
			        String   fPrefix,
			        String   fInfix,
			        String   fPostfix )
	throws CodecEncodeException
    {
	// System.out.println( "MathematicaCDCodec.encodeBinary" );
	
	OMApplication tApplication = (OMApplication) fOMObject;
	
	if( tApplication.getElementAt( 1 ) != null ||
	    tApplication.getElementAt( 2 ) != null ) 
	{    
	    StringBuffer tResult = new StringBuffer();
	    
	    tResult.append( fPrefix );
	    
	    String tLeafResult = mParent.encodeOMObject( 
	        tApplication.getElementAt( 1 ) );
	    
	    tResult.append( tLeafResult );
	    tResult.append( fInfix );
	    
	    tLeafResult = mParent.encodeOMObject( 
	        tApplication.getElementAt( 2 ) );
	    
	    tResult.append( tLeafResult );
	    tResult.append( fPostfix );
	    
	    return tResult.toString();
	}
	
	throw new CodecEncodeException( "Unable to encode: " + fOMObject );
    }
    
    
    /**
     * Encodes an n-ary application. <p>
     *
     * @since 1.0
     */
    
    public String encodeNary( OMObject fOMObject, 
			      String   fPrefix,
			      String   fInfix,
			      String   fPostfix )
	throws CodecEncodeException
    {
	// System.out.println( "MathematicaCDCodec.encodeNary" );
	
	OMApplication tApplication = (OMApplication) fOMObject;
	
	if ( tApplication.getLength() >= 3 )
	{
	    StringBuffer  tResult = new StringBuffer();
	    Enumeration   tEnum   = tApplication.getElements().elements();
	    
	    tEnum.nextElement();	
	    
	    tResult.append( fPrefix );
	    
	    for( ; tEnum.hasMoreElements(); )
	    {
		tResult.append( 
		    mParent.encodeOMObject( (OMObject) tEnum.nextElement() ) );
		
		if ( tEnum.hasMoreElements() )
		    tResult.append( fInfix );
	    }
	    
	    tResult.append( fPostfix );
	    
	    return tResult.toString();
	}
	
        throw new CodecEncodeException( "Unable to encode: " + fOMObject );
    }
    
    
    /**
     * Encodes an unary application. <p>
     *
     * @since 1.0
     */
    
    public String encodeUnary( OMObject fOMObject, 
			       String   fPrefix,
			       String   fPostfix )
	throws CodecEncodeException
    {
	// System.out.println( "MathematicaCDCodec.encodeUnary" );
	
	OMApplication tApplication = (OMApplication) fOMObject;
	
	if( tApplication.getElementAt( 1 ) != null )
	{
	    StringBuffer tResult = new StringBuffer();
	    
	    tResult.append( fPrefix );
	    
	    String tLeafResult = mParent.encodeOMObject( 
	        tApplication.getElementAt( 1 ) );
	    
	    tResult.append( tLeafResult );
	    tResult.append( fPostfix );
	    
	    return tResult.toString();
	}
	
	throw new CodecEncodeException( "Unable to encode: " + fOMObject );
    }    
}
