/*
 * $Id: alg1Codec.java,v 1.4 2002/11/21 22:46:03 mriem Exp $
 *
 * Copyright (c) 1999-2002, RIACA, Technische Universiteit Eindhoven (TU/e).
 * All Rights Reserved.
 *
 * The contents of this file are subject to the RIACA Public License
 * Version 1.0 (the "License"); you may not use this file except in
 * compliance with the License. A copy of the License is available at
 * http://www.riaca.win.tue.nl/
 */


package mathematicaCodec;

import nl.tue.win.riaca.openmath.codec.CDCodec;
import nl.tue.win.riaca.openmath.codec.Codec;
import nl.tue.win.riaca.openmath.codec.CodecDecodeException;
import nl.tue.win.riaca.openmath.codec.CodecEncodeException;
import nl.tue.win.riaca.openmath.lang.OMObject;
import nl.tue.win.riaca.openmath.lang.OMSymbol;

/**
 * A CD-codec that handles the OpenMath 'alg1' CD. <p>
 *
 * @author Manfred Riem (mriem@win.tue.nl)
 * @version 1.0
 */

public class alg1Codec
    extends CDCodec
{
    /**
     * Stores the names in this codec. <p>
     *
     * @since 1.0
     */
   
    public String[] sNames = 
    {
	"one",
	"zero"
    };


    /**
     * Constructor. <p>
     *
     * @since 1.0
     */

    public alg1Codec()
    {
	super();

	// System.out.println( "alg1Codec.<init>" );
    }

 
    /**
     * Constructor. <p>
     *
     * @since 1.0
     */
   
    public alg1Codec( Codec fParent )
    {
	super();
	
	mParent = fParent;
	
	// System.out.println( "alg1Codec.<init>" );
    }
  
  
    /**
     * Encodes the given OMObject to Mathematica-syntax. <p>
     *
     * @since 1.0
     */
   
    public String encodeOMObject( OMObject fOMObject )
        throws  CodecEncodeException
    {
	// System.out.println( "alg1Codec.encode" );
	
	if ( fOMObject instanceof OMSymbol )
	{
	    OMSymbol tOMSymbol = (OMSymbol) fOMObject;
	    
	    switch( getIndex( tOMSymbol.getName() ) )
	    {
	        case 0  : return encodeOne( fOMObject );
	        case 1  : return encodeZero( fOMObject );
	    }
	}

	throw new CodecEncodeException( "Unable to encode: " + fOMObject );
    }


    /**
     * Decodes the given Mathematica-syntax to a OMObject. <p>
     *
     * @since 1.0
     */
  
    public OMObject decodeOMObject( String fMathematicaSyntax )
        throws  CodecDecodeException
    {
	// System.out.println( "alg1Codec.decode" );
       
	throw new CodecDecodeException( "Unable to decode: " + 
					fMathematicaSyntax );
    }
  
  
    /**
     * Gets the index for the name. <p>
     *
     * @since 1.0
     */
   
    private int getIndex( String fName )
    {
	// System.out.println( "alg1Codec.getIndex" );
	
	for( int i=0; i<sNames.length; i++ )
	{
	    if ( fName.equals( sNames[i] ) )
		return i;
	}
    
	return -1;
    }
 

    /**
     * Encodes the alg1.one. <p>
     *
     * @since 1.0
     */
   
    public String encodeOne( OMObject fOMObject )
        throws  CodecEncodeException
    {
	// System.out.println( "alg1Codec.encodeOne" );
    
	return "1";
    }


    /**
     * Encodes the alg1.zero. <p>
     *
     * @since 1.0
     */
   
    public String encodeZero( OMObject fOMObject )
        throws  CodecEncodeException
    {
	// System.out.println( "alg1Codec.encodeZero" );
     
	return "0";
    }
}
