/*
 * $Id: bigfloat1Codec.java,v 1.5 2002/11/25 10:22:50 mriem Exp $
 *
 * Copyright (c) 1999-2002, RIACA, Technische Universiteit Eindhoven (TU/e).
 * All Rights Reserved.
 *
 * The contents of this file are subject to the RIACA Public License
 * Version 1.0 (the "License"); you may not use this file except in
 * compliance with the License. A copy of the License is available at
 * http://www.riaca.win.tue.nl/
 */


package mathematicaCodec;

import nl.tue.win.riaca.openmath.codec.*;
import nl.tue.win.riaca.openmath.lang.*;
import java.util.*;


/**
 * A CD-codec that handles the OpenMath 'bigfloat1' CD. <p>
 *
 * @author Manfred Riem (mriem@win.tue.nl)
 * @version 1.0
 */

public class bigfloat1Codec
    extends CDCodec
{
    /**
     * Stores the names in this codec. <p>
     *
     * @since 1.0
     */
   
    public String[] sNames = 
    {
        "bigfloat",
        "bigfloatprec"
    };


    /**
     * Constructor. <p>
     *
     * @since 1.0
     */

    public bigfloat1Codec()
    {
        super();

        // System.out.println( "bigfloat1Codec.<init>" );
    }

 
    /**
     * Constructor. <p>
     *
     * @since 1.0
     */
   
    public bigfloat1Codec( Codec fParent )
    {
        super();
	    
        mParent = fParent;
	
        // System.out.println( "bigfloat1Codec.<init>" );
    }
  
  
    /**
     * Encodes the given OMObject to Mathematica-syntax. <p>
     *
     * @since 1.0
     */
   
    public String encodeOMObject( OMObject fOMObject )
        throws  CodecEncodeException
    {
	// System.out.println( "bigfloat1Codec.encode" );
	    
	OMApplication tApplication = (OMApplication) fOMObject;
	OMObject      tOMObject    = tApplication.getElementAt( 0 );
	    
	if ( tOMObject instanceof OMSymbol )
	{
	    OMSymbol tOMSymbol = (OMSymbol) tOMObject;
	        
	    switch( getIndex( tOMSymbol.getName() ) )
	    {
		case 0 : return encodeBigFloat( fOMObject );
                case 1 : return encodeBigFloatPrec( fOMObject );
            }
	}
    
        throw new CodecEncodeException( "Unable to encode: " + fOMObject );
    }


    /**
     * Decodes the given Mathematica-syntax to a OMObject. <p>
     *
     * @since 1.0
     */
  
    public OMObject decodeOMObject( String fMathematicaSyntax )
        throws  CodecDecodeException
    {
        // System.out.println( "bigfloat1Codec.decode" );
        
        throw new CodecDecodeException( "Unable to decode: " + 
					fMathematicaSyntax );
    }
  
  
    /**
     * Gets the index for the name. <p>
     *
     * @since 1.0
     */
   
    private int getIndex( String fName )
    {
        // System.out.println( "bigfloat1Codec.getIndex" );
        
        for( int i=0; i<sNames.length; i++ )
        {
            if ( fName.equals( sNames[i] ) )
                return i;
        }
        
        return -1;
    }
  

    /**
     * Encodes the bigfloat1.bigfloat. <p>
     *
     * @since 1.0
     */
   
    public String encodeBigFloat( OMObject fOMObject )
        throws  CodecEncodeException
    {
        // System.out.println( "bigfloat1Codec.encodeBigFloat" );
        
        StringBuffer  tResult      = new StringBuffer();
        OMApplication tApplication = (OMApplication) fOMObject;
        
        tResult.append( "Times[" );
        
        if ( tApplication.getElementAt( 1 ) != null )
        {
    	    tResult.append( 
                mParent.encodeOMObject( tApplication.getElementAt( 1 ) ) );
        } 
        else
	    throw new CodecEncodeException( "Unable to encode: " + fOMObject );
	
        tResult.append( ",Power[" );
	
        if ( tApplication.getElementAt( 2 ) != null )
        {
            tResult.append(
                mParent.encodeOMObject( tApplication.getElementAt( 2 ) ) );
        }
        else
	    throw new CodecEncodeException( "Unable to encode: " + fOMObject );
	
        tResult.append( "," );
        
        if ( tApplication.getElementAt( 3 ) != null )
        {
            tResult.append(
                mParent.encodeOMObject( tApplication.getElementAt( 3 ) ) );
        }
        else
	    throw new CodecEncodeException( "Unable to encode: " + fOMObject );
        
        tResult.append( "]" );
        tResult.append( "]" );
        
	return tResult.toString();
    }
    
    
    /**
     * Encodes the bigfloat1.bigfloatprec. <p>
     *
     * @since 1.0
     */
    
    public String encodeBigFloatPrec( OMObject fOMObject )
        throws CodecEncodeException
    {
        // System.out.println( "bigfloat1Codec.encodeBigFloatPrec" );
	
	throw new CodecEncodeException( "Unable to encode: " + fOMObject );
    }
}

