/*
 * $Id: calculus1Codec.java,v 1.7 2002/11/25 10:22:50 mriem Exp $
 *
 * Copyright (c) 1999-2002, RIACA, Technische Universiteit Eindhoven (TU/e).
 * All Rights Reserved.
 *
 * The contents of this file are subject to the RIACA Public License
 * Version 1.0 (the "License"); you may not use this file except in
 * compliance with the License. A copy of the License is available at
 * http://www.riaca.win.tue.nl/
 */


package mathematicaCodec;

import nl.tue.win.riaca.openmath.codec.*;
import nl.tue.win.riaca.openmath.lang.*;
import java.util.*;




/**
 * A CD-codec that handles the OpenMath 'calculus1' CD. <p>
 *
 * @author Manfred Riem (mriem@win.tue.nl)
 * @version 1.1
 */

public class calculus1Codec
    extends CDCodec
{
    /**
     * Stores the names in this codec. <p>
     *
     * @since 1.0
     */
   
    public String[] sNames = 
    {
    	"defint",
    	"diff",
    	"int",
    	"partialdiff"
    };


    /**
     * Constructor. <p>
     *
     * @since 1.0
     */

    public calculus1Codec()
    {
	super();
        
	// System.out.println( "calculus1Codec.<init>" );
    }
    
    
    /**
     * Constructor. <p>
     *
     * @since 1.0
     */
   
    public calculus1Codec( Codec fParent )
    {
	super();
	
	mParent = fParent;
	
	// System.out.println( "calculus1Codec.<init>" );
    }
    
    
    /**
     * Encodes the given OMObject to Mathematica-syntax. <p>
     *
     * @since 1.0
     */
   
    public String encodeOMObject( OMObject fOMObject )
        throws CodecEncodeException
    {
	// System.out.println( "calculus1Codec.encode" );
	
	OMApplication tApplication = (OMApplication) fOMObject;
	OMObject      tOMObject    = tApplication.firstElement();
	
	if ( tOMObject instanceof OMSymbol )
	{
	    OMSymbol tOMSymbol = (OMSymbol) tOMObject;
	    
	    switch( getIndex( tOMSymbol.getName() ) )
	    {
		case 0 : return encodeDefInt( fOMObject );
		case 1 : return encodeDiff( fOMObject );
		case 2 : return encodeInt( fOMObject );
		case 3 : return encodePartialDiff( fOMObject );
	    }
	}
        
        throw new CodecEncodeException( "Unable to encode : " + fOMObject );
    }
    
    
    /**
     * Decodes the given Mathematica-syntax to a OMObject. <p>
     *
     * @since 1.0
     */
  
    public OMObject decodeOMObject( String fMathematicaSyntax )
        throws  CodecDecodeException
    {
	// System.out.println( "calculus1Codec.decode" );
	
        throw new CodecDecodeException( "Unable to decode: " + 
					fMathematicaSyntax );
    }
    
    
    /**
     * Gets the index for the name. <p>
     *
     * @since 1.0
     */
    
    private int getIndex( String fName )
    {
	// System.out.println( "calculus1Codec.getIndex" );
	
	for( int i=0; i<sNames.length; i++ )
	{
	    if ( fName.equals( sNames[i] ) )
		return i;
	}
	
	return -1;
    }
    
    
    /**
     * Encodes the calculus1.defint.
     *
     * @since 1.0
     */
    
    public String encodeDefInt( OMObject fOMObject )
        throws  CodecEncodeException
    {
	// System.out.println( "calculus1Codec.encodeDefInt" );
	
	StringBuffer  tResult      = new StringBuffer();
	OMApplication tApplication = (OMApplication) fOMObject;
	
	tResult.append( "Integrate[" );
	
	if ( tApplication.getElementAt( 2 ) != null )
	{
	    OMBinding tBinding = (OMBinding) tApplication.getElementAt( 2 );
	    
	    tResult.append(
		 mParent.encodeOMObject( tBinding.getBody() ) );
	}
	else
	    throw new CodecEncodeException( 
                "Unable to encode : " + fOMObject.toString() );
	
	tResult.append( ",List[" );
	
	if ( tApplication.getElementAt( 2 ) != null )
	{
	    OMBinding tBinding = (OMBinding) tApplication.getElementAt( 2 );
	    Enumeration tEnum  = tBinding.getVariables().elements();
	    
	    for( ; tEnum.hasMoreElements(); )
	    {
		tResult.append( 
		    mParent.encodeOMObject(
			(OMObject) tEnum.nextElement() ) );

		if ( tEnum.hasMoreElements() )
		    throw new CodecEncodeException( 
                        "Unable to encode : " + fOMObject.toString() );
	    }
	}

	tResult.append( "," );

        if ( tApplication.getElementAt( 1 ) != null )
        {
	    OMApplication tIntervalApplication = 
		(OMApplication) tApplication.getElementAt( 1 );

	    OMSymbol tIntervalSymbol = 
		(OMSymbol) tIntervalApplication.getElementAt( 0 );
 
	    if ( tIntervalSymbol.getCD().equals( "interval1" ) &&
		 tIntervalSymbol.getName().equals( "interval" ) )
	    {
		tResult.append(
		    mParent.encodeOMObject( 
                        tIntervalApplication.getElementAt( 1 ) ) );
	
		tResult.append( "," );

		tResult.append(
		    mParent.encodeOMObject( 
                        tIntervalApplication.getElementAt( 2 ) ) );
	    }
	    else
		throw new CodecEncodeException( 
                    "Unable to encode : " + fOMObject.toString() );
	}
	else
	    throw new CodecEncodeException( 
                "Unable to encode : " + fOMObject.toString() );
	
	tResult.append( "]]" );
	return tResult.toString();
    }


    /**
     * Encodes the calculus1.diff. 
     *
     * @since 1.0
     */
   
    public String encodeDiff( OMObject fOMObject )
        throws  CodecEncodeException
    {
	// System.out.println( "calculus1Codec.encodeDiff" );
     
	StringBuffer  tResult      = new StringBuffer();
	OMApplication tApplication = (OMApplication) fOMObject;
    
	if ( tApplication.getElementAt( 1 ) != null )
	{
	    OMBinding   tBinding = (OMBinding) tApplication.getElementAt( 1 );
	    Enumeration tEnum    = tBinding.getVariables().elements();
	    
	    tResult.append( "Function[List[" );
	
	    for( ; tEnum.hasMoreElements(); )
	    {
		tResult.append( 
		   mParent.encodeOMObject(
		       (OMObject) tEnum.nextElement() ) );
	    }
	
	    tResult.append( "], Evaluate[" );
	    tResult.append( "D[" );

	    tResult.append(
		 mParent.encodeOMObject( tBinding.getBody() ) );

	    tResult.append( "," );

	    tEnum = tBinding.getVariables().elements();
	    
	    for( ; tEnum.hasMoreElements(); )
	    {
		tResult.append( 
		    mParent.encodeOMObject(
			(OMObject) tEnum.nextElement() ) );

		if ( tEnum.hasMoreElements() )
		    throw new CodecEncodeException( 
                        "Unable to encode : " + fOMObject.toString() );
	    }
	    
	    tResult.append( "]]]" );
	}
	else
	    throw new CodecEncodeException( 
                "Unable to encode : " + fOMObject.toString() );

	return tResult.toString();
    }


    /**
     * Encodes the calculus1.int.
     *
     * @since 1.0
     */
   
    public String encodeInt( OMObject fOMObject )
        throws CodecEncodeException
    {
	System.out.println( "calculus1Codec.encodeInt" );
     
	StringBuffer  tResult      = new StringBuffer();
	OMApplication tApplication = (OMApplication) fOMObject;
    
	if ( tApplication.getElementAt( 0 ) != null )
	{
	    OMBinding   tBinding = (OMBinding) tApplication.getElementAt( 1 );
            Enumeration tEnum    = tBinding.getVariables().elements();
	    
	    tResult.append( "Function[List[" );
	    
	    for( ; tEnum.hasMoreElements(); )
	    {
		tResult.append( 
		    mParent.encodeOMObject(
		        (OMObject) tEnum.nextElement() ) );
	    }
	    
	    tResult.append( "], Evaluate[" );
	    tResult.append( "Integrate[" );
            
	    tResult.append(
	        mParent.encodeOMObject( tBinding.getBody() ) );
            
	    tResult.append( "," );
	    
	    tEnum = tBinding.getVariables().elements();
	    
	    for( ; tEnum.hasMoreElements(); )
	    {
		tResult.append( 
		    mParent.encodeOMObject(
		        (OMObject) tEnum.nextElement() ) );
                
		if ( tEnum.hasMoreElements() )
		    throw new CodecEncodeException( 
                        "Unable to encode : " + fOMObject.toString() );
	    }
            
	    tResult.append( "]]]" );
	}
	else
	    throw new CodecEncodeException( 
                "Unable to encode : " + fOMObject.toString() );
	
	return tResult.toString();
    }
    
    
    /**
     * Encodes the calculus1.partialdiff.
     *
     * @since 1.0
     */
    
    public String encodePartialDiff( OMObject fOMObject )
        throws  CodecEncodeException
    {
	// System.out.println( "calculus1Codec.encodePartialDiff" );
	
	StringBuffer  tResult      = new StringBuffer();
	OMApplication tApplication = (OMApplication) fOMObject;
    	
	if ( tApplication.getElementAt( 2 ) != null )
	{
	    OMBinding   tBinding = (OMBinding) tApplication.getElementAt( 2 );
	    Enumeration tEnum    = tBinding.getVariables().elements();
	    
	    tResult.append( "Function[List[" );
	    
	    for( ; tEnum.hasMoreElements(); )
	    {
		tResult.append( 
		   mParent.encodeOMObject(
		       (OMObject) tEnum.nextElement() ) );
		
		if ( tEnum.hasMoreElements() )
		    tResult.append( "," );
	    }
	    
	    tResult.append( "], Evaluate[" );
	    tResult.append( "D[" );
	    
	    tResult.append(
		 mParent.encodeOMObject( tBinding.getBody() ) );
	    
	    tResult.append( "," );
	    
	    OMApplication tList = 
		(OMApplication) tApplication.getElementAt( 1 ); 
	    
	    int tInteger = 1;
	    int tIndex   = 1;
	    
	    while( tList.getElementAt( tIndex  ) != null )
	    {
		OMInteger tOMInteger = 
		    (OMInteger) tList.getElementAt( tIndex );
		
		tInteger = new Integer( 
                      tOMInteger.getInteger() ).intValue();
		
		int tEnumIndex = 1;
		tEnum = tBinding.getVariables().elements();
		
		for(  ; tEnum.hasMoreElements(); )
		{
		    if ( tInteger == tEnumIndex )
		    {
			tResult.append( 
			    mParent.encodeOMObject(
			        (OMObject) tEnum.nextElement() ) );
			break;
		    }
		    else
		    {
			tEnum.nextElement();
			tEnumIndex++;
		    }
		   
		}
		
       tIndex++;
		
       if ( tList.getElementAt( tIndex) != null )
		    tResult.append( "," );
	    }
	}
	
	tResult.append( "]]]" );
	return tResult.toString();
    }
    
    
    /**
     * Decodes the Mathematica (Integrate) expression. <p>
     *
     * @since 1.0
     */
    
    public OMObject decodeIntegrate( String fMathematicaSyntax )
        throws  CodecDecodeException
    {
	// System.out.println( "calculus1Codec.decodeIntegrate" );
        
        throw new CodecDecodeException( 
	    "Unable to decode: " + fMathematicaSyntax );
    }
}
