/*
 * $Id: fns1Codec.java,v 1.5 2002/11/20 23:51:51 mriem Exp $
 *
 * Copyright (c) 1999-2002, RIACA, Technische Universiteit Eindhoven (TU/e).
 * All Rights Reserved.
 *
 * The contents of this file are subject to the RIACA Public License
 * Version 1.0 (the "License"); you may not use this file except in
 * compliance with the License. A copy of the License is available at
 * http://www.riaca.win.tue.nl/
 */


package mathematicaCodec;

import nl.tue.win.riaca.openmath.codec.*;
import nl.tue.win.riaca.openmath.lang.*;
import java.util.*;


/**
 * A CD-codec that handles the OpenMath 'fns1' CD. <p>
 *
 * @author Ernesto Reinaldo Barreiro (ereinald@win.tue.nl)
 * @author Manfred Riem (mriem@win.tue.nl)
 * @version 1.0
 */

public class fns1Codec
    extends CDCodec
{
    /**
     * Stores the names in this codec. <p>
     *
     * @since 1.0
     */
   
    public String[] sNames = 
    {
        "domain",
        "identity",
        "image",
        "inverse",
        "lambda",
        "left_compose",
        "range"
    };


    /**
     * Stores the Mathematica names. <p>
     *
     * @since 1.0
     */

    public String[] sMathNames =
    {
        "Function"
    };


    /**
     * Constructor. <p>
     *
     * @since 1.0
     */

    public fns1Codec()
    {
        super();
        
        // System.out.println( "fns1Codec.<init>" );
    }

 
    /**
     * Constructor. <p>
     *
     * @since 1.0
     */
   
    public fns1Codec( Codec fParent )
    {
        super();
	    
        mParent = fParent;
	
        // System.out.println( "fns1Codec.<init>" );
    }
  
  
    /**
     * Encodes the given OMObject to Mathematica-syntax. <p>
     *
     * @since 1.0
     */
   
    public String encodeOMObject( OMObject fOMObject )
        throws  CodecEncodeException
    {
        // System.out.println( "fns1Codec.encode" );
        	
        if ( fOMObject instanceof OMApplication )
        {
            OMApplication tApplication = (OMApplication) fOMObject;
            OMObject      tOMObject    = tApplication.getElementAt( 0 );
	
            if ( tOMObject instanceof OMSymbol )
            {
                OMSymbol tOMSymbol = (OMSymbol) tOMObject;
	    
                switch( getIndex( tOMSymbol.getName() ) )
                {
                    case 0 : return encodeDomain( fOMObject );
                    case 1 : return encodeIdentity( fOMObject );
                    case 2 : return encodeImage( fOMObject );
                    case 3 : return encodeInverse( fOMObject );
                    case 5 : return encodeLeftCompose( fOMObject );
                    case 6 : return encodeRange( fOMObject );
                }
            }
        }
        
        if ( fOMObject instanceof OMBinding )
        {
            OMBinding tOMBinding = (OMBinding) fOMObject;
            OMObject  tOMObject  = tOMBinding.getBinder();
	    
            if ( tOMObject instanceof OMSymbol )
            {
                OMSymbol tOMSymbol = (OMSymbol) tOMObject;
	    
                switch( getIndex( tOMSymbol.getName() ) )
                {
                    case 4 : return encodeLambda( fOMObject );
                }
            }
        }
    
        throw new CodecEncodeException( "Unable to encode: " + 
            fOMObject.toString() );
    }


    /**
     * Decodes the given Mathematica-syntax to a OMObject. <p>
     *
     * @since 1.0
     */
  
    public OMObject decodeOMObject( String fMathematicaSyntax )
        throws  CodecDecodeException
    {
        // System.out.println( "fns1Codec.decode" );
    
        String tOperator = 
	    fMathematicaSyntax.substring( 
	        0, fMathematicaSyntax.indexOf( '[' ) );

        String tArguments =
	    fMathematicaSyntax.substring( 
	        fMathematicaSyntax.indexOf( '[' ) );
        
	if ( tOperator != null )
	{
	    tOperator = tOperator.trim();
            
	    switch( getMathIndex( tOperator ) )
	    {
		case 0  : return decodeFunction( tArguments );
		default : break;
	    }
	}
	
        throw new CodecDecodeException( 
            "Unable to decode: " + fMathematicaSyntax );
    }
  
  
    /**
     * Gets the index for the name. <p>
     *
     * @since 1.0
     */
   
    private int getIndex( String fName )
    {
        // System.out.println( "fns1Codec.getIndex" );
	    
	for( int i=0; i<sNames.length; i++ )
	{
	    if ( fName.equals( sNames[i] ) )
		return i;
	}
        
	return -1;
    }
    
    
    /**
     * Gets the index for the Mathematica operator. <p>
     *
     * @since 1.0
     */
   
    private int getMathIndex( String fOperator )
    {
	// System.out.println( "arith1Codec.getMathIndex" );
	
	for( int i=0; i<sNames.length; i++ )
        {
	    if ( fOperator.equals( sMathNames[i] ) )
		return i;
	}
        
	return -1;
    }
    
    
    /**
     * Encodes the fns1.domain.
     *
     * @since 1.0
     */
   
    public String encodeDomain( OMObject fOMObject )
        throws  CodecEncodeException
    {
	// System.out.println( "fns1Codec.encodeDomain" );

        throw new CodecEncodeException( 
	    "Unable to encode: " + fOMObject.toString() );
    }
    
    
    /**
     * Encodes the fns1.identity. 
     *
     * @since 1.0
     */
    
    public String encodeIdentity( OMObject fOMObject )
        throws  CodecEncodeException
    {
	// System.out.println( "fns1Codec.encodeIdentity" );
	
        throw new CodecEncodeException( 
            "Unable to encode: " + fOMObject.toString() );
    }
    
    
    /**
     * Encodes the fns1.image 
     *
     * @since 1.0
     */
    
    public String encodeImage( OMObject fOMObject )
        throws  CodecEncodeException
    {
        // System.out.println( "fns1Codec.encodeImage" );
     
        throw new CodecEncodeException( 
            "Unable to encode: " + fOMObject.toString() );
    }
    
    
    /**
     * Encodes the fns1.inverse 
     *
     * @since 1.0
     */
    
    public String encodeInverse( OMObject fOMObject )
        throws  CodecEncodeException
    {
	// System.out.println( "fns1Codec.encodeInverse" );
     
        throw new CodecEncodeException( 
            "Unable to encode: " + fOMObject.toString() );
    }


    /**
     * Encodes the fns1.lambda.
     *
     * @since 1.0
     */
    
    public String encodeLambda( OMObject fOMObject )
        throws CodecEncodeException
    {
        // System.out.println( "fns1Codec.encodeLambda" );
        
        OMBinding tOMBinding = (OMBinding) fOMObject;
        OMSymbol  tSymbol    = (OMSymbol)  tOMBinding.getBinder();
        
        StringBuffer tResult = new StringBuffer();
        
        tResult.append( "Function[" );
        
        Enumeration  tEVariables = tOMBinding.getVariables().elements();
        StringBuffer tVarStr     = new StringBuffer();
                
        int tIndex = 0;
        
        while ( tEVariables.hasMoreElements() )
        {
            OMVariable tVar = (OMVariable) tEVariables.nextElement();
            
            if ( tIndex > 0 )
                tVarStr.append(", ");
            
            tVarStr.append( tVar.getName() );            
            
            tIndex++;            
        }
        
        if ( tIndex > 1 )
            tResult.append( "{" + tVarStr + "}, " );
        else
            tResult.append( tVarStr.toString() + ", " );
        
        tResult.append( mParent.encodeOMObject( tOMBinding.getBody() ) );
        tResult.append( "]" );
        
        return tResult.toString();
    }


    /**
     * Encodes the fns1.left_compose. 
     *
     * @since 1.0
     */
   
    public String encodeLeftCompose( OMObject fOMObject )
        throws  CodecEncodeException
    {
        // System.out.println( "fns1Codec.encodeLeftCompose" );
        
        throw new CodecEncodeException( 
            "Unable to encode: " + fOMObject.toString() );
    }


    /**
     * Encodes the fns1.range. 
     *
     * @since 1.0
     */
   
    public String encodeRange( OMObject fOMObject )
        throws  CodecEncodeException
    {
        // System.out.println( "fns1Codec.encodeRange" );
        
        throw new CodecEncodeException( 
            "Unable to encode: " + fOMObject.toString() );
    }
    
    
    /**
     * Decodes the Mathematica (Function) expression. <p>
     *
     * @since 1.0
     */

    public OMObject decodeFunction( String fMathematicaSyntax )
        throws  CodecDecodeException
    {
        // System.out.println( "fns1Codec.decodeFunction" );
        
	OMBinding tOMBinding = new OMBinding();
       
	tOMBinding.setBinder( new OMSymbol( "fns1", "lambda" ) );
	
    	MathematicaCodec tParentCodec = (MathematicaCodec) mParent;
	Vector tVector = tParentCodec.parseArguments( fMathematicaSyntax );
        
    	if ( tVector.size() == 2 )
	{
	    Vector tVarVector = new Vector();
    	    String tVarString = (String) tVector.elementAt( 0 );
	    
	    /*
    	     * Create and fill the vector for the OMBVARs.
    	     */
	    
	    Vector tTempVector = tParentCodec.parseArguments( tVarString );
	    Enumeration tTempEnum = tTempVector.elements();
	    
    	    for( ; tTempEnum.hasMoreElements(); )
	    {
		tVarVector.addElement(
		    tParentCodec.decodeOMObject(
		        (String) tTempEnum.nextElement() ) );
    	    }
            
	    OMObject tOMObject = 
		tParentCodec.decodeOMObject( 
                    (String) tVector.elementAt( 0 ) );
            
    	    tOMBinding.setVariables( tVarVector );
	    
	    /*
    	     * Add the body. 
    	     */
	    
	    tOMObject =
		tParentCodec.decodeOMObject(
		    (String) tVector.elementAt( 1 ) );
	    
	    tOMBinding.setBody( tOMObject );
    	}
	else
            throw new CodecDecodeException( 
	        "Unable to decode: " + fMathematicaSyntax );
	
	return tOMBinding;
    }
}
