/*
 * $Id: interval1Codec.java,v 1.3 2002/08/16 08:32:32 mriem Exp $
 *
 * Copyright (c) 1999-2002, RIACA, Technische Universiteit Eindhoven (TU/e).
 * All Rights Reserved.
 *
 * The contents of this file are subject to the RIACA Public License
 * Version 1.0 (the "License"); you may not use this file except in
 * compliance with the License. A copy of the License is available at
 * http://www.riaca.win.tue.nl/
 */


package mathematicaCodec;

import nl.tue.win.riaca.openmath.codec.*;
import nl.tue.win.riaca.openmath.lang.*;
import java.util.*;


/**
 * A CD-codec that handles the OpenMath 'interval1' CD. <p>
 *
 * @author Manfred Riem (mriem@win.tue.nl)
 * @version 1.0
 */

public class interval1Codec
    extends CDCodec
{
    /**
     * Stores the names in this codec. <p>
     *
     * @since 1.0
     */
   
    public String[] sNames = 
    {
    	"integer_interval",
    	"interval",
    	"interval_cc",
    	"interval_co",
    	"interval_oc",
    	"interval_oo"
    };


    /**
     * Constructor. <p>
     *
     * @since 1.0
     */

    public interval1Codec()
    {
        super();
        
    	// System.out.println( "interval1Codec.<init>" );
    }

 
    /**
     * Constructor. <p>
     *
     * @since 1.0
     */
   
    public interval1Codec( Codec fParent )
    {
	super();
	
    	mParent = fParent;
	    
    	// System.out.println( "interval1Codec.<init>" );
    }
  
  
    /**
     * Encodes the given OMObject to Mathematica-syntax. <p>
     *
     * @since 1.0
     */
   
    public String encodeOMObject( OMObject fOMObject )
        throws  CodecEncodeException
    {
	// System.out.println( "interval1Codec.encode" );
	    
    	if ( fOMObject instanceof OMSymbol )
	{
	    OMSymbol tOMSymbol = (OMSymbol) fOMObject;
	    
    	    switch( getIndex( tOMSymbol.getName() ) )
	    {
		case 0 : return encodeIntegerInterval( fOMObject );
    	        case 1 : return encodeInterval( fOMObject );
    	        case 2 : return encodeIntervalCC( fOMObject );
    	        case 3 : return encodeIntervalCO( fOMObject );
    	        case 4 : return encodeIntervalOC( fOMObject );
    	        case 5 : return encodeIntervalOO( fOMObject );
    	    }
	}
    
        throw new CodecEncodeException( 
            "Unable to encode: " + fOMObject.toString()  );
    }
    
    
    /**
     * Decodes the given Mathematica-syntax to a OMObject. <p>
     *
     * @since 1.0
     */
    
    public OMObject decodeOMObject( String fMathematicaSyntax )
        throws  CodecDecodeException
    {
    	// System.out.println( "interval1Codec.decode" );
	
        throw new CodecDecodeException( 
            "Unable to decode: " + fMathematicaSyntax );
    }
    
    
    /**
     * Gets the index for the name. <p>
     *
     * @since 1.0
     */
    
    private int getIndex( String fName )
    {
    	// System.out.println( "interval1Codec.getIndex" );
	    
    	for( int i=0; i<sNames.length; i++ )
	{
	    if ( fName.equals( sNames[i] ) )
		return i;
    	}
	
    	return -1;
    }
    
    
    /**
     * Encodes the interval1.integer_interval. <p>
     *
     * @since 1.0
     */
    
    public String encodeIntegerInterval( OMObject fOMObject )
        throws  CodecEncodeException
    {
	// System.out.println( "interval1Codec.encodeIntegerInterval" );
	
        throw new CodecEncodeException( 
            "Unable to encode: " + fOMObject.toString() );
    }


    /**
     * Encodes the interval1.interval. <p>
     *
     * @since 1.0
     */
   
    public String encodeInterval( OMObject fOMObject )
        throws CodecEncodeException
    {
    	// System.out.println( "interval1Codec.encodeInterval" );
     
        throw new CodecEncodeException( 
            "Unable to encode: " + fOMObject.toString() );
    }


    /**
     * Encodes the interval1.interval_cc. <p>
     *
     * @since 1.0
     */
   
    public String encodeIntervalCC( OMObject fOMObject )
        throws  CodecEncodeException
    {
    	// System.out.println( "interval1Codec.encodeIntervalCC" );
     
        throw new CodecEncodeException( 
            "Unable to encode: " + fOMObject.toString() );
    }


    /**
     * Encodes the interval1.interval_co. <p>
     *
     * @since 1.0
     */
   
    public String encodeIntervalCO( OMObject fOMObject )
        throws  CodecEncodeException
    {
    	// System.out.println( "interval1Codec.encodeIntervalCO" );
     
        throw new CodecEncodeException( 
            "Unable to encode: " + fOMObject.toString() );
    }


    /**
     * Encodes the interval1.interval_oc. <p>
     *
     * @since 1.0
     */
   
    public String encodeIntervalOC( OMObject fOMObject )
        throws  CodecEncodeException
    {
    	// System.out.println( "interval1Codec.encodeIntervalOC" );
     
        throw new CodecEncodeException( 
            "Unable to encode: " + fOMObject.toString() );
    }


    /**
     * Encodes the interval1.interval_oo. <p>
     *
     * @since 1.0
     */
   
    public String encodeIntervalOO( OMObject fOMObject )
        throws  CodecEncodeException
    {
	    // System.out.println( "interval1Codec.encodeIntervalOO" );
         
        throw new CodecEncodeException( 
            "Unable to encode: " + fOMObject.toString() );
    }
}
