/*
 * $Id: limit1Codec.java,v 1.4 2002/11/20 23:51:51 mriem Exp $
 *
 * Copyright (c) 1999-2002, RIACA, Technische Universiteit Eindhoven (TU/e).
 * All Rights Reserved.
 *
 * The contents of this file are subject to the RIACA Public License
 * Version 1.0 (the "License"); you may not use this file except in
 * compliance with the License. A copy of the License is available at
 * http://www.riaca.win.tue.nl/
 */


package mathematicaCodec;

import nl.tue.win.riaca.openmath.codec.*;
import nl.tue.win.riaca.openmath.lang.*;
import java.util.*;


/**
 * A CD-codec that handles the OpenMath 'limit1' CD. <p>
 *
 * @author Manfred Riem (mriem@win.tue.nl)
 * @version 1.0
 */

public class limit1Codec
    extends CDCodec
{
    /**
     * Stores the names in this codec. <p>
     *
     * @since 1.0
     */
   
    public String[] sNames = 
    {
	"above",
	"below",
	"both_sides",
	"limit",
	"null"
    };


    /**
     * Constructor. <p>
     *
     * @since 1.0
     */

    public limit1Codec()
    {
	super();

	// System.out.println( "limit1Codec.<init>" );
    }

 
    /**
     * Constructor. <p>
     *
     * @since 1.0
     */
   
    public limit1Codec( Codec fParent )
    {
	super();
	
	mParent = fParent;
	
	// System.out.println( "limit1Codec.<init>" );
    }
  
  
    /**
     * Encodes the given OMObject to Mathematica-syntax. <p>
     *
     * @since 1.0
     */
   
    public String encodeOMObject( OMObject fOMObject )
        throws  CodecEncodeException
    {
	// System.out.println( "limit1Codec.encode" );
	
        if ( fOMObject instanceof OMApplication )
        {
            OMApplication tOMApplication = (OMApplication) fOMObject;
            
            if ( tOMApplication.getElementAt( 0 ) instanceof OMSymbol )
            {
                OMSymbol tOMSymbol = (OMSymbol) tOMApplication.getElementAt( 0 );
	    
                switch( getIndex( tOMSymbol.getName() ) )
                {
                    case 0 :
                        return encodeAbove( fOMObject );
                    case 1 :
                        return encodeBelow( fOMObject );
                    case 2 :
                        return encodeBothSides( fOMObject );
                    case 3 :
                        return encodeLimit( fOMObject );
                    case 4 :
                        return encodeNull( fOMObject );
                }
            }
        }
    
        throw new CodecEncodeException( 
            "Unable to encode: " + fOMObject );
    }


    /**
     * Decodes the given Mathematica-syntax to a OMObject. <p>
     *
     * @since 1.0
     */
  
    public OMObject decodeOMObject( String fMathematicaSyntax )
        throws  CodecDecodeException
    {
	// System.out.println( "limit1Codec.decode" );
       
        throw new CodecDecodeException( 
            "Unable to decode: " + fMathematicaSyntax );
    }
  
  
    /**
     * Gets the index for the name. <p>
     *
     * @since 1.0
     */
   
    private int getIndex( String fName )
    {
	// System.out.println( "limit1Codec.getIndex" );
	
	for( int i=0; i<sNames.length; i++ )
	{
	    if ( fName.equals( sNames[i] ) )
		return i;
	}
    
	return -1;
    }
 

    /**
     * Encodes the limit1.above. <p>
     *
     * @since 1.0
     */
   
    public String encodeAbove( OMObject fOMObject )
        throws  CodecEncodeException
    {
	// System.out.println( "limit1Codec.encodeAbove" );
    
        throw new CodecEncodeException( 
            "Unable to encode: " + fOMObject );
    }


    /**
     * Encodes the limit1.below. <p>
     *
     * @since 1.0
     */
   
    public String encodeBelow( OMObject fOMObject )
        throws  CodecEncodeException
    {
	// System.out.println( "limit1Codec.encodeBelow" );
     
        throw new CodecEncodeException( 
            "Unable to encode: " + fOMObject );
    }


    /**
     * Encodes the limit1.both_sides. <p>
     *
     * @since 1.0
     */
   
    public String encodeBothSides( OMObject fOMObject )
        throws  CodecEncodeException
    {
	// System.out.println( "limit1Codec.encodeBothSides" );
     
        throw new CodecEncodeException( 
            "Unable to encode: " + fOMObject );
    }


    /**
     * Encodes the limit1.limit. <p>
     *
     * @since 1.0
     */
   
    public String encodeLimit( OMObject fOMObject )
        throws  CodecEncodeException
    {
	// System.out.println( "limit1Codec.encodeLimit" );
     
	StringBuffer tResult = new StringBuffer();
	OMApplication tApplication = (OMApplication) fOMObject;
        int tIndex = 1;
        
	tResult.append( "Limit[" );
        
        /*
         * Get the function. 
         */
        
        OMBinding tBinding = (OMBinding) tApplication.getElementAt( 3 );
        OMObject  tBObject = tBinding.getBody();
        
        tResult.append (
            mParent.encodeOMObject( tBObject ) );
        
        tResult.append( "," );
        
        /*
         * To where with which variable 
         */
        
        tResult.append( "Rule[" );
        
        Enumeration tEnum = tBinding.getVariables().elements();
        
        for( ; tEnum.hasMoreElements(); )
        {           
            tResult.append(
                mParent.encodeOMObject( (OMObject) tEnum.nextElement() ) );

            if ( tEnum.hasMoreElements() )
                throw new CodecEncodeException( "Invalid format" );
        }

        tResult.append( ", " );
        
        tResult.append( 
            mParent.encodeOMObject( tApplication.getElementAt( 1 ) ) );

        tResult.append( "]" );
        
	OMSymbol tDirection = (OMSymbol) tApplication.getElementAt( 2 );
        
        if ( tDirection.getName().equals( "above" ) )
            tResult.append( ",Rule[Direction, -1]" );
        
        if ( tDirection.getName().equals( "below" ) )
            tResult.append( ",Rule[Direction, 1]" );
        
        if ( tDirection.getName().equals( "both_sides" ) )
        {
        }
        
        if ( tDirection.getName().equals( "null" ) )
        {
        }
        
	tResult.append( "]" );
	return tResult.toString();
    }


    /**
     * Encodes the limit1.null. <p>
     *
     * @since 1.0
     */
   
    public String encodeNull( OMObject fOMObject )
        throws  CodecEncodeException
    {
	// System.out.println( "limit1Codec.encodeNull" );
     
        throw new CodecEncodeException( 
            "Unable to encode: " + fOMObject );
    }
}
