/*
 * $Id: linalg2Codec.java,v 1.4 2002/11/20 23:51:51 mriem Exp $
 *
 * Copyright (c) 1999-2002, RIACA, Technische Universiteit Eindhoven (TU/e).
 * All Rights Reserved.
 *
 * The contents of this file are subject to the RIACA Public License
 * Version 1.0 (the "License"); you may not use this file except in
 * compliance with the License. A copy of the License is available at
 * http://www.riaca.win.tue.nl/
 */


package mathematicaCodec;

import java.util.Enumeration;
import java.util.Vector;

import nl.tue.win.riaca.openmath.codec.CDCodec;
import nl.tue.win.riaca.openmath.codec.Codec;
import nl.tue.win.riaca.openmath.codec.CodecDecodeException;
import nl.tue.win.riaca.openmath.codec.CodecEncodeException;
import nl.tue.win.riaca.openmath.lang.OMApplication;
import nl.tue.win.riaca.openmath.lang.OMObject;
import nl.tue.win.riaca.openmath.lang.OMSymbol;


/**
 * A CD-codec that handles the OpenMath 'linalg2' CD. <p>
 *
 * @author Manfred Riem
 * @version 1.0
 */

public class linalg2Codec
    extends CDCodec
{
    /**
     * Stores the names in this codec. <p>
     *
     * @since 1.0
     */
   
    public String[] sNames = 
    {
	"matrix",
	"matrixrow",
	"vector"
    };
    /**
     * Stores the Mathematica equivalents to the OpenMath symbols. <p>
     *
     * @since 1.0
     */

    public String[] sMathNames =
    {
    		"MatrixForm",
    		"List"
    };


    /**
     * Constructor. <p>
     *
     * @since 1.0
     */

    public linalg2Codec()
    {
	super();

	// System.out.println( "linalg2Codec.<init>" );
    }

 
    /**
     * Constructor. <p>
     *
     * @since 1.0
     */
   
    public linalg2Codec( Codec fParent )
    {
	super();
	
	mParent = fParent;
	
	// System.out.println( "linalg2Codec.<init>" );
    }
  
  
    /**
     * Encodes the given OMObject to Mathematica-syntax. <p>
     *
     * @since 1.0
     */
   
    public String encodeOMObject( OMObject fOMObject )
        throws  CodecEncodeException
    {
	// System.out.println( "linalg2Codec.encode" );
	
	if ( fOMObject instanceof OMApplication )
	{
            OMApplication tOMApplication = (OMApplication) fOMObject;
            
            if ( tOMApplication.getElementAt( 0 ) instanceof OMSymbol )
            {
                OMSymbol tOMSymbol = (OMSymbol) tOMApplication.getElementAt( 0 );
	    
                switch( getIndex( tOMSymbol.getName() ) )
                {
                    case 0 :
                        return encodeMatrix( fOMObject );
                    case 1 :
                        return encodeMatrixRow( fOMObject );
                    case 2 :
                        return encodeVector( fOMObject );

                }
	    }
	}
    
        throw new CodecEncodeException( 
            "Unable to encode: " + fOMObject );
    }
  
  
  
  
  
    /**
     * Gets the index for the name. <p>
     *
     * @since 1.0
     */
   
    private int getIndex( String fName )
    {
	// System.out.println( "linalg2Codec.getIndex" );
	
	for( int i=0; i<sNames.length; i++ )
	{
	    if ( fName.equals( sNames[i] ) )
		return i;
	}
    
	return -1;
    }
 

    /**
     * Encodes the linalg2.matrix. <p>
     *
     * @since 1.0
     */
   
    public String encodeMatrix( OMObject fOMObject )
        throws  CodecEncodeException
    {
	// System.out.println( "linalg2Codec.encodeMatrix" );
    
	StringBuffer tResult = new StringBuffer();
	OMApplication tApplication = (OMApplication) fOMObject;
	Enumeration   tEnum        = tApplication.getElements().elements();
	/*System.out.print("\n");
	System.out.print(tApplication.getElementAt(0));
	System.out.print("\n");
    System.out.print(tApplication.getElementAt(1));
    System.out.print("\n");
	System.out.print(tApplication.getElementAt(2));
	System.out.print("\n");*/
	
	
	tResult.append( "MatrixForm[List[" );
	if ( tApplication.getLength()>= 3 )
	{
	    tEnum.nextElement();
	    
	    for( ; tEnum.hasMoreElements(); )
	    {
		tResult.append( 
		    mParent.encodeOMObject( (OMObject) tEnum.nextElement() ) );
		
		if ( tEnum.hasMoreElements() )
		    tResult.append( "," );
	    }	

	tResult.append( "]]" );
	return tResult.toString();
}
	
	throw new CodecEncodeException( "Unable to encode: " + fOMObject );
   }

    /**
     * Encodes the linalg2.matrixrow. <p>
     *
     * @since 1.0
     */
   
    public String encodeMatrixRow( OMObject fOMObject )
        throws  CodecEncodeException
    {
	// System.out.println( "linalg2Codec.encodeMatrixRow" );
     
	StringBuffer tResult = new StringBuffer();
	OMApplication tApplication = (OMApplication) fOMObject;
	Enumeration   tEnum        = tApplication.getElements().elements();
       /* System.out.print("\n        	        System.out.println(nbocurrence("List",maToMa));
");
        System.out.print(tApplication.getLength());
    	System.out.print("\n");
    	System.out.print(tApplication.getElementAt(0));
    	System.out.print("\n");
        System.out.print(tApplication.getElementAt(1));
        System.out.print("\n");
    	System.out.print(tApplication.getElementAt(2));
    	System.out.print("\n");*/
	tResult.append( "List[" );
	
	if ( tApplication.getLength() >= 3 )
	{
	    tEnum.nextElement();
	    
	    for( ; tEnum.hasMoreElements(); )
	    {
		tResult.append( 
		    mParent.encodeOMObject( (OMObject) tEnum.nextElement() ) );
		
		if ( tEnum.hasMoreElements() )
		    tResult.append( "," );
	    }	

	tResult.append( "]" );
	return tResult.toString();    
}
	
	throw new CodecEncodeException( "Unable to encode: " + fOMObject );
   }

    /**
     * Encodes the linalg2.vector. <p>
     *
     * @since 1.0
     */
   
    public String encodeVector( OMObject fOMObject )
        throws  CodecEncodeException
    {
	// System.out.println( "linalg2Codec.encodeVector" );
     
	StringBuffer tResult = new StringBuffer();
	OMApplication tApplication = (OMApplication) fOMObject;
        int tIndex = 1;
        
	tResult.append( "Transpose[List[" );
	
        while( tApplication.getElementAt( tIndex ) != null )
        {
            tResult.append( 
                mParent.encodeOMObject( tApplication.getElementAt( tIndex ) ) );
            
            tIndex++;
            
            if ( tApplication.getElementAt( tIndex ) != null )
                tResult.append( "," );
	}

	tResult.append( "]]" );
	return tResult.toString();
    }
    
    /**
     * Decodes the Mathematica (MatrixForm) expression. <p>
     *
     * @since 1.0
     */

    public OMObject decodeMatrixForm( String fMathematicaSyntax )
        throws  CodecDecodeException
        
    {
	// System.out.println( "linalg2Codec.decodeDet" );
    	if (fMathematicaSyntax.contentEquals("{") != false) 
    	{fMathematicaSyntax= fMathematicaSyntax.replaceAll("}", "");
    	fMathematicaSyntax= fMathematicaSyntax.replaceFirst("{", "");}
    	else
    	{
    	int indexlist= fMathematicaSyntax.indexOf("List");	
    	//fMathematicaSyntax= fMathematicaSyntax.replaceFirst("List", "");
    	System.out.println( fMathematicaSyntax);
    	//int indexbrac= fMathematicaSyntax.indexOf("[", indexlist);	
    	//fMathematicaSyntax= fMathematicaSyntax.substring(1, fMathematicaSyntax.length()-1);
    	//System.out.println("ff"+ fMathematicaSyntax);
    	}
    	
       	fMathematicaSyntax.trim();
    	
	OMApplication tOMApplication = new OMApplication();
       
	tOMApplication.addElement( new OMSymbol( "linalg2", "matrix" ) );
	
	MathematicaCodec tParentCodec = (MathematicaCodec) mParent;
	Vector tVector = tParentCodec.parseArguments( fMathematicaSyntax );
	Enumeration tEnum = tVector.elements();
	for( ; tEnum.hasMoreElements(); )
	{
	    OMObject tOMObject = 
		tParentCodec.decodeOMObject( 
                    (String) tEnum.nextElement() );
 
	    if ( tOMObject != null )
		tOMApplication.addElement( tOMObject );
	    else
		throw new CodecDecodeException( "Unable to decode: " + 
						fMathematicaSyntax );
	}
   
	return tOMApplication;
    }
    /**
     * Decodes the Mathematica (List) expression. <p>
     *
     * @since 1.0
     */

    public OMObject decodeList( String fMathematicaSyntax )
        throws  CodecDecodeException
    {
	// System.out.println( "linalg2Codec.decodeDet" );

	OMApplication tOMApplication = new OMApplication();
       
	tOMApplication.addElement( new OMSymbol( "linalg2", "matrixrow" ) );
	
	MathematicaCodec tParentCodec = (MathematicaCodec) mParent;
	Vector tVector = tParentCodec.parseArguments( fMathematicaSyntax );
	Enumeration tEnum = tVector.elements();
	for( ; tEnum.hasMoreElements(); )
	{
	    OMObject tOMObject = 
		tParentCodec.decodeOMObject( 
                    (String) tEnum.nextElement() );
 
	    if ( tOMObject != null )
		tOMApplication.addElement( tOMObject );
	    else
		throw new CodecDecodeException( "Unable to decode: " + 
						fMathematicaSyntax );
	}
   
	return tOMApplication;
    }
    /**
     * Decodes the given Mathematica-syntax to a OMObject. <p>
     *
     * @since 1.0
     */
  
    public OMObject decodeOMObject( String fMathematicaSyntax )
        throws CodecDecodeException
    {
	// System.out.println( "arith1Codec.decode" );
	
	String tOperator = 
	    fMathematicaSyntax.substring( 
                0, fMathematicaSyntax.indexOf( '[' ) );
	
	String tArguments =
	    fMathematicaSyntax.substring( fMathematicaSyntax.indexOf( '[' ) );
	
	if ( tOperator != null )
	{
	    tOperator = tOperator.trim();
	    
	    switch( getMathIndex( tOperator ) )
	    {
	        case 0  : return decodeMatrixForm( tArguments );
		   case 1  : return decodeList (tArguments );
      	        
	    }
	}
	
	throw new CodecDecodeException( 
            "Unable to decode: " + fMathematicaSyntax );}
    
	/**
     * Gets the index for the Mathematiyca operator. <p>
     *
     * @since 1.0
     */
    
    private int getMathIndex( String fOperator )
    {
	// System.out.println( "arith1Codec.getMathIndex" );
	
	for( int i=0; i<sNames.length; i++ )
	{
	    if ( fOperator.equals( sMathNames[i] ) )
		return i;
	}
	
	return -1;
    }
}
