
/*
 * $Id$
 *
 * Copyright (c) 1999-2004, RIACA, Technische Universiteit Eindhoven (TU/e).
 * All Rights Reserved.
 *
 * ---------------------------------------------------------------------------
 *
 *  The contents of this file are subject to the RIACA Public License
 *  Version 1.0 (the "License"). You may not use this file except in
 *  compliance with the License. A copy of the License is available at
 *  http://www.riaca.win.tue.nl/
 *
 * ---------------------------------------------------------------------------
 */
package mathematicaCodec;


import nl.tue.win.riaca.openmath.codec.CDCodec;
import nl.tue.win.riaca.openmath.codec.Codec;
import nl.tue.win.riaca.openmath.codec.CodecDecodeException;
import nl.tue.win.riaca.openmath.codec.CodecEncodeException;
import nl.tue.win.riaca.openmath.lang.OMApplication;
import nl.tue.win.riaca.openmath.lang.OMObject;
import nl.tue.win.riaca.openmath.lang.OMSymbol;

/**
 * A CD-codec that handles the OpenMath 'linalg3' CD. <p>
 *
 * @author Manfred N. Riem (mriem@win.tue.nl)
 * @version $Revision$
 */
public class linalg4Codec extends CDCodec {
    /**
     * Stores the names in this codec. <p>
     */
    public String[] sNames = {
        "characterstic_eqn",
        "columncount",
        "eigenvalue",
		"eigenvector",
		"rank",
		"rowcount",
		"size"
    };
    /**
     * Stores the Mathematica equivalents to the OpenMath symbols. <p>
     *
     * @since 1.0
     */

    public String[] sMathNames =
    {
    		"Eigenvectors"
    		
    };
    /**
     * Constructor. <p>
     */
    public linalg4Codec() {
        super();
    }
    
    /**
     * Constructor. <p>
     */
    public linalg4Codec( Codec fParent ) {
        super();
        mParent = fParent;
    }
    
    /**
     * Encodes the given OMObject to Mathematica-syntax. <p>
     */
    public String encodeOMObject( OMObject fOMObject )
    throws  CodecEncodeException {
        
        if ( fOMObject instanceof OMApplication ) {
            OMApplication tOMApplication = (OMApplication) fOMObject;
            
            if ( tOMApplication.getElementAt( 0 ) instanceof OMSymbol ) {
                OMSymbol tOMSymbol = (OMSymbol) tOMApplication.getElementAt( 0 );
                
                switch( getIndex( tOMSymbol.getName() ) ) {
                    case 0 :
                        return encodeCharacterstic_Eqn( fOMObject );
                //    case 1 :
                  //      return encodeColumnCount( fOMObject );
                    case 2 :
                        return encodeEigenValue( fOMObject );
                    case 3 :
                        return encodeEigenVector( fOMObject );
                    case 4 :
                        return encodeRank( fOMObject );
                    case 5 :
                        return encodeRowcount( fOMObject );
                    case 6 :
                        return encodeSize( fOMObject );
                        
                }
            }
        }
        
        throw new CodecEncodeException(
        "Unable to encode: " + fOMObject );
    }
    
    /**
     * Decodes the given Mathematica-syntax to a OMObject. <p>
     */
    public OMObject decodeOMObject( String fMathematicaSyntax )
    throws  CodecDecodeException {
        
        throw new CodecDecodeException(
        "Unable to decode: " + fMathematicaSyntax );
    }
    
    /**
     * Gets the index for the name. <p>
     *
     * @since 1.0
     */
    private int getIndex( String fName ) {
        for( int i=0; i<sNames.length; i++ ) {
            if ( fName.equals( sNames[i] ) )
                return i;
        }
        return -1;
    }
    
    /**
     * Encodes the linalg4.characterstic_eqn <p>
     */
    public String encodeCharacterstic_Eqn( OMObject fOMObject )
    throws CodecEncodeException {
        
        StringBuffer  result      = new StringBuffer();
        OMApplication application = (OMApplication) fOMObject;
        
        result.append( "CharactersticPolynomial[" );
        
        if (application.getLength() > 1) {
            result.append(
                mParent.encodeOMObject(application.getElementAt(1)));
        }
        else
            throw new CodecEncodeException("Unable to encode: " + fOMObject);
        
        result.append(",x]");
        return result.toString();
    }
    
    /**
     * Encodes the linalg4.size <p>
     */
    public String encodeSize( OMObject fOMObject )
    throws CodecEncodeException {
        
        StringBuffer  result      = new StringBuffer();
        OMApplication application = (OMApplication) fOMObject;
        
        result.append( "Dimensions[" );
        
        if (application.getLength() > 1) {
            result.append(
                mParent.encodeOMObject(application.getElementAt(1)));
        }
        else
            throw new CodecEncodeException("Unable to encode: " + fOMObject);
        
        result.append("]");
        return result.toString();
    }
    
    
    /**
     * Encodes the linalg4.rank <p>
     */
    public String encodeRank( OMObject fOMObject )
    throws CodecEncodeException {
        
        StringBuffer  result      = new StringBuffer();
        OMApplication application = (OMApplication) fOMObject;
        
        result.append( "MatrixRank[" );
        
        if (application.getLength() > 1) {
            result.append(
                mParent.encodeOMObject(application.getElementAt(1)));
        }
        else
            throw new CodecEncodeException("Unable to encode: " + fOMObject);
        
        result.append("]");
        return result.toString();
    }
    
    
    /**
     * Encodes the linalg4.rowcount <p>
     */
    public String encodeRowcount( OMObject fOMObject )
    throws CodecEncodeException {
        
        StringBuffer  result      = new StringBuffer();
        OMApplication application = (OMApplication) fOMObject;
        
        result.append( "Length[" );
        
        if (application.getLength() > 1) {
            result.append(
                mParent.encodeOMObject(application.getElementAt(1)));
        }
        else
            throw new CodecEncodeException("Unable to encode: " + fOMObject);
        
        result.append("]");
        return result.toString();
    }
    
    /**
     * Encodes the lingalg4.eigenvector
     */
    public String encodeEigenVector( OMObject fOMObject )
        throws  CodecEncodeException {
        
        StringBuffer  result      = new StringBuffer();
        OMApplication tApplication = (OMApplication) fOMObject;
        
        result.append( "Eigenvectors[" );
        
        if ( tApplication.getElementAt( 1 ) != null )
    	{
        	result.append( 
    		mParent.encodeOMObject( tApplication.getElementAt( 1 ) ) );
    	} 
    	else
    	    throw new CodecEncodeException( "Unable to encode: " + fOMObject );
    	
        result.append( "]" );
    	return result.toString();
        }
    
    
    /**
     * Encodes the lingalg4.eigenvalue
     */
    public String encodeEigenValue( OMObject fOMObject )
        throws  CodecEncodeException {
        
        StringBuffer  result      = new StringBuffer();
        OMApplication tApplication = (OMApplication) fOMObject;
        
        result.append( "Eigenvalue[" );
        
        if ( tApplication.getLength() > 2 ) {
            result.append(
            mParent.encodeOMObject( tApplication.getElementAt( 1 ) ) );
        }
        else
            throw new CodecEncodeException( "Unable to encode: " + fOMObject );
        result.append( "]" );
        return result.toString();
    }
    
    
    
}
