

/*
 * $Id$
 *
 * Copyright (c) 1999-2004, RIACA, Technische Universiteit Eindhoven (TU/e).
 * All Rights Reserved.
 *
 * ---------------------------------------------------------------------------
 *
 *  The contents of this file are subject to the RIACA Public License
 *  Version 1.0 (the "License"). You may not use this file except in
 *  compliance with the License. A copy of the License is available at
 *  http://www.riaca.win.tue.nl/
 *
 * ---------------------------------------------------------------------------
 */
package mathematicaCodec;

import java.util.Enumeration;
import java.util.Vector;

import nl.tue.win.riaca.openmath.codec.CDCodec;
import nl.tue.win.riaca.openmath.codec.Codec;
import nl.tue.win.riaca.openmath.codec.CodecDecodeException;
import nl.tue.win.riaca.openmath.codec.CodecEncodeException;
import nl.tue.win.riaca.openmath.lang.OMApplication;
import nl.tue.win.riaca.openmath.lang.OMObject;
import nl.tue.win.riaca.openmath.lang.OMSymbol;



/**
 * A CD-codec that handles the OpenMath 'linalgeig1' CD. <p>
 *
 * @author Manfred N. Riem (mriem@win.tue.nl)
 * @version $Revision$
 */
public class linalgeig1Codec extends CDCodec {
	  /**
     * Stores the names in this codec. <p>
     */
	 public String[] sNames = {
        "eigenspace",
        "eigenvalues",
        "is_eigenvalue",
        "is_eigenvector"
    };
	 /**
	     * Stores the Mathematica equivalents to the OpenMath symbols. <p>
	     *
	     * @since 1.0
	     */

	    public String[] sMathNames =
	    {
	    		"Eigenvalues"
	    		
	    };
	 
    /**
     * Constructor. <p>
     */
    public linalgeig1Codec() {
      super();
        
    }
    
    /**
     * Constructor. <p>
     */
    public linalgeig1Codec( Codec fParent ) {
        this();
        mParent = fParent;
    }
    
    /**
     * Encodes the given OMObject to Mathematica-syntax. <p>
     */
    public String encodeOMObject( OMObject fOMObject )
    throws  CodecEncodeException {
        
        if ( fOMObject instanceof OMApplication ) {
            OMApplication tOMApplication = (OMApplication) fOMObject;
            
            if ( tOMApplication.getElementAt( 0 ) instanceof OMSymbol ) {
                OMSymbol tOMSymbol = (OMSymbol) tOMApplication.getElementAt( 0 );
                                
                                switch( getIndex( tOMSymbol.getName() ) ) {
                                    //case 0 :
                                   //     return encodeEigenSpace( fOMObject );
                                    case 1 :
                                        return encodeEigenValue( fOMObject );
                                    case 2 :
                                        return encodeIsEigenValue( fOMObject );
                                    case 3 :
                                        return encodeIsEigenVector( fOMObject );
                                        
                                }
                            }
                        }
                        
                        throw new CodecEncodeException(
                        "Unable to encode: " + fOMObject );
                    }

                    /**
                     * Decodes the given Mathematica-syntax to a OMObject. <p>
                     */
                    public OMObject decodeOMObject( String fMathematicaSyntax )
                    throws  CodecDecodeException {
                        
                    	String tOperator = 
                    		    fMathematicaSyntax.substring( 
                    	                0, fMathematicaSyntax.indexOf( '[' ) );
                    		
                    		String tArguments =
                    		    fMathematicaSyntax.substring( fMathematicaSyntax.indexOf( '[' ) );
                    		
                    		if ( tOperator != null )
                    		{
                    		    tOperator = tOperator.trim();
                    		    
                    		    switch( getMathIndex( tOperator ) )
                    		    {
                    		        case 0  : return decodeEigenValues( tArguments );
                    			  
                    		        
                    		    }
                    		}
                    		
                    		throw new CodecDecodeException( 
                    	            "Unable to decode: " + fMathematicaSyntax );}
                    
                    /**
                     * Encodes the linalgeig1.eigenvalues. <p>
                     */
                    public String encodeEigenValue(  OMObject fOMObject )
                            throws  CodecEncodeException {
                    
                    	StringBuffer  tResult      = new StringBuffer();
                    	OMApplication tApplication = (OMApplication) fOMObject;
                   /* System.out.print("\n");
                    System.out.print(tApplication.getLength());
                	System.out.print("\n");
                	System.out.print(tApplication.getElementAt(0));
                	System.out.print("\n");
                    System.out.print(tApplication.getElementAt(1));
                    System.out.print("\n");*/
                	
                    
                    tResult.append( "Eigenvalues[" );
                    
                    if ( tApplication.getElementAt( 1 ) != null )
                	{
                    	    tResult.append( 
                		mParent.encodeOMObject( tApplication.getElementAt( 1 ) ) );
                	} 
                	else
                	    throw new CodecEncodeException( "Unable to encode: " + fOMObject );
                	
                        tResult.append( "]" );
                	return tResult.toString();
                    }
                    
                    /**
                     * Encodes the linalgeig1.is_eigenvalue. <p>
                     */
                    public String encodeIsEigenValue( OMObject object )
                    throws CodecEncodeException {
                    
                    StringBuffer  result      = new StringBuffer();
                    OMApplication application = (OMApplication) object;
                    
                    result.append( "MemberQ[Eigenvalues[" );
                    
                    if (application.getLength() > 1) {
                        result.append(
                            mParent.encodeOMObject(application.getElementAt(1)));
                    }
                    else
                        throw new CodecEncodeException("Unable to encode: " + object);
                    
                    if (application.getLength() > 2) {
                        result.append(
                            mParent.encodeOMObject(application.getElementAt(2)));
                    }
                    else
                        throw new CodecEncodeException("Unable to encode: " + object);
                    
                    result.append("]]");
                    return result.toString();
                    }
                    
                    /**
                     * Encodes the linalgeig1.is_eigenvector. <p>
                     */
                    public String encodeIsEigenVector( OMObject object )
                    throws CodecEncodeException {
                    
                    StringBuffer  result      = new StringBuffer();
                    OMApplication application = (OMApplication) object;
                    
                    result.append( "MemberQ[Eigenvectors[" );
                    
                    if (application.getLength() > 1) {
                        result.append(
                            mParent.encodeOMObject(application.getElementAt(1)));
                    }
                    else
                        throw new CodecEncodeException("Unable to encode: " + object);
                    
                    if (application.getLength() > 2) {
                        result.append(
                            mParent.encodeOMObject(application.getElementAt(2)));
                    }
                    else
                        throw new CodecEncodeException("Unable to encode: " + object);
                    
                    result.append("]]");
                    return result.toString();
                    }
                    /**
                     * Gets the index for the name. <p>
                     *
                     * @since 1.0
                     */
                    private int getIndex( String fName ) {
                        for( int i=0; i<sNames.length; i++ ) {
                            if ( fName.equals( sNames[i] ) )
                                return i;
                        }
                        return -1;
                    }
                	/**
                     * Gets the index for the Mathematiyca operator. <p>
                     *
                     * @since 1.0
                     */
                    
                    private int getMathIndex( String fOperator )
                    {
                	// System.out.println( "arith1Codec.getMathIndex" );
                	
                	for( int i=0; i<sNames.length; i++ )
                	{
                	    if ( fOperator.equals( sMathNames[i] ) )
                		return i;
                	}
                	
                	return -1;
                    }
                    
                    
                    /**
                     * Decodes the Mathematica (EigenValues) expression. <p>
                     *
                     * @since 1.0
                     */

                    public OMObject decodeEigenValues( String fMathematicaSyntax )
                        throws  CodecDecodeException
                        
                    {
                	// System.out.println( "linalg2Codec.decodeDet" );
                    	
                    	
                	OMApplication tOMApplication = new OMApplication();
                       
                	tOMApplication.addElement( new OMSymbol( "linalgeig1", "eigenvalues" ) );
                	
                	MathematicaCodec tParentCodec = (MathematicaCodec) mParent;
                	Vector tVector = tParentCodec.parseArguments( fMathematicaSyntax );
                	Enumeration tEnum = tVector.elements();
                	for( ; tEnum.hasMoreElements(); )
                	{
                	    OMObject tOMObject = 
                		tParentCodec.decodeOMObject( 
                                    (String) tEnum.nextElement() );
                 
                	    if ( tOMObject != null )
                		tOMApplication.addElement( tOMObject );
                	    else
                		throw new CodecDecodeException( "Unable to decode: " + 
                						fMathematicaSyntax );
                	}
                   
                	return tOMApplication;
                    }
}
