/*
 * $Id: logic1Codec.java,v 1.4 2002/11/20 23:51:51 mriem Exp $
 *
 * Copyright (c) 1999-2002, RIACA, Technische Universiteit Eindhoven (TU/e).
 * All Rights Reserved.
 *
 * The contents of this file are subject to the RIACA Public License
 * Version 1.0 (the "License"); you may not use this file except in
 * compliance with the License. A copy of the License is available at
 * http://www.riaca.win.tue.nl/
 */


package mathematicaCodec;

import nl.tue.win.riaca.openmath.codec.*;
import nl.tue.win.riaca.openmath.lang.*;

import java.util.*;


/**
 * A CD-codec that handles the OpenMath 'logic1' CD. <p>
 *
 * @author Manfred Riem (mriem@win.tue.nl)
 * @version 1.0
 */

public class logic1Codec
    extends CDCodec
{
    /**
     * Stores the names in this codec. <p>
     *
     * @since 1.0
     */
   
    public String[] sNames = 
    {
	"and",
	"equivalent",
	"false",
	"implies",
	"not",
	"or",
	"true",
	"xor"
    };
    
    
    /**
     * Constructor. <p>
     *
     * @since 1.0
     */
    
    public logic1Codec()
    {
	super();
	
	// System.out.println( "logic1Codec.<init>" );
    }
    
    
    /**
     * Constructor. <p>
     *
     * @since 1.0
     */
   
    public logic1Codec( Codec fParent )
    {
	super();
	
	mParent = fParent;
	
	// System.out.println( "logic1Codec.<init>" );
    }
    
    
    /**
     * Encodes the given OMObject to Mathematica-syntax. <p>
     *
     * @since 1.0
     */
    
    public String encodeOMObject( OMObject fOMObject )
        throws  CodecEncodeException
    {
	// System.out.println( "logic1Codec.encode" );
	
	if ( fOMObject instanceof OMSymbol )
	{
	    OMSymbol tOMSymbol = (OMSymbol) fOMObject;
	    
	    switch( getIndex( tOMSymbol.getName() ) )
	    {
                case 2 :
                    return encodeFalse( fOMObject );
                case 6 :
                    return encodeTrue( fOMObject );
	    }
	}
        
        if ( fOMObject instanceof OMApplication )
        {
            OMApplication tOMApplication = (OMApplication) fOMObject;
	    
            if ( tOMApplication.getElementAt( 0 ) instanceof OMSymbol )
            {
                OMSymbol tOMSymbol = (OMSymbol) tOMApplication.getElementAt( 0 );
		
                switch( getIndex( tOMSymbol.getName() ) )
                {
                    case 0 :
                        return encodeAnd( fOMObject );
                    case 1 :
                        return encodeEquivalent( fOMObject );
                    case 3 :
                        return encodeImplies( fOMObject );
                    case 4:
                        return encodeNot( fOMObject );
                    case 5 :
                        return encodeOr( fOMObject );
                    case 7 :
                        return encodeXor( fOMObject );
                }
	    }
        }
	
        throw new CodecEncodeException( 
	    "Unable to encode: " + fOMObject );
    }


    /**
     * Decodes the given Mathematica-syntax to a OMObject. <p>
     *
     * @since 1.0
     */
  
    public OMObject decodeOMObject( String fMathematicaSyntax )
        throws  CodecDecodeException
    {
	// System.out.println( "logic1Codec.decode" );
       
        throw new CodecDecodeException( 
            "Unable to decode: " + fMathematicaSyntax );
    }
  
  
    /**
     * Gets the index for the name. <p>
     *
     * @since 1.0
     */
   
    private int getIndex( String fName )
    {
	// System.out.println( "logic1Codec.getIndex" );
	
	for( int i=0; i<sNames.length; i++ )
	{
	    if ( fName.equals( sNames[i] ) )
		return i;
	}
    
	return -1;
    }
 

    /**
     * Encodes the logic1.and. <p>
     *
     * @since 1.0
     */
   
    public String encodeAnd( OMObject fOMObject )
    		 throws CodecEncodeException
    		    {
	// System.out.println( "logic1Codec.encodeAnd" );
    
    	
    		// System.out.println( "arith1Codec.encodePlus" );
    		
    		StringBuffer  tResult      = new StringBuffer();
    		OMApplication tApplication = (OMApplication) fOMObject;
    		Enumeration   tEnum        = tApplication.getElements().elements();
    		
    		tEnum.nextElement();
    		
    		if ( tApplication.getLength() >= 3 )
    		{
    		    tResult.append( "And[" );
    		    
    		    for( ; tEnum.hasMoreElements(); )
    		    {
    			tResult.append( 
    			    mParent.encodeOMObject( (OMObject) tEnum.nextElement() ) );
    			
    			if ( tEnum.hasMoreElements() )
    			    tResult.append( "," );
    		    }
    		    
    		    tResult.append( "]" );
    		    
    		    return tResult.toString();
    		}

            throw new CodecEncodeException( "Unable to encode: " + fOMObject );
        }
        
    	    


    /**
     * Encodes the logic1.equivalent. <p>
     *
     * @since 1.0
     */
   
    public String encodeEquivalent( OMObject fOMObject )
        throws  CodecEncodeException
    {
	// System.out.println( "logic1Codec.encodeEquivalent" );
     
	StringBuffer tResult = new StringBuffer();
	OMApplication tApplication = (OMApplication) fOMObject;
        
	tResult.append( "Equal[" );
	
        tResult.append( 
            mParent.encodeOMObject( tApplication.getElementAt( 1 ) ) );
            
        tResult.append( "," );

        tResult.append( 
            mParent.encodeOMObject( tApplication.getElementAt( 2 ) ) );

        tResult.append( "]" );
	return tResult.toString();    
    }


    /**
     * Encodes the logic1.false. <p>
     *
     * @since 1.0
     */
   
    public String encodeFalse( OMObject fOMObject )
        throws  CodecEncodeException
    {
	// System.out.println( "logic1Codec.encodeFalse" );
     
	return "False";
    }


    /**
     * Encodes the logic1.implies. <p>
     *
     * @since 1.0
     */
   
    public String encodeImplies( OMObject fOMObject )
        throws  CodecEncodeException
    {
	// System.out.println( "logic1Codec.encodeImplies" );
     
	StringBuffer tResult = new StringBuffer();
	OMApplication tApplication = (OMApplication) fOMObject;
        
	tResult.append( "Implies[" );
	
        tResult.append( 
            mParent.encodeOMObject( tApplication.getElementAt( 1 ) ) );
            
        tResult.append( "," );

        tResult.append( 
            mParent.encodeOMObject( tApplication.getElementAt( 2 ) ) );

        tResult.append( "]" );
	return tResult.toString();    
    }


    /**
     * Encodes the logic1.not. <p>
     *
     * @since 1.0
     */
   
    public String encodeNot( OMObject fOMObject )
        throws  CodecEncodeException
    {
	// System.out.println( "logic1Codec.encodeNot" );
     
	StringBuffer tResult = new StringBuffer();
	OMApplication tApplication = (OMApplication) fOMObject;
        
	tResult.append( "Not[" );
	
        tResult.append( 
            mParent.encodeOMObject( tApplication.getElementAt( 1 ) ) );
            
        tResult.append( "]" );
	return tResult.toString();    
    }


    /**
     * Encodes the logic1.or. <p>
     *
     * @since 1.0
     */
   
    public String encodeOr( OMObject fOMObject )
        throws  CodecEncodeException
    {
	// System.out.println( "logic1Codec.encodeOr" );
     
	StringBuffer tResult = new StringBuffer();
	OMApplication tApplication = (OMApplication) fOMObject;
        int tIndex = 1;
        
	tResult.append( "Or[" );
	
        while( tApplication.getElementAt( tIndex ) != null )
        {
            tResult.append( 
                mParent.encodeOMObject( tApplication.getElementAt( tIndex ) ) );
            
            tIndex++;
            
            if ( tApplication.getElementAt( tIndex ) != null )
                tResult.append( "," );
	}

	tResult.append( "]" );
	return tResult.toString();    
    }


    /**
     * Encodes the logic1.true. <p>
     *
     * @since 1.0
     */
   
    public String encodeTrue( OMObject fOMObject )
        throws  CodecEncodeException
    {
	// System.out.println( "logic1Codec.encodeTrue" );
     
	return "True";
    }


    /**
     * Encodes the logic1.xor. <p>
     *
     * @since 1.0
     */
   
    public String encodeXor( OMObject fOMObject )
        throws  CodecEncodeException
    {
	// System.out.println( "logic1Codec.encodeXor" );
     
	StringBuffer tResult = new StringBuffer();
	OMApplication tApplication = (OMApplication) fOMObject;
        int tIndex = 1;
        
	tResult.append( "Xor[" );
	
        while( tApplication.getElementAt( tIndex ) != null )
        {
            tResult.append( 
                mParent.encodeOMObject( tApplication.getElementAt( tIndex ) ) );
            
            tIndex++;
            
            if ( tApplication.getElementAt( tIndex ) != null )
                tResult.append( "," );
	}

	tResult.append( "]" );
	return tResult.toString();    
    }
}
