/*
 * $Id: minmax1Codec.java,v 1.4 2002/11/20 23:51:51 mriem Exp $
 *
 * Copyright (c) 1999-2002, RIACA, Technische Universiteit Eindhoven (TU/e).
 * All Rights Reserved.
 *
 * The contents of this file are subject to the RIACA Public License
 * Version 1.0 (the "License"); you may not use this file except in
 * compliance with the License. A copy of the License is available at
 * http://www.riaca.win.tue.nl/
 */


package mathematicaCodec;

import nl.tue.win.riaca.openmath.codec.*;
import nl.tue.win.riaca.openmath.lang.*;
import java.util.*;


/**
 * A CD-codec that handles the OpenMath 'minmax1' CD. <p>
 *
 * @author Manfred Riem (mriem@win.tue.nl)
 * @version 1.0
 */

public class minmax1Codec
    extends CDCodec
{
    /**
     * Stores the names in this codec. <p>
     *
     * @since 1.0
     */
   
    public String[] sNames = 
    {
    	"max",
	"min"
    };


    /**
     * Constructor. <p>
     *
     * @since 1.0
     */

    public minmax1Codec()
    {
    	super();
        
    	// System.out.println( "minmax1Codec.<init>" );
    }

 
    /**
     * Constructor. <p>
     *
     * @since 1.0
     */
   
    public minmax1Codec( Codec fParent )
    {
    	super();
	    
	mParent = fParent;
	    
    	// System.out.println( "minmax1Codec.<init>" );
    }
  
  
    /**
     * Encodes the given OMObject to Mathematica-syntax. <p>
     *
     * @since 1.0
     */
   
    public String encodeOMObject( OMObject fOMObject )
        throws  CodecEncodeException
    {
	// System.out.println( "minmax1Codec.encode" );
	
        if ( fOMObject instanceof OMApplication )
        {
            OMApplication tOMApplication = (OMApplication) fOMObject;
            
            if ( tOMApplication.getElementAt( 0 ) instanceof OMSymbol )
            {
                OMSymbol tOMSymbol = (OMSymbol) tOMApplication.getElementAt( 0 );
	    
                switch( getIndex( tOMSymbol.getName() ) )
                {
                    case 0 : return encodeMax( fOMObject );
                    case 1 : return encodeMin( fOMObject );
                }
	    }
	}
	
        throw new CodecEncodeException( 
            "Unable to encode: " + fOMObject.toString() );
    }


    /**
     * Decodes the given Mathematica-syntax to a OMObject. <p>
     *
     * @since 1.0
     */
  
    public OMObject decodeOMObject( String fMathematicaSyntax )
        throws  CodecDecodeException
    {
    	// System.out.println( "minmax1Codec.decode" );
       
        throw new CodecDecodeException( 
            "Unable to decode: " + fMathematicaSyntax );
    }
  
  
    /**
     * Gets the index for the name. <p>
     *
     * @since 1.0
     */
   
    private int getIndex( String fName )
    {
    	// System.out.println( "minmax1Codec.getIndex" );
	    
    	for( int i=0; i<sNames.length; i++ )
	{
	    if ( fName.equals( sNames[i] ) )
    	    	return i;
	}
	
    	return -1;
    }
 

    /**
     * Encodes the minmax1.max. <p>
     *
     * @since 1.0
     */
   
    public String encodeMax( OMObject fOMObject )
        throws  CodecEncodeException
    {
    	// System.out.println( "minmax1Codec.encodeMax" );
    
	StringBuffer tResult = new StringBuffer();
    	OMApplication tApplication = (OMApplication) fOMObject;
        int tIndex = 1;
        
	tResult.append( "Max[" );
	
        while( tApplication.getElementAt( tIndex ) != null )
        {
            tResult.append( 
                mParent.encodeOMObject( tApplication.getElementAt( tIndex ) ) );
            
            tIndex++;
            
            if ( tApplication.getElementAt( tIndex ) != null )
                tResult.append( "," );
    	}

	tResult.append( "]" );
    	return tResult.toString();    
    }
    
    
    /**
     * Encodes the minmax1.min. <p>
     *
     * @since 1.0
     */
    
    public String encodeMin( OMObject fOMObject )
        throws  CodecEncodeException
    {
    	// System.out.println( "minmax1Codec.encodeMin" );
	
	StringBuffer tResult = new StringBuffer();
	OMApplication tApplication = (OMApplication) fOMObject;
        int tIndex = 1;
        
    	tResult.append( "Min[" );
	
        while( tApplication.getElementAt( tIndex ) != null )
        {
            tResult.append( 
                mParent.encodeOMObject( tApplication.getElementAt( tIndex ) ) );
            
            tIndex++;
            
            if ( tApplication.getElementAt( tIndex ) != null )
                tResult.append( "," );
	}
        
    	tResult.append( "]" );
	return tResult.toString();    
    }
}

