/*
 * $Id: quant1Codec.java,v 1.4 2002/11/20 23:51:51 mriem Exp $
 *
 * Copyright (c) 1999-2002, RIACA, Technische Universiteit Eindhoven (TU/e).
 * All Rights Reserved.
 *
 * The contents of this file are subject to the RIACA Public License
 * Version 1.0 (the "License"); you may not use this file except in
 * compliance with the License. A copy of the License is available at
 * http://www.riaca.win.tue.nl/
 */


package mathematicaCodec;

import nl.tue.win.riaca.openmath.codec.*;
import nl.tue.win.riaca.openmath.lang.*;
import java.util.*;


/**
 * A CD-codec that handles the OpenMath 'quant1' CD. <p>
 *
 * @author Manfred Riem (mriem@win.tue.nl)
 * @version 1.0
 */

public class quant1Codec
    extends CDCodec
{
    /**
     * Stores the names in this codec. <p>
     *
     * @since 1.0
     */
   
    public String[] sNames = 
    {
    	"exists",
	"forall"
    };


    /**
     * Stores the Mathematica names in this codec. <p>
     *
     * @since 1.0
     */
   
    public String[] sMathNames = 
    {
        "Exists",
        "ForAll"
    };
    
    
    /**
     * Constructor. <p>
     *
     * @since 1.0
     */

    public quant1Codec()
    {
    	super();
	
    	// System.out.println( "quant1Codec.<init>" );
    }

 
    /**
     * Constructor. <p>
     *
     * @since 1.0
     */
   
    public quant1Codec( Codec fParent )
    {
	super();
	
    	mParent = fParent;
	
	// System.out.println( "quant1Codec.<init>" );
    }
  
  
    /**
     * Encodes the given OMObject to Mathematica-syntax. <p>
     *
     * @since 1.0
     */
   
    public String encodeOMObject( OMObject fOMObject )
        throws  CodecEncodeException
    {
        // System.out.println( "quant1Codec.encodeOMObject" );
	
        if ( fOMObject instanceof OMApplication )
        {
            OMApplication tApplication = (OMApplication) fOMObject;
            OMObject      tOMObject    = tApplication.getElementAt( 0 );
	
            if ( tOMObject instanceof OMSymbol )
            {
                OMSymbol tOMSymbol = (OMSymbol) tOMObject;
	    
                switch( getIndex( tOMSymbol.getName() ) )
                {
                    case 0 : return encodeExists( fOMObject );
                    case 1 : return encodeForAll( fOMObject );
                }
            }
    	}
    
	throw new CodecEncodeException( "Unable to encode: " + fOMObject.toString() );
    }


    /**
     * Decodes the given Mathematica-syntax to a OMObject. <p>
     *
     * @since 1.0
     */
  
    public OMObject decodeOMObject( String fMathematicaSyntax )
        throws  CodecDecodeException
    {
    	// System.out.println( "quant1Codec.decode" );
       
        try
        {
            String tOperator = 
                fMathematicaSyntax.substring( 
                    0, fMathematicaSyntax.indexOf( '[' ) );

            String tArguments =
                fMathematicaSyntax.substring( 
                    fMathematicaSyntax.indexOf( '[' ) );

            if ( tOperator != null )
            {
                tOperator = tOperator.trim();

                switch( getMathIndex( tOperator ) )
                {
                    case 0 :
                        return decodeExists( tArguments );
                    case 1 :
                        return decodeForAll( tArguments );
                }
            }
    	}
        catch( Exception tException ) {}
    
	throw new CodecDecodeException( "Unable to decode: " + fMathematicaSyntax );
    }
  
  
    /**
     * Gets the index for the name. <p>
     *
     * @since 1.0
     */
   
    private int getIndex( String fName )
    {
    	// System.out.println( "quant1Codec.getIndex" );
	
	for( int i=0; i<sNames.length; i++ )
    	{
	    if ( fName.equals( sNames[i] ) )
		return i;
    	}
	
	return -1;
    }


    /**
     * Gets the index for the Mathematica operator. <p>
     *
     * @since 1.0
     */
   
    private int getMathIndex( String fOperator )
    {
    	// System.out.println( "quant1Codec.getMathIndex" );
	
	for( int i=0; i<sNames.length; i++ )
    	{
	    if ( fOperator.equals( sMathNames[i] ) )
		return i;
    	}
	
	return -1;
    }
  

    /**
     * Encodes the quant1.exists. <p>
     *
     * @since 1.0
     */
   
    public String encodeExists( OMObject fOMObject )
        throws  CodecEncodeException
    {
    	// System.out.println( "quant1Codec.encodeExists" );
	
	StringBuffer  tResult      = new StringBuffer();
    	OMApplication tApplication = (OMApplication) fOMObject;
	
	tResult.append( "Exists[" );
	
    	if ( tApplication.getElementAt( 1 ) != null )
	{
	    tResult.append(
	        mParent.encodeOMObject( tApplication.getElementAt( 1 ) ) );
    	}
	else
            throw new CodecEncodeException( 
                "Unable to encode: " + fOMObject.toString()  );
	
    	tResult.append( "," );
	
        if ( tApplication.getElementAt( 2 ) != null )
        {
	        tResult.append(
    		 mParent.encodeOMObject( tApplication.getElementAt( 2 ) ) );
	    }
    	else
            throw new CodecEncodeException( 
                "Unable to encode: " + fOMObject.toString()  );
	
    	tResult.append( "]" );
	return tResult.toString();
    }


    /**
     * Encodes the quant1.forall. <p>
     *
     * @since 1.0
     */
   
    public String encodeForAll( OMObject fOMObject )
        throws  CodecEncodeException
    {
    	// System.out.println( "quant1Codec.encodeForAll" );
	
	StringBuffer  tResult      = new StringBuffer();
    	OMApplication tApplication = (OMApplication) fOMObject;
	
	tResult.append( "ForAll[" );
	
    	if ( tApplication.getElementAt( 1 ) != null )
	{
	    tResult.append(
	        mParent.encodeOMObject( tApplication.getElementAt( 1 ) ) );
	}
    	else
            throw new CodecEncodeException( 
                "Unable to encode: " + fOMObject.toString()  );
	
    	tResult.append( "," );
	
        if ( tApplication.getElementAt( 2 ) != null )
	{
	    tResult.append(
	        mParent.encodeOMObject( tApplication.getElementAt( 2 ) ) );
    	}
	else
            throw new CodecEncodeException( 
                "Unable to encode: " + fOMObject.toString()  );
	
    	tResult.append( "]" );
	return tResult.toString();
    }
    
    
    /**
     * Decodes 'Exists'. <p>
     *
     * @since 1.0
     */
    
    public OMObject decodeExists( String fMathematicaSyntax )
        throws  CodecDecodeException
    {
    	// System.out.println( "quant1Codec.decodeExists" );
	
	OMApplication tOMApplication = new OMApplication();
    	tOMApplication.addElement( new OMSymbol( "quant1", "exists" ) );
	
	MathematicaCodec tParentCodec = (MathematicaCodec) mParent;
    	Vector tVector = tParentCodec.parseArguments( fMathematicaSyntax );
	
	if ( tVector.size() == 2 )
    	{
	    OMObject tOMObject = 
		tParentCodec.decodeOMObject( 
                    (String) tVector.elementAt( 0 ) );
	    
	    tOMApplication.addElement( tOMObject );
	    
    	    tOMObject = tParentCodec.decodeOMObject(
	        (String) tVector.elementAt( 1 ) );		
	    
    	    tOMApplication.addElement( tOMObject );
	}
    	else
            throw new CodecDecodeException( 
                "Unable to decode: " + fMathematicaSyntax  );
	
    	return tOMApplication;
    }
    
    
    /**
     * Decodes 'ForAll'. <p>
     *
     * @since 1.0
     */

    public OMObject decodeForAll( String fMathematicaSyntax )
        throws  CodecDecodeException
    {
    	// System.out.println( "quant1Codec.decodeForAll" );

    	OMApplication tOMApplication = new OMApplication();
	tOMApplication.addElement( new OMSymbol( "quant1", "forall" ) );
	
    	MathematicaCodec tParentCodec = (MathematicaCodec) mParent;
	Vector tVector = tParentCodec.parseArguments( fMathematicaSyntax );
	
    	if ( tVector.size() == 2 )
	{
	    OMObject tOMObject = 
		tParentCodec.decodeOMObject( 
                    (String) tVector.elementAt( 0 ) );
	    
	    tOMApplication.addElement( tOMObject );
	    
    	    tOMObject = tParentCodec.decodeOMObject(
	        (String) tVector.elementAt( 1 ) );		
	    
	    tOMApplication.addElement( tOMObject );
    	}
	else
            throw new CodecDecodeException( 
                "Unable to decode: " + fMathematicaSyntax  );
   
    	return tOMApplication;
    }
}

