/*
 * $Id: set1Codec.java,v 1.3 2002/08/16 08:32:32 mriem Exp $
 *
 * Copyright (c) 1999-2002, RIACA, Technische Universiteit Eindhoven (TU/e).
 * All Rights Reserved.
 *
 * The contents of this file are subject to the RIACA Public License
 * Version 1.0 (the "License"); you may not use this file except in
 * compliance with the License. A copy of the License is available at
 * http://www.riaca.win.tue.nl/
 */


package mathematicaCodec;

import nl.tue.win.riaca.openmath.codec.*;
import nl.tue.win.riaca.openmath.lang.*;
import java.util.*;


/**
 * A CD-codec that handles the OpenMath 'set1'-cd. <p>
 *
 * @author Manfred Riem (mriem@win.tue.nl)
 * @version 1.0
 */

public class set1Codec
    extends CDCodec
{
    /**
     * Stores the names in this codec. <p>
     *
     * @since 1.0
     */
   
    public String[] sNames = 
    {
    	"cartesian_product",
    	"emptyset",
    	"in",
    	"intersect",
    	"map",
    	"notin",
    	"notprsubset",
    	"notsubset",
    	"prsubset",
    	"set",
    	"setdiff",
    	"size",
    	"subset",
    	"suchthat",
    	"union"
    };


    /**
     * Constructor. <p>
     *
     * @since 1.0
     */

    public set1Codec()
    {
    	super();
        
    	// System.out.println( "set1Codec.<init>" );
    }

 
    /**
     * Constructor. <p>
     *
     * @since 1.0
     */
   
    public set1Codec( Codec fParent )
    {
    	super();
	    
    	mParent = fParent;
	
    	// System.out.println( "set1Codec.<init>" );
    }
  
  
    /**
     * Encodes the given OMObject to Mathematica-syntax. <p>
     *
     * @since 1.0
     */
   
    public String encodeOMObject( OMObject fOMObject )
        throws  CodecEncodeException
    {
    	// System.out.println( "set1Codec.encode" );
	
        throw new CodecEncodeException( "Unable to encode: " + fOMObject.toString() );
    }


    /**
     * Decodes the given Mathematica-syntax to a OMObject. <p>
     *
     * @since 1.0
     */
  
    public OMObject decodeOMObject( String fMathematicaSyntax )
        throws  CodecDecodeException
    {
    	// System.out.println( "set1Codec.decode" );
       
        throw new CodecDecodeException( "Unable to decode: " + fMathematicaSyntax );
    }
  
  
    /**
     * Gets the index for the name. <p>
     *
     * @since 1.0
     */
   
    private int getIndex( String fName )
    {
    	// System.out.println( "set1Codec.getIndex" );
	
    	for( int i=0; i<sNames.length; i++ )
	{
	    if ( fName.equals( sNames[i] ) )
		return i;
    	}
	
	return -1;
    }
}

