/*
 * $Id: transc1Codec.java,v 1.5 2002/12/06 12:45:34 mriem Exp $
 *
 * Copyright (c) 1999-2002, RIACA, Technische Universiteit Eindhoven (TU/e).
 * All Rights Reserved.
 *
 * The contents of this file are subject to the RIACA Public License
 * Version 1.0 (the "License"); you may not use this file except in
 * compliance with the License. A copy of the License is available at
 * http://www.riaca.win.tue.nl/
 */


package mathematicaCodec;

import nl.tue.win.riaca.openmath.codec.*;
import nl.tue.win.riaca.openmath.lang.*;
import java.util.*;


/**
 * A CD-codec that handles the OpenMath 'transc1' CD. <p>
 *
 * @author Manfred Riem (mriem@win.tue.nl)
 * @version 1.3
 */

public class transc1Codec
    extends MathematicaCDCodec
{
    /**
     * Stores the names in this codec. <p>
     *
     * @since 1.0
     */
   
    public String[] sNames = 
    {
    	"arccos",
    	"arccosh",
    	"arccot",
    	"arccoth",
    	"arccsc",
    	"arccsch",
    	"arcsec",
    	"arcsech",
    	"arcsin",
    	"arcsinh",
    	"arctan",
    	"arctanh",
	"cos",
    	"cosh",
    	"cot",
    	"coth",
    	"csc",
    	"csch",
    	"exp",
    	"ln",
    	"log",
    	"sec",
    	"sech",
    	"sin",
    	"sinh",
    	"tan",
    	"tanh"
    };
    
    
    /**
     * Stores the Mathematica equivalents to the OpenMath symbols. <p>
     *
     * @since 1.0
     */

    public String[] sMathNames =
    {
    	"ArcCos",
    	"ArcCosh",
	"ArcCot",
    	"ArcCoth",
    	"ArcCsc",
    	"ArcCsch",
    	"ArcSec",
    	"ArcSech",
    	"ArcSin",
    	"ArcSinh",
    	"ArcTan",
    	"ArcTanh",
        "Cos",
        "Cosh",
        "Cot",
        "Coth",
        "Csc",
        "Csch",
        "Exp",
        "Log",
        "Sec",
        "Sech",
        "Sin",
        "Sinh",
        "Tan",
        "Tanh"
    };

    
    /**
     * Constructor. <p>
     *
     * @since 1.0
     */

    public transc1Codec()
    {
    	super();
        
	// System.out.println( "transc1Codec.<init>" );
    }

 
    /**
     * Constructor. <p>
     *
     * @since 1.0
     */
   
    public transc1Codec( Codec fParent )
    {
    	super();
	
	mParent = fParent;
	
    	// System.out.println( "transc1Codec.<init>" );
    }
  
  
    /**
     * Encodes the given OMObject to Mathematica-syntax. <p>
     *
     * @since 1.0
     */
   
    public String encodeOMObject( OMObject fOMObject )
        throws  CodecEncodeException
    {
	// System.out.println( "transc1Codec.encode" );
	    
    	OMApplication tApplication = (OMApplication) fOMObject;
	OMObject      tOMObject    = tApplication.getElementAt( 0 );
	
    	if ( tOMObject instanceof OMSymbol )
	{
	    OMSymbol tOMSymbol = (OMSymbol) tOMObject;
	    
    	    switch( getIndex( tOMSymbol.getName() ) )
	    {
	    case 0  : return encodeUnary( fOMObject, "ArcCos[",  "]" );
	    case 1  : return encodeUnary( fOMObject, "ArcCosh[", "]" );
	    case 2  : return encodeUnary( fOMObject, "ArcCot[",  "]" );
	    case 3  : return encodeUnary( fOMObject, "ArcCoth[", "]" );
	    case 4  : return encodeUnary( fOMObject, "ArcCsc[",  "]" );
	    case 5  : return encodeUnary( fOMObject, "ArcCsch[", "]" );
	    case 6  : return encodeUnary( fOMObject, "ArcSec[",  "]" );
	    case 7  : return encodeUnary( fOMObject, "ArcSech[", "]" );
	    case 8  : return encodeUnary( fOMObject, "ArcSin[",  "]" );
	    case 9  : return encodeUnary( fOMObject, "ArcSinh[", "]" );
	    case 10 : return encodeUnary( fOMObject, "ArcTan[",  "]" );
	    case 11 : return encodeUnary( fOMObject, "ArcTanh[", "]" );
	    case 12 : return encodeUnary( fOMObject, "Cos[",     "]" );
	    case 13 : return encodeUnary( fOMObject, "Cosh[",    "]" );
	    case 14 : return encodeUnary( fOMObject, "Cot[",     "]" );
	    case 15 : return encodeUnary( fOMObject, "Coth[",    "]" );
	    case 16 : return encodeUnary( fOMObject, "Csc[",     "]" );
	    case 17 : return encodeUnary( fOMObject, "Csch[",    "]" );
	    case 18 : return encodeUnary( fOMObject, "Exp[",     "]" );
	    case 19 : return encodeUnary( fOMObject, "Log[",     "]" );
	    case 20 : return encodeUnary( fOMObject, "Log[",     "]" );
	    case 21 : return encodeUnary( fOMObject, "Sec[",     "]" );
	    case 22 : return encodeUnary( fOMObject, "Sech[",    "]" );
	    case 23 : return encodeUnary( fOMObject, "Sin[",     "]" );
	    case 24 : return encodeUnary( fOMObject, "Sinh[",    "]" );
	    case 25 : return encodeUnary( fOMObject, "Tan[",     "]" );
	    case 26 : return encodeUnary( fOMObject, "Tanh[",    "]" );
            }
	}
	
        throw new CodecEncodeException( "Unable to encode: " + fOMObject );
    }
    
    
    /**
     * Decodes the given Mathematica-syntax to a OMObject. <p>
     *
     * @since 1.0
     */
  
    public OMObject decodeOMObject( String fMathematicaSyntax )
        throws  CodecDecodeException
    {
    	// System.out.println( "transc1Codec.decode" );
       
	String tOperator = 
	    fMathematicaSyntax.substring( 
                0, fMathematicaSyntax.indexOf( '[' ) );

    	String tArguments =
	    fMathematicaSyntax.substring( fMathematicaSyntax.indexOf( '[' ) );

    	if ( tOperator != null )
	{
	    tOperator = tOperator.trim();
	    
    	    switch( getMathIndex( tOperator ) )
	    {
		case 0  : return decodeArcCos( tArguments );
		case 1  : return decodeArcCosh( tArguments );
      	        case 2  : return decodeArcCot( tArguments );
    	        case 3  : return decodeArcCoth( tArguments );
    	        case 4  : return decodeArcCsc( tArguments );
    	        case 5  : return decodeArcCsch( tArguments );
    	        case 6  : return decodeArcSec( tArguments );
    	        case 7  : return decodeArcSech( tArguments );
    	        case 8  : return decodeArcSin( tArguments );
    	        case 9  : return decodeArcSinh( tArguments );
    	        case 10 : return decodeArcTan( tArguments );
    	        case 11 : return decodeArcTanh( tArguments );
                case 12 : return decodeCos( tArguments );
                case 13 : return decodeCosh( tArguments );
                case 14 : return decodeCot( tArguments );
                case 15 : return decodeCoth( tArguments );
                case 16 : return decodeCsc( tArguments );
                case 17 : return decodeCsch( tArguments );
                case 18 : return decodeExp( tArguments );
                case 19 : return decodeLog( tArguments );
                case 20 : return decodeSec( tArguments );
                case 21 : return decodeSech( tArguments );
                case 22 : return decodeSin( tArguments );
                case 23 : return decodeSinh( tArguments );
                case 24 : return decodeTan( tArguments );
                case 25 : return decodeTanh( tArguments );
                default : break;
    	    }
	}
	
        throw new CodecDecodeException( "Unable to decode: " + fMathematicaSyntax );
    }
    
    
    /**
     * Gets the index for the name. <p>
     *
     * @since 1.0
     */
   
    private int getIndex( String fName )
    {
	// System.out.println( "transc1Codec.getIndex" );
	
	for( int i=0; i<sNames.length; i++ )
	{
	    if ( fName.equals( sNames[i] ) )
		return i;
	}
    
	return -1;
    }
 

    /**
     * Gets the index for the Mathematica operator. <p>
     *
     * @since 1.0
     */
   
    private int getMathIndex( String fOperator )
    {
	// System.out.println( "transc1Codec.getMathIndex" );
	
	for( int i=0; i<sNames.length; i++ )
	{
	    if ( fOperator.equals( sMathNames[i] ) )
		return i;
	}
    
	return -1;
    }
    
    
    /**
     * Encodes the transc1.sec. <p>
     *
     * @since 1.0
     */
    
    public String encodeSec( OMObject fOMObject )
        throws  CodecEncodeException
    {
	// System.out.println( "transc1Codec.encodeSec" );
	
	StringBuffer  tResult      = new StringBuffer();
	OMApplication tApplication = (OMApplication) fOMObject;
	
	tResult.append( "Sec[" );
	
	if ( tApplication.getElementAt( 1 ) != null )
	{
    	    tResult.append( 
		mParent.encodeOMObject( tApplication.getElementAt( 1 ) ) );
	} 
	else
            throw new CodecEncodeException( "Unable to encode: " + fOMObject );
	
        tResult.append( "]" );
	return tResult.toString();
    }
    
    
    /**
     * Encodes the transc1.sech. <p>
     *
     * @since 1.0
     */
    
    public String encodeSech( OMObject fOMObject )
        throws  CodecEncodeException
    {
	// System.out.println( "transc1Codec.encodeSech" );
	
	StringBuffer  tResult      = new StringBuffer();
	OMApplication tApplication = (OMApplication) fOMObject;
	
	tResult.append( "Sech[" );
	
	if ( tApplication.getElementAt( 1 ) != null )
	{
    	    tResult.append( 
		mParent.encodeOMObject( tApplication.getElementAt( 1 ) ) );
	} 
	else
	    throw new CodecEncodeException( "Unable to encode: " + fOMObject );
	
        tResult.append( "]" );
	return tResult.toString();
    }

    
    /**
     * Encodes the transc1.sin. <p>
     *
     * @since 1.0
     */
   
    public String encodeSin( OMObject fOMObject )
        throws  CodecEncodeException
    {
	// System.out.println( "transc1Codec.encodeSin" );
	
	StringBuffer  tResult      = new StringBuffer();
	OMApplication tApplication = (OMApplication) fOMObject;
	
	tResult.append( "Sin[" );
	
	if ( tApplication.getElementAt( 1 ) != null )
	{
    	    tResult.append( 
		mParent.encodeOMObject( tApplication.getElementAt( 1 ) ) );
	} 
	else
	    throw new CodecEncodeException( "Unable to encode: " + fOMObject );
	
        tResult.append( " Degree]" );
	return tResult.toString();
    }


    /**
     * Encodes the transc1.sinh. <p>
     *
     * @since 1.0
     */
   
    public String encodeSinh( OMObject fOMObject )
        throws  CodecEncodeException
    {
	// System.out.println( "transc1Codec.encodeSinh" );
     
	StringBuffer  tResult      = new StringBuffer();
	OMApplication tApplication = (OMApplication) fOMObject;
    
	tResult.append( "Sinh[" );

	if ( tApplication.getElementAt( 1 ) != null )
	{
    	    tResult.append( 
		mParent.encodeOMObject( tApplication.getElementAt( 1 ) ) );
	} 
	else
	    throw new CodecEncodeException( "Unable to encode: " + fOMObject );
   
        tResult.append( "]" );
	return tResult.toString();
    }


    /**
     * Encodes the transc1.tan. <p>
     *
     * @since 1.0
     */
   
    public String encodeTan( OMObject fOMObject )
        throws  CodecEncodeException
    {
	// System.out.println( "transc1Codec.encodeTan" );
     
	StringBuffer  tResult      = new StringBuffer();
	OMApplication tApplication = (OMApplication) fOMObject;
    
	tResult.append( "Tan[" );

	if ( tApplication.getElementAt( 1 ) != null )
	{
    	    tResult.append( 
		mParent.encodeOMObject( tApplication.getElementAt( 1 ) ) );
	} 
	else
	    throw new CodecEncodeException( "Unable to encode: " + fOMObject );
   
        tResult.append( " Degree]" );
	return tResult.toString();
    }

    
    /**
     * Encodes the transc1.tanh. <p>
     *
     * @since 1.0
     */
   
    public String encodeTanh( OMObject fOMObject )
        throws  CodecEncodeException
    {
	// System.out.println( "transc1Codec.encodeTanh" );
     
	StringBuffer  tResult      = new StringBuffer();
	OMApplication tApplication = (OMApplication) fOMObject;
    
	tResult.append( "Tanh[" );

	if ( tApplication.getElementAt( 1 ) != null )
	{
    	    tResult.append( 
		mParent.encodeOMObject( tApplication.getElementAt( 1 ) ) );
	} 
	else
	    throw new CodecEncodeException( "Unable to encode: " + fOMObject );
   
        tResult.append( "]" );
	return tResult.toString();
    }


    /**
     * Decodes the ArcCos Mathematica expression. <p>
     *
     * @since 1.0
     */

    public OMObject decodeArcCos( String fMathematicaSyntax )
        throws  CodecDecodeException
    {
	// System.out.println( "arith1Codec.decodeArcCos" );
	
	OMApplication tOMApplication = new OMApplication();
	
	tOMApplication.addElement( new OMSymbol( "transc1", "arccos" ) );
	
	MathematicaCodec tParentCodec = (MathematicaCodec) mParent;
	Vector tVector = tParentCodec.parseArguments( fMathematicaSyntax );
	
	if ( tVector.size() == 1 )
	{
	    OMObject tOMObject = 
		tParentCodec.decodeOMObject( 
                    (String) tVector.elementAt( 0 ) );
	    
	    tOMApplication.addElement( tOMObject );
	}
	else
	    throw new CodecDecodeException( "Unable to decode: " + fMathematicaSyntax );
	
	return tOMApplication;
    }

    
    /**
     * Decodes the ArcCosh Mathematica expression. <p>
     *
     * @since 1.0
     */

    public OMObject decodeArcCosh( String fMathematicaSyntax )
        throws  CodecDecodeException
    {
	// System.out.println( "arith1Codec.decodeArcCosh" );

	OMApplication tOMApplication = new OMApplication();
       
	tOMApplication.addElement( new OMSymbol( "transc1", "arccosh" ) );
	
	MathematicaCodec tParentCodec = (MathematicaCodec) mParent;
	Vector tVector = tParentCodec.parseArguments( fMathematicaSyntax );

	if ( tVector.size() == 1 )
	{
	    OMObject tOMObject = 
		tParentCodec.decodeOMObject( 
                    (String) tVector.elementAt( 0 ) );
 
	    tOMApplication.addElement( tOMObject );
	}
	else
	    throw new CodecDecodeException( "Unable to decode: " + fMathematicaSyntax );
   
	return tOMApplication;
    }

    
    /**
     * Decodes the ArcCot Mathematica expression. <p>
     *
     * @since 1.0
     */

    public OMObject decodeArcCot( String fMathematicaSyntax )
        throws  CodecDecodeException
    {
	// System.out.println( "arith1Codec.decodeArcCot" );

	OMApplication tOMApplication = new OMApplication();
	tOMApplication.addElement( new OMSymbol( "transc1", "arccot" ) );
	
	MathematicaCodec tParentCodec = (MathematicaCodec) mParent;
	Vector tVector = tParentCodec.parseArguments( fMathematicaSyntax );

	if ( tVector.size() == 1 )
	{
	    OMObject tOMObject = 
		tParentCodec.decodeOMObject( 
                    (String) tVector.elementAt( 0 ) );
 
	    tOMApplication.addElement( tOMObject );
	}
	else
	    throw new CodecDecodeException( "Unable to decode: " + fMathematicaSyntax );
   
	return tOMApplication;
    }

    
    /**
     * Decodes the ArcCoth Mathematica expression. <p>
     *
     * @since 1.0
     */

    public OMObject decodeArcCoth( String fMathematicaSyntax )
        throws  CodecDecodeException
    {
	// System.out.println( "arith1Codec.decodeArcCoth" );

	OMApplication tOMApplication = new OMApplication();
	tOMApplication.addElement( new OMSymbol( "transc1", "arccoth" ) );
	
	MathematicaCodec tParentCodec = (MathematicaCodec) mParent;
	Vector tVector = tParentCodec.parseArguments( fMathematicaSyntax );

	if ( tVector.size() == 1 )
	{
	    OMObject tOMObject = 
		tParentCodec.decodeOMObject( 
                    (String) tVector.elementAt( 0 ) );
 
	    tOMApplication.addElement( tOMObject );
	}
	else
	    throw new CodecDecodeException( "Unable to decode: " + fMathematicaSyntax );
   
	return tOMApplication;
    }


    /**
     * Decodes the ArcCsc Mathematica expression. <p>
     *
     * @since 1.0
     */

    public OMObject decodeArcCsc( String fMathematicaSyntax )
        throws  CodecDecodeException
    {
	// System.out.println( "arith1Codec.decodeArcCsc" );

	OMApplication tOMApplication = new OMApplication();
       
	tOMApplication.addElement( new OMSymbol( "transc1", "arccsc" ) );
	
	MathematicaCodec tParentCodec = (MathematicaCodec) mParent;
	Vector tVector = tParentCodec.parseArguments( fMathematicaSyntax );

	if ( tVector.size() == 1 )
	{
	    OMObject tOMObject = 
		tParentCodec.decodeOMObject( 
                    (String) tVector.elementAt( 0 ) );
 
	    tOMApplication.addElement( tOMObject );
	}
	else
	    throw new CodecDecodeException( "Unable to decode: " + fMathematicaSyntax );
   
	return tOMApplication;
    }


    /**
     * Decodes the ArcCsch Mathematica expression. <p>
     *
     * @since 1.0
     */

    public OMObject decodeArcCsch( String fMathematicaSyntax )
        throws  CodecDecodeException
    {
	// System.out.println( "arith1Codec.decodeArcCsch" );

	OMApplication tOMApplication = new OMApplication();
       
	tOMApplication.addElement( new OMSymbol( "transc1", "arccsch" ) );
	
	MathematicaCodec tParentCodec = (MathematicaCodec) mParent;
	Vector tVector = tParentCodec.parseArguments( fMathematicaSyntax );

	if ( tVector.size() == 1 )
	{
	    OMObject tOMObject = 
		tParentCodec.decodeOMObject( 
                    (String) tVector.elementAt( 0 ) );
 
	    tOMApplication.addElement( tOMObject );
	}
	else
	    throw new CodecDecodeException( "Unable to decode: " + fMathematicaSyntax );
   
	return tOMApplication;
    }

    
    /**
     * Decodes the ArcSec Mathematica expression. <p>
     *
     * @since 1.0
     */

    public OMObject decodeArcSec( String fMathematicaSyntax )
        throws  CodecDecodeException
    {
	// System.out.println( "arith1Codec.decodeArcSec" );

	OMApplication tOMApplication = new OMApplication();
       
	tOMApplication.addElement( new OMSymbol( "transc1", "arcsec" ) );
	
	MathematicaCodec tParentCodec = (MathematicaCodec) mParent;
	Vector tVector = tParentCodec.parseArguments( fMathematicaSyntax );

	if ( tVector.size() == 1 )
	{
	    OMObject tOMObject = 
		tParentCodec.decodeOMObject( 
                    (String) tVector.elementAt( 0 ) );
 
	    tOMApplication.addElement( tOMObject );
	}
	else
	    throw new CodecDecodeException( "Unable to decode: " + fMathematicaSyntax );
   
	return tOMApplication;
    }

    
    /**
     * Decodes the ArcSech Mathematica expression. <p>
     *
     * @since 1.0
     */

    public OMObject decodeArcSech( String fMathematicaSyntax )
        throws  CodecDecodeException
    {
	// System.out.println( "arith1Codec.decodeArcSech" );

	OMApplication tOMApplication = new OMApplication();
       
	tOMApplication.addElement( new OMSymbol( "transc1", "arcsech" ) );
	
	MathematicaCodec tParentCodec = (MathematicaCodec) mParent;
	Vector tVector = tParentCodec.parseArguments( fMathematicaSyntax );

	if ( tVector.size() == 1 )
	{
	    OMObject tOMObject = 
		tParentCodec.decodeOMObject( 
                    (String) tVector.elementAt( 0 ) );
 
	    tOMApplication.addElement( tOMObject );
	}
	else
	    throw new CodecDecodeException( "Unable to decode: " + fMathematicaSyntax );
   
	return tOMApplication;
    }


    /**
     * Decodes the ArcSin Mathematica expression. <p>
     *
     * @since 1.0
     */

    public OMObject decodeArcSin( String fMathematicaSyntax )
        throws  CodecDecodeException
    {
	// System.out.println( "arith1Codec.decodeArcSin" );

	OMApplication tOMApplication = new OMApplication();
       
	tOMApplication.addElement( new OMSymbol( "transc1", "arcsin" ) );
	
	MathematicaCodec tParentCodec = (MathematicaCodec) mParent;
	Vector tVector = tParentCodec.parseArguments( fMathematicaSyntax );

	if ( tVector.size() == 1 )
	{
	    OMObject tOMObject = 
		tParentCodec.decodeOMObject( 
                    (String) tVector.elementAt( 0 ) );
 
	    tOMApplication.addElement( tOMObject );
	}
	else
	    throw new CodecDecodeException( "Unable to decode: " + fMathematicaSyntax );
   
	return tOMApplication;
    }


    /**
     * Decodes the ArcSinh Mathematica expression. <p>
     *
     * @since 1.0
     */

    public OMObject decodeArcSinh( String fMathematicaSyntax )
        throws  CodecDecodeException
    {
	// System.out.println( "arith1Codec.decodeArcSinh" );

	OMApplication tOMApplication = new OMApplication();
       
	tOMApplication.addElement( new OMSymbol( "transc1", "arcsinh" ) );
	
	MathematicaCodec tParentCodec = (MathematicaCodec) mParent;
	Vector tVector = tParentCodec.parseArguments( fMathematicaSyntax );

	if ( tVector.size() == 1 )
	{
	    OMObject tOMObject = 
		tParentCodec.decodeOMObject( 
                    (String) tVector.elementAt( 0 ) );
 
	    tOMApplication.addElement( tOMObject );
	}
	else
	    throw new CodecDecodeException( "Unable to decode: " + fMathematicaSyntax );
   
	return tOMApplication;
    }

    
    /**
     * Decodes the ArcTan Mathematica expression. <p>
     *
     * @since 1.0
     */

    public OMObject decodeArcTan( String fMathematicaSyntax )
        throws  CodecDecodeException
    {
	// System.out.println( "arith1Codec.decodeArcTan" );

	OMApplication tOMApplication = new OMApplication();
       
	tOMApplication.addElement( new OMSymbol( "transc1", "arctan" ) );
	
	MathematicaCodec tParentCodec = (MathematicaCodec) mParent;
	Vector tVector = tParentCodec.parseArguments( fMathematicaSyntax );

	if ( tVector.size() == 1 )
	{
	    OMObject tOMObject = 
		tParentCodec.decodeOMObject( 
                    (String) tVector.elementAt( 0 ) );
 
	    tOMApplication.addElement( tOMObject );
	}
	else
	    throw new CodecDecodeException( "Unable to decode: " + fMathematicaSyntax );
   
	return tOMApplication;
    }

    
    /**
     * Decodes the ArcTanh Mathematica expression. <p>
     *
     * @since 1.0
     */

    public OMObject decodeArcTanh( String fMathematicaSyntax )
        throws  CodecDecodeException
    {
	// System.out.println( "arith1Codec.decodeArcTanh" );

	OMApplication tOMApplication = new OMApplication();
       
	tOMApplication.addElement( new OMSymbol( "transc1", "arctanh" ) );
	
	MathematicaCodec tParentCodec = (MathematicaCodec) mParent;
	Vector tVector = tParentCodec.parseArguments( fMathematicaSyntax );

	if ( tVector.size() == 1 )
	{
	    OMObject tOMObject = 
		tParentCodec.decodeOMObject( 
                    (String) tVector.elementAt( 0 ) );
 
	    tOMApplication.addElement( tOMObject );
	}
	else
	    throw new CodecDecodeException( "Unable to decode: " + fMathematicaSyntax );
	
	return tOMApplication;
    }
    
    
    /**
     * Decodes the Cos Mathematica expression. <p>
     *
     * @since 1.0
     */

    public OMObject decodeCos( String fMathematicaSyntax )
        throws  CodecDecodeException
    {
	// System.out.println( "arith1Codec.decodeCos" );

	OMApplication tOMApplication = new OMApplication();
       
	tOMApplication.addElement( new OMSymbol( "transc1", "cos" ) );
	
	MathematicaCodec tParentCodec = (MathematicaCodec) mParent;
	Vector tVector = tParentCodec.parseArguments( fMathematicaSyntax );

	if ( tVector.size() == 1 )
	{
	    OMObject tOMObject = 
		tParentCodec.decodeOMObject( 
                    (String) tVector.elementAt( 0 ) );
 
	    tOMApplication.addElement( tOMObject );
	}
	else
	    throw new CodecDecodeException( "Unable to decode: " + fMathematicaSyntax );
	
	return tOMApplication;
    }

    
    /**
     * Decodes the Cosh Mathematica expression. <p>
     *
     * @since 1.0
     */

    public OMObject decodeCosh( String fMathematicaSyntax )
        throws  CodecDecodeException
    {
	// System.out.println( "arith1Codec.decodeCosh" );

	OMApplication tOMApplication = new OMApplication();
       
	tOMApplication.addElement( new OMSymbol( "transc1", "cosh" ) );
	
	MathematicaCodec tParentCodec = (MathematicaCodec) mParent;
	Vector tVector = tParentCodec.parseArguments( fMathematicaSyntax );

	if ( tVector.size() == 1 )
	{
	    OMObject tOMObject = 
		tParentCodec.decodeOMObject( 
                    (String) tVector.elementAt( 0 ) );
 
	    tOMApplication.addElement( tOMObject );
	}
	else
	    throw new CodecDecodeException( "Unable to decode: " + fMathematicaSyntax );
	
	return tOMApplication;
    }


    /**
     * Decodes the Cot Mathematica expression. <p>
     *
     * @since 1.0
     */

    public OMObject decodeCot( String fMathematicaSyntax )
        throws  CodecDecodeException
    {
	// System.out.println( "arith1Codec.decodeCot" );

	OMApplication tOMApplication = new OMApplication();
       
	tOMApplication.addElement( new OMSymbol( "transc1", "cot" ) );
	
	MathematicaCodec tParentCodec = (MathematicaCodec) mParent;
	Vector tVector = tParentCodec.parseArguments( fMathematicaSyntax );

	if ( tVector.size() == 1 )
	{
	    OMObject tOMObject = 
		tParentCodec.decodeOMObject( 
                    (String) tVector.elementAt( 0 ) );
 
	    tOMApplication.addElement( tOMObject );
	}
	else
	    throw new CodecDecodeException( "Unable to decode: " + fMathematicaSyntax );
	
	return tOMApplication;
    }

    
    /**
     * Decodes the Coth Mathematica expression. <p>
     *
     * @since 1.0
     */

    public OMObject decodeCoth( String fMathematicaSyntax )
        throws  CodecDecodeException
    {
	// System.out.println( "arith1Codec.decodeCoth" );

	OMApplication tOMApplication = new OMApplication();
       
	tOMApplication.addElement( new OMSymbol( "transc1", "arccoth" ) );
	
	MathematicaCodec tParentCodec = (MathematicaCodec) mParent;
	Vector tVector = tParentCodec.parseArguments( fMathematicaSyntax );

	if ( tVector.size() == 1 )
	{
	    OMObject tOMObject = 
		tParentCodec.decodeOMObject( 
                    (String) tVector.elementAt( 0 ) );
 
	    tOMApplication.addElement( tOMObject );
	}
	else
	    throw new CodecDecodeException( "Unable to decode: " + fMathematicaSyntax );
   
	return tOMApplication;
    }

    
    /**
     * Decodes the Csc Mathematica expression. <p>
     *
     * @since 1.0
     */

    public OMObject decodeCsc( String fMathematicaSyntax )
        throws  CodecDecodeException
    {
	// System.out.println( "arith1Codec.decodeCsc" );

	OMApplication tOMApplication = new OMApplication();
       
	tOMApplication.addElement( new OMSymbol( "transc1", "csc" ) );
	
	MathematicaCodec tParentCodec = (MathematicaCodec) mParent;
	Vector tVector = tParentCodec.parseArguments( fMathematicaSyntax );

	if ( tVector.size() == 1 )
	{
	    OMObject tOMObject = 
		tParentCodec.decodeOMObject( 
                    (String) tVector.elementAt( 0 ) );
 
	    tOMApplication.addElement( tOMObject );
	}
	else
	    throw new CodecDecodeException( "Unable to decode: " + fMathematicaSyntax );
   
	return tOMApplication;
    }

    
    /**
     * Decodes the Csch Mathematica expression. <p>
     *
     * @since 1.0
     */

    public OMObject decodeCsch( String fMathematicaSyntax )
        throws  CodecDecodeException
    {
	// System.out.println( "arith1Codec.decodeCsch" );

	OMApplication tOMApplication = new OMApplication();
       
	tOMApplication.addElement( new OMSymbol( "transc1", "csch" ) );
	
	MathematicaCodec tParentCodec = (MathematicaCodec) mParent;
	Vector tVector = tParentCodec.parseArguments( fMathematicaSyntax );

	if ( tVector.size() == 1 )
	{
	    OMObject tOMObject = 
		tParentCodec.decodeOMObject( 
                    (String) tVector.elementAt( 0 ) );
 
	    tOMApplication.addElement( tOMObject );
	}
	else
	    throw new CodecDecodeException( "Unable to decode: " + fMathematicaSyntax );
   
	return tOMApplication;
    }

      
    /**
     * Decodes the Exp Mathematica expression. <p>
     *
     * @since 1.0
     */

    public OMObject decodeExp( String fMathematicaSyntax )
        throws  CodecDecodeException
    {
	// System.out.println( "arith1Codec.decodeExp" );

	OMApplication tOMApplication = new OMApplication();
       
	tOMApplication.addElement( new OMSymbol( "transc1", "exp" ) );
	
	MathematicaCodec tParentCodec = (MathematicaCodec) mParent;
	Vector tVector = tParentCodec.parseArguments( fMathematicaSyntax );

	if ( tVector.size() == 1 )
	{
	    OMObject tOMObject = 
		tParentCodec.decodeOMObject( 
                    (String) tVector.elementAt( 0 ) );
 
	    tOMApplication.addElement( tOMObject );
	}
	else
	    throw new CodecDecodeException( "Unable to decode: " + fMathematicaSyntax );
   
	return tOMApplication;
    }

    
    /**
     * Decodes the Log Mathematica expression. <p>
     *
     * @since 1.0
     */

    public OMObject decodeLog( String fMathematicaSyntax )
        throws  CodecDecodeException
    {
	// System.out.println( "arith1Codec.decodeLog" );

	OMApplication tOMApplication = null;
        
	MathematicaCodec tParentCodec = (MathematicaCodec) mParent;
	Vector tVector = tParentCodec.parseArguments( fMathematicaSyntax );

	if ( tVector.size() == 1 )
	{
            tOMApplication = new OMApplication();
            tOMApplication.addElement( new OMSymbol( "transc1", "ln" ) );

            OMObject tOMObject = 
		tParentCodec.decodeOMObject( 
                    (String) tVector.elementAt( 0 ) );
 
	    tOMApplication.addElement( tOMObject );
	}
        
        if ( tVector.size() == 2 )
        {
            tOMApplication = new OMApplication();
            tOMApplication.addElement( new OMSymbol( "transc1", "log" ) );

            OMObject tOMObject = 
		tParentCodec.decodeOMObject( 
                    (String) tVector.elementAt( 0 ) );
 
	    tOMApplication.addElement( tOMObject );

	    tOMObject = 
		tParentCodec.decodeOMObject( 
                    (String) tVector.elementAt( 1 ) );
 
	    tOMApplication.addElement( tOMObject );
        }
        
        if ( tVector.size() != 1 &&
             tVector.size() != 2 )
	    throw new CodecDecodeException( "Unable to decode: " + fMathematicaSyntax );

	return tOMApplication;
    }

    
    /**
     * Decodes the Sec Mathematica expression. <p>
     *
     * @since 1.0
     */

    public OMObject decodeSec( String fMathematicaSyntax )
        throws  CodecDecodeException
    {
	// System.out.println( "arith1Codec.decodeSec" );

	OMApplication tOMApplication = new OMApplication();
       
	tOMApplication.addElement( new OMSymbol( "transc1", "sec" ) );
	
	MathematicaCodec tParentCodec = (MathematicaCodec) mParent;
	Vector tVector = tParentCodec.parseArguments( fMathematicaSyntax );

	if ( tVector.size() == 1 )
	{
	    OMObject tOMObject = 
		tParentCodec.decodeOMObject( 
                    (String) tVector.elementAt( 0 ) );
 
	    tOMApplication.addElement( tOMObject );
	}
	else
	    throw new CodecDecodeException( "Unable to decode: " + fMathematicaSyntax );
   
	return tOMApplication;
    }

    
    /**
     * Decodes the Sech Mathematica expression. <p>
     *
     * @since 1.0
     */

    public OMObject decodeSech( String fMathematicaSyntax )
        throws  CodecDecodeException
    {
	// System.out.println( "arith1Codec.decodeSech" );

	OMApplication tOMApplication = new OMApplication();
       
	tOMApplication.addElement( new OMSymbol( "transc1", "sech" ) );
	
	MathematicaCodec tParentCodec = (MathematicaCodec) mParent;
	Vector tVector = tParentCodec.parseArguments( fMathematicaSyntax );

	if ( tVector.size() == 1 )
	{
	    OMObject tOMObject = 
		tParentCodec.decodeOMObject( 
                    (String) tVector.elementAt( 0 ) );
 
	    tOMApplication.addElement( tOMObject );
	}
	else
	    throw new CodecDecodeException( "Unable to decode: " + fMathematicaSyntax );
   
	return tOMApplication;
    }


    /**
     * Decodes the Sin Mathematica expression. <p>
     *
     * @since 1.0
     */

    public OMObject decodeSin( String fMathematicaSyntax )
        throws  CodecDecodeException
    {
	// System.out.println( "arith1Codec.decodeSin" );

	OMApplication tOMApplication = new OMApplication();
       
	tOMApplication.addElement( new OMSymbol( "transc1", "sin" ) );
	
	MathematicaCodec tParentCodec = (MathematicaCodec) mParent;
	Vector tVector = tParentCodec.parseArguments( fMathematicaSyntax );

	if ( tVector.size() == 1 )
	{
	    OMObject tOMObject = 
		tParentCodec.decodeOMObject( 
                    (String) tVector.elementAt( 0 ) );
 
	    tOMApplication.addElement( tOMObject );
	}
	else
	    throw new CodecDecodeException( "Unable to decode: " + fMathematicaSyntax );
   
	return tOMApplication;
    }

    
    /**
     * Decodes the Sinh Mathematica expression. <p>
     *
     * @since 1.0
     */

    public OMObject decodeSinh( String fMathematicaSyntax )
        throws  CodecDecodeException
    {
	// System.out.println( "arith1Codec.decodeSinh" );

	OMApplication tOMApplication = new OMApplication();
       
	tOMApplication.addElement( new OMSymbol( "transc1", "sinh" ) );
	
	MathematicaCodec tParentCodec = (MathematicaCodec) mParent;
	Vector tVector = tParentCodec.parseArguments( fMathematicaSyntax );

	if ( tVector.size() == 1 )
	{
	    OMObject tOMObject = 
		tParentCodec.decodeOMObject( 
                    (String) tVector.elementAt( 0 ) );
 
	    tOMApplication.addElement( tOMObject );
	}
	else
	    throw new CodecDecodeException( "Unable to decode: " + fMathematicaSyntax );
   
	return tOMApplication;
    }


    /**
     * Decodes the Tan Mathematica expression. <p>
     *
     * @since 1.0
     */

    public OMObject decodeTan( String fMathematicaSyntax )
        throws  CodecDecodeException
    {
	// System.out.println( "arith1Codec.decodeTan" );

	OMApplication tOMApplication = new OMApplication();
       
	tOMApplication.addElement( new OMSymbol( "transc1", "tan" ) );
	
	MathematicaCodec tParentCodec = (MathematicaCodec) mParent;
	Vector tVector = tParentCodec.parseArguments( fMathematicaSyntax );

	if ( tVector.size() == 1 )
	{
	    OMObject tOMObject = 
		tParentCodec.decodeOMObject( 
                    (String) tVector.elementAt( 0 ) );
 
	    tOMApplication.addElement( tOMObject );
	}
	else
	    throw new CodecDecodeException( "Unable to decode: " + fMathematicaSyntax );
   
	return tOMApplication;
    }

    /**
     * Decodes the Tanh Mathematica expression. <p>
     *
     * @since 1.0
     */

    public OMObject decodeTanh( String fMathematicaSyntax )
        throws  CodecDecodeException
    {
	// System.out.println( "arith1Codec.decodeTanh" );

	OMApplication tOMApplication = new OMApplication();
       
	tOMApplication.addElement( new OMSymbol( "transc1", "tanh" ) );
	
	MathematicaCodec tParentCodec = (MathematicaCodec) mParent;
	Vector tVector = tParentCodec.parseArguments( fMathematicaSyntax );

	if ( tVector.size() == 1 )
	{
	    OMObject tOMObject = 
		tParentCodec.decodeOMObject( 
                    (String) tVector.elementAt( 0 ) );
 
	    tOMApplication.addElement( tOMObject );
	}
	else
	    throw new CodecDecodeException( "Unable to decode: " + fMathematicaSyntax );
   
	return tOMApplication;
    }


    /**
     * Register the decoders.
     *
     * @since 1.0
     */
    
    public void registerDecoders() {}
    
    
    /**
     * Deregister the decoder.
     *
     * @since 1.0
     */

    public void deregisterDecoders() {}
}
