package mathematicaGUI;

import java.awt.BorderLayout;
import java.awt.GridLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.Vector;

import javax.swing.JButton;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JSplitPane;
import javax.swing.JTextArea;

import mathematicaPhrasebook.MathematicaPhrasebook;
import nl.tue.win.riaca.openmath.phrasebook.PhrasebookException;

/**
 * A panel for Mathematica. <p>
 *
 * @author Manfred Riem (mriem@win.tue.nl)
 * @version 1.0
 */

public class MathematicaPanel
    extends JPanel
{
    /**
	 * 
	 */
	private static final long serialVersionUID = 1L;





	/**
     * Stores the phrasebook. <p>
     *
     * @since 1.0
     */


	protected MathematicaPhrasebook mPhrasebook = new MathematicaPhrasebook();


    
    

    /**
     * Stores the input text-area. <p>
     *
     * @since 1.0
     */

    private JTextArea mInput;


    /**
     * Stores the output text-area. <p>
     *
     * @since 1.0
     */

    private JTextArea mOutput;


  

  




    /**
     * Stores the Mathematica -> OM button. <p>
     *
     * @since 1.0
     */

    private JButton mNativeOM = new JButton( "Math -> OM" );


    /**
     * Stores the OM -> Mathematica button. <p>
     *
     * @since 1.0
     */

    private JButton mOMNative = new JButton( "OM -> Math" );


    /**
     * Stores the Mathematica -> Mathematica button. <p>
     *
     * @since 1.0
     */

    private JButton mNativeNative = new JButton( "Math -> Math" );


    

    /**
     * Constructor. <p>
     *
     * @since 1.0
     */

    public MathematicaPanel()
    {
        super();
	
        System.out.println( "MathematicaPanel.<init>" );
	
        init();
    }


    /**
     * Initializes the components. <p>
     *
     * @since 1.0
     */

    public void init()
    {
        System.out.println( "MathematicaPanel.init" );

        mInput  = new JTextArea();
        mOutput = new JTextArea();

        try
        {
            mInput.setDragEnabled( true );
            mOutput.setDragEnabled( true );
        }
        catch( NoSuchMethodError tError )
        {
            tError.printStackTrace();
        }

        JSplitPane tSplitPane = new JSplitPane(
            JSplitPane.VERTICAL_SPLIT,
            new JScrollPane( mInput ),
            new JScrollPane( mOutput )
        );
        
        tSplitPane.setOneTouchExpandable( true );
        tSplitPane.setDividerLocation( 0.5 );

        JPanel tPanel = new JPanel();
        tPanel.setLayout( new GridLayout( 7, 1, 5, 5 ) );
        tPanel.add( mOMNative );      
        tPanel.add( mNativeNative );
        tPanel.add( mNativeOM );
        
        

        mNativeOM.addActionListener( new ActionListener()
            {
                public void actionPerformed( ActionEvent fEvent )
                {
                    buttonNativeOM();
                }
            }
        );

        mOMNative.addActionListener( new ActionListener()
            {
                public void actionPerformed( ActionEvent fEvent )
                {
                    buttonOMNative();
                }
            }
        );

        mNativeNative.addActionListener( new ActionListener()
            {
                public void actionPerformed( ActionEvent fEvent )
                {
                    buttonNativeNative();
                }
            }
        );

        
 
      
 
        

      

        setLayout( new BorderLayout() );
        add( tSplitPane, BorderLayout.CENTER );
        add( tPanel,     BorderLayout.EAST   );

   

     
    

    


        /*
         * Lets initialize the phrasebook with the extra CD's. Note that
         * we assume we have them available on the local classpath!
         */

      /*  try
        {
            mPhrasebook.addCD( "arith2",    "", 
	        "matematicaCodec.arith2Codec"    );
	    
            mPhrasebook.addCD( "combinat1", "", 
                "matematicaCodec.combinat1Codec" );
	    
            mPhrasebook.addCD( "fns2",      "", 
                "matematicaCodec.fns2Codec"      );
	    
            mPhrasebook.addCD( "list2",     "", 
                "matematicaCodec.list2Codec"     );
	    
            mPhrasebook.addCD( "setname2",  "", 
                "matematicaCodec.setname2Codec"  );
        }
        catch( PhrasebookException tPhrasebookException )
        {
            tPhrasebookException.printStackTrace();
        }*/
    }


   


    /**
     * Handles button Mathematica -> OM. <p>
     *
     * @since 1.0
     */

    protected void buttonNativeOM()
    {
        System.out.println( "MathematicaPanel.buttonNativeOM" );
        
    	String tInput = mInput.getText();
	

        Vector tArgs = new Vector();
        tArgs.addElement( tInput );

        try {
			mOutput.setText( mPhrasebook.perform(  "EVAL_NATIVE_OPENMATH", tArgs ) );
		} catch (PhrasebookException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
    }   


    /**
     * Handles button OM -> Mathematica. <p>
     *
     * @since 1.0
     */

    protected void buttonOMNative()
    {
        System.out.println( "MathematicaPanel.buttonOMNative" );
        
        Vector tArgs = new Vector();
        tArgs.addElement( mInput.getText() );

        try {
			mOutput.setText( mPhrasebook.perform( 
			    "EVAL_OPENMATH_NATIVE", tArgs ) );
		} catch (PhrasebookException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
    }


    protected void buttonNativeNative()
    {
    	
System.out.println( "MathematicaPanel.buttonNativeNative" );
        
        Vector tArgs = new Vector();
        tArgs.addElement( mInput.getText() );

        try {
			mOutput.setText( mPhrasebook.perform( 
			    "EVAL_NATIVE_NATIVE", tArgs ) );
		} catch (PhrasebookException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
        
    	

      

       
    }
}
