

package mathematicaPhrasebook;




import java.util.Vector;

import mathematicaCodec.MathematicaCodec;
import nl.tue.win.riaca.openmath.codec.CodecDecodeException;
import nl.tue.win.riaca.openmath.codec.CodecEncodeException;
import nl.tue.win.riaca.openmath.codec.CodecException;
import nl.tue.win.riaca.openmath.phrasebook.Phrasebook;
import nl.tue.win.riaca.openmath.phrasebook.PhrasebookException;

/**
 * Implements the phrasebook which uses a TCP/IP based server as back-end. <p>
 *
 * @author Manfred Riem (mriem@win.tue.nl)
 * @version 1.3
 */

public class MathematicaPhrasebook
    extends Phrasebook {
    /**
     *  Stores the codec we use.
     *
     *  @since 1.0
     */
    
   protected MathematicaCodec mCodec = null;

   
    
    public MathematicaPhrasebook() {
        super();
        // System.out.println( "MathematicaSocketPhrasebook.<init>" );
   
        mCodec = new MathematicaCodec();
    }
    
    
    
    /**
     * Performs a command and get the result. <p>
     *
     * @since 1.0
     */
    
    public String perform( String fMethod, Vector fArguments )
        throws PhrasebookException {
    	 //System.out.println( "HolLightPhrasebook.perform" );
          
        
        if ( fMethod.equalsIgnoreCase( "EVAL_OPENMATH_NATIVE" ) )
            return performOpenMathToMathemattica( fArguments );
        
        if ( fMethod.equalsIgnoreCase( "EVAL_NATIVE_OPENMATH" ) )
            return performMAthematicaToOpenMath( fArguments );
        
        if ( fMethod.equalsIgnoreCase( "EVAL_NATIVE_NATIVE" ) )
            return performEvalNativeNative( fArguments );
	
        if ( fMethod.equalsIgnoreCase( "ENCODE" ) )
        	return performEncode( fArguments );
	
        if ( fMethod.equalsIgnoreCase( "DECODE" ) )
        	return performDecode( fArguments );
        
        if ( fMethod.equalsIgnoreCase( "ADDCD" ) )
        	return performAddCD( fArguments );
	
        if ( fMethod.equalsIgnoreCase( "REMOVECD" ) )
        	return performRemoveCD( fArguments );
	
    	throw new PhrasebookException( "Unsupported method: " + fMethod );
    }
    
    /**
     * Performs OPENMATH_To_Mathematica. <p>
     *
     * This requires OpenMath syntax as input and it will return Mathematica
     * as its output. <p>
     *
     * @since 1.0
     */
    private String performOpenMathToMathemattica(Vector fArguments) {
    	//System.out.println( 
    			  // "HolLightPhrasebook.performEvalOpenMathNative" );
    			
    		    	Object tElement0 = fArguments.elementAt( 0 );
    		        String tCommand  = null;
    		        String tInteger = (String)tElement0;
    		     
    		        try {
    		            tCommand = mCodec.encode( tInteger.trim() );
    		           tCommand = "FullForm[" + tCommand + "]";               
    		        }
    		        catch( CodecEncodeException tCodecEncodeException ) {
    			    return 
    				"<OMOBJ>" + 
    				" <OMA>"  +
    				"  <OMS cd=\"error\" name=\"encode\"/>" +
    				"  <OMSTR>" + tCodecEncodeException.getMessage() + "</OMSTR>" +
    				" </OMA>" +
    				"</OMOBJ>";
    		        }
    			return tCommand;
    			//return performCall( tCommand ).trim();
	}
   

	/**
     * Performs Mathematica_To_OPENMATH. <p>
     *
     * This requires Mathematica syntax as input and it will return openMath
     * as its output. <p>
     *
     * @since 1.0
     */
    
    private String performMAthematicaToOpenMath(Vector fArguments) {

    	// System.out.println( "HolLightPhrasebook.performEncode" );
    	
        try {
	    Object tElement0 = fArguments.elementAt( 0 );//System.out.println((String) tElement0+"9bal codec" );
	    String str = mCodec.decode((String) tElement0 );
	    //System.out.println(str +"mathematicaphrasebook");
	    return str;
        }
        catch( CodecDecodeException tCodecDecodeException ) {
	    return 
		"<OMOBJ>" +
		" <OMA>"  +
		"  <OMS cd=\"error\" name=\"decode\"/>" +
		" <OMSTR>" + tCodecDecodeException.getMessage() + "</OMSTR>" +
		" </OMA>" +
		"</OMOBJ>";
        }
	}

	/**
     * Adds a CD. <p>
     *
     * @since 1.3
     */
    
    protected String performAddCD( Vector fArguments )
	throws PhrasebookException {
    	// System.out.println( "MathematicaSocketPhrasebook.performAddCD" );
    	
    	String tName      = (String) fArguments.elementAt( 0 );
    	String tLocation  = (String) fArguments.elementAt( 1 );
    	String tClassName = (String) fArguments.elementAt( 2 );
    	
    	addCD( tName.trim(), tLocation.trim(), tClassName.trim() );
    	
    	return "";
    }
    /**
     * Remove a CD. <p>
     *
     * @since 1.3
     */
    
    protected String performRemoveCD( Vector fArguments )
	throws PhrasebookException {
	// System.out.println( "MathematicaSocketPhrasebook.performRemoveCD" );
	
	String tName = (String) fArguments.elementAt( 0 );
	
	removeCD( tName.trim() );
	
	return "";
    }
    
    
    /**
     * Encodes the OpenMath object to MAthematicat Syntax. <p>
     *
     * @since 1.3
     */
    
    protected String performEncode( Vector fArguments )
	throws PhrasebookException {

    	// System.out.println( "MathematicaSocketPhrasebook.performEncode" );
    	
        try {
	    Object tElement0 = fArguments.elementAt( 0 );
	    return mCodec.encode( (String) tElement0 );
        }
        catch( CodecEncodeException tCodecEncodeException ) {
	    return 
		"<OMOBJ>" +
		" <OMA>"  +
		"  <OMS cd=\"error\" name=\"encode\"/>" +
		"  <OMSTR>" + tCodecEncodeException.getMessage() + "</OMSTR>" +
		" </OMA>" +
		"</OMOBJ>";
        }
    }
    
    /**
     * Decodes from Mathematica Syntax to an OpenMath object. <p>
     *
     * @since 1.3
     */
    
    protected String performDecode( Vector fArguments )
	throws PhrasebookException {
	
    	// System.out.println( "MathematicaSocketPhrasebook.performEncode" );
    	
        try {
	    Object tElement0 = fArguments.elementAt( 0 );
	    
	    return mCodec.decode( (String) tElement0 );
        }
        catch( CodecDecodeException tCodecDecodeException ) {
	    return 
		"<OMOBJ>" +
		" <OMA>"  +
		"  <OMS cd=\"error\" name=\"decode\"/>" +
		" <OMSTR>" + tCodecDecodeException.getMessage() + "</OMSTR>" +
		" </OMA>" +
		"</OMOBJ>";
        }
    }
    
    /**
     * Performs an EVAL. <p>
     *
     * @since 1.0
     */
    
    protected String performEval( Vector fArguments )
        throws PhrasebookException {
    	// System.out.println( "MathematicaSocketPhrasebook.performEval" );
	    
    	Object tElement0 = fArguments.elementAt( 0 );
    	String tCommand  = null;
    	try {
                tCommand = mCodec.encode( (String) tElement0 );
                tCommand = "FullForm[" + tCommand + "]";
            }
            catch( CodecEncodeException tCodecEncodeException ) {
    	    return 
    		"<OMOBJ>" + 
    		" <OMA>"  +
    		"  <OMS cd=\"error\" name=\"encode\"/>" +
    		"  <OMSTR>" + tCodecEncodeException.getMessage() + "</OMSTR>" +
    		" </OMA>" +
    		"</OMOBJ>";
            }
    	
            //tResult = performCall( tCommand ).trim();
            
            try {
    	    return mCodec.decode( tCommand.trim() );
            }
            catch( CodecDecodeException tCodecDecodeException ) {
    	    return 
    		"<OMOBJ>" +
    		" <OMA>"  +
    		"  <OMS cd=\"error\" name=\"decode\"/>" +
    		" <OMSTR>" + tCodecDecodeException.getMessage() + "</OMSTR>" +
    		" </OMA>" +
    		"</OMOBJ>";
            }
    }
    
   
    
   
    
    /**
     * Performs EVAL_NATIVE_NATIVE. <p>
     *
     * This requires Mathematica syntax as input and it will return
     * Mathematica syntax as its output. <p>
     * @return 
     *
     * @since 1.0
     */
    
    protected String performEvalNativeNative( Vector fArguments )
        throws PhrasebookException {
    	Object tElement0 = fArguments.elementAt( 0 );
    	System.out.println("Input \n" +(String)tElement0 );
    	return null;
		
	}

    
 /**
  * To dynamically add a CD to the codec.
  *
  * @since 1.0
  */
 
 public void addCD( String fName, String fLocation, String fClassName )
     throws PhrasebookException {
     // System.out.println( "MathematicaSocketPhrasebook.addCD" );
	
     try {
         mCodec.addCD( fName, fLocation, fClassName );
     }
     catch( CodecException tCodecException ) {
         throw new PhrasebookException( tCodecException.getMessage() );
     }
 }
 
 /**
  * To dynamically remove a CD from the codec.
  *
  * @since 1.0
  */
 
 public void removeCD( String fName )
     throws PhrasebookException {
     // System.out.println( "MathematicaSocketPhrasebook.removeCD" );
	
     try {
         mCodec.removeCD( fName );
     }
     catch( CodecException tCodecException ) {
         throw new PhrasebookException( tCodecException.getMessage() );
     }
 }
}

